;*************************************************************************************
;          ______  ______  ______  ______  __   __  ______
;         / __  / / __  / / ____/ / __  / /  | / / / __  /
;        / / / / / /_/ / / /___  / /_/ / /   |/ / / / / /
;       / / / / / ____/ / ____/ /    _/ / /|   / / / / /
;      / /_/ / / /     / /___  / /| |  / / |  / / /_/ /
;     /_____/ /_/     /_____/ /_/ |_| /_/  |_/ /_____/
;      A dynamic And object-oriented programming language
;
;     F U N C T I O N   P L O T T E R
;
;*************************************************************************************
;   (c) 2010 Josef Sniatecki
; 
; This program is free software; you can redistribute it and/or modify it under the
; terms of the GNU General Public License As published by the Free Software Foundation;
; either version 3 of the License, Or (at your option) any later version.
;
; This program is distributed in the hope that it will be useful, but WITHOUT ANY
; WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
; PARTICULAR PURPOSE. See the GNU General Public License For more details.
;
; You should have received a copy of the GNU General Public License along With this
; program; if not, see "http://www.gnu.org/licenses/".
;*************************************************************************************



EnableExplicit

;*******************************************************
; Includes
;*******************************************************

IncludePath "..\..\..\..\Includes"
IncludeFile "Standard.pbi"

IncludeFile "EditorGadget.pbi"
IncludeFile "Cursor.pbi"

  Structure QueueItem_Memory
    StructureUnion
      *Memory
      *Value.OpernoValue
      *Object.OpernoObject
      *Item.QueueItem
    EndStructureUnion
  EndStructure
  #Templates_Queue_Memory = #True
IncludeFile "Queue.pbi"

IncludeFile "Operno.pbi"



;******************************************
; Variables
;******************************************

Global Event.i, Window.i, Gadget.i, Type.i, Menu.i

Global NewList Function.s()
Global *Coord.OpernoCoordinateSystem

Global Example.i, ExampleFunction.s



;******************************************
; Sources
;******************************************

IncludePath ""
IncludeFile "Common.pbi"



;******************************************
; Procedures
;******************************************

Procedure.i DrawFunctions()
  OpernoCoordinateSystemDrawBackground(*Coord)
  ForEach Function()
    OpernoCoordinateSystemGraph(*Coord, Function())
  Next
  SetGadgetState(#Main_Coord, OpernoCoordinateSystemImageID(*Coord))
EndProcedure



;******************************************
; Configurations
;******************************************

;SetCurrentDirectory(GetPathPart(ProgramFilename()))

InitOperno("..\..\DLL\Operno.dll")

OpernoEval("main:import(math)")

*Coord = OpernoCreateCoordinateSystem(300, 300, -5, -5, 5, 5)
OpernoCoordinateSystemDrawBackground(*Coord)



;******************************************
; Main
;******************************************

OpenFunctionPlotterWindow()
SetGadgetState(#Main_Coord, OpernoCoordinateSystemImageID(*Coord))

Repeat
  Event  = WaitWindowEvent()
  Gadget = EventGadget()
  Type   = EventType()
  
  Select Event
    Case #PB_Event_Gadget
      Select Gadget
        Case #Main_Draw
          AddElement(Function())
          Function() = GetGadgetText(#Main_Input)
          OpernoCoordinateSystemGraph(*Coord, Function())
          If OpernoError()
            MessageRequester("Error", OpernoErrorMessage())
          EndIf
          SetGadgetState(#Main_Coord, OpernoCoordinateSystemImageID(*Coord))
        Case #Main_Clear
          OpernoCoordinateSystemDrawBackground(*Coord)
          ClearList(Function())
          SetGadgetState(#Main_Coord, OpernoCoordinateSystemImageID(*Coord))
        Case #Main_Sin
          SetGadgetText(#Main_Input, GetGadgetText(#Main_Input) + "sin(x)")
        Case #Main_Cos
          SetGadgetText(#Main_Input, GetGadgetText(#Main_Input) + "cos(x)")
        Case #Main_Tan
          SetGadgetText(#Main_Input, GetGadgetText(#Main_Input) + "tan(x)")
        Case #Main_Pow
          SetGadgetText(#Main_Input, GetGadgetText(#Main_Input) + "^x")
        Case #Main_Sqrt
          SetGadgetText(#Main_Input, GetGadgetText(#Main_Input) + "^0.5")
        Case #Main_Log
          SetGadgetText(#Main_Input, GetGadgetText(#Main_Input) + "log(x)")
        Case #Main_Fac
          SetGadgetText(#Main_Input, GetGadgetText(#Main_Input) + "fac(x)")
        Case #Main_Sum
          SetGadgetText(#Main_Input, GetGadgetText(#Main_Input) + "(repeat(a, b)[n] (n)):sum")
        Case #Main_Zoom
          Define Zoom.f = 50 / GetGadgetState(#Main_Zoom)
          *Coord\MinX    = -5 * Zoom
          *Coord\MaxX    =  5 * Zoom
          *Coord\MinY    = -5 * Zoom
          *Coord\MaxY    =  5 * Zoom
          *Coord\XFactor = *Coord\Width /(*Coord\MaxX - *Coord\MinX)
          *Coord\YFactor = *Coord\Height/(*Coord\MaxY - *Coord\MinY)
          DrawFunctions()
        Case #Main_DrawExample
          Select Example
            Case 0
              ExampleFunction = "0.5x - 1"
            Case 1
              ExampleFunction = "0.8x^2 + 2x - 2"
            Case 2
              ExampleFunction = "0.5x^3 - 1.5x"
            Case 3
              ExampleFunction = "sin(1.5sin(x))"
              Example = -1
          EndSelect
          Example + 1
          ClearList(Function())
          AddElement(Function())
          Function() = ExampleFunction
          SetGadgetText(#Main_Input, Function())
          DrawFunctions()
        Case #Main_Save
          Define Filename.s = SaveFileRequester("Save", "", "*.bmp", 0)
          If Filename
            If Not GetExtensionPart(Filename)
              Filename + ".bmp"
            EndIf
            OpernoCoordinateSystemSaveImage(*Coord, Filename)
          EndIf
        Case #Main_Console
          DisableWindow(#MainWindow, #True)
          OpernoOpenUI()
          OpernoSetTruncation(0)
          DisableWindow(#MainWindow, #False)
          StickyWindow(#MainWindow, 1)
      EndSelect
    Case #PB_Event_CloseWindow
      Break
  EndSelect
ForEver



CloseWindow(#MainWindow)
End
; IDE Options = PureBasic 4.41 (Windows - x86)
; CursorPosition = 181
; FirstLine = 153
; Folding = -
; EnableXP
; Executable = FunctionPlotter.exe