IncludeFile "Operno.pbi"



Global *DrawingLibrary.OpernoObject

Global Window.i
Global Output.i
Global PreviousPen.i



Procedure.i Std_Drawing_CreatePen()
  Protected Style.i, Color.i, Width.i
  
  Style = #PS_SOLID
  Color = #Black
  Width = 1
  
  OpernoPopIValue(@Style)
  OpernoPopIValue(@Color)
  OpernoPopIValue(@Width)
  
  ProcedureReturn OpernoPushIValue(CreatePen_(Style, Width, Color))
EndProcedure
Procedure.i Std_Drawing_UsePen()
  Protected Pen.i
  
  If OpernoPopIValue(@Pen)
    PreviousPen = SelectObject_(Output, Pen)
  Else
    _OpernoArgumentError(1)
  EndIf
EndProcedure
Procedure.i Std_Drawing_Line()
  Protected StartX.i, StartY.i, EndX.i, EndY.i
  
  If OpernoPopIValue(@StartX) And OpernoPopIValue(@StartY) And OpernoPopIValue(@EndX) And OpernoPopIValue(@EndY)
    MoveToEx_(Output, StartX, StartY, 0)
    LineTo_(Output, EndX, EndY)
  Else
    _OpernoArgumentError(4)
  EndIf
EndProcedure
Procedure.i Std_Drawing_Rectangle()
  Protected Left.i, Top.i, Right.i, Bottom.i
  
  If OpernoPopIValue(@Left) And OpernoPopIValue(@Top) And OpernoPopIValue(@Right) And OpernoPopIValue(@Bottom)
    Rectangle_(Output, Left, Top, Right, Bottom)
  Else
    _OpernoArgumentError(4)
  EndIf
EndProcedure
Procedure.i Std_Drawing_RoundRectangle()
  Protected Left.i, Top.i, Right.i, Bottom.i, Width.i, Height.i
  
  If OpernoPopIValue(@Left) And OpernoPopIValue(@Top) And OpernoPopIValue(@Right) And OpernoPopIValue(@Bottom) And OpernoPopIValue(@Width) And OpernoPopIValue(@Height)
    RoundRect_(Output, Left, Top, Right, Bottom, Width, Height)
  Else
    _OpernoArgumentError(4)
  EndIf
EndProcedure
Procedure.i Std_Drawing_Circle()
  Protected Left.i, Top.i, Right.i, Bottom.i
  
  If OpernoPopIValue(@Left) And OpernoPopIValue(@Top) And OpernoPopIValue(@Right) And OpernoPopIValue(@Bottom)
    Arc_(Output, Left, Top, Right, Bottom, 0, 90, 0, 90)
  Else
    _OpernoArgumentError(4)
  EndIf
EndProcedure
Procedure.i Std_Drawing_BezierCurve()
  Protected Dim ControlPoint.Point(3), Index.i
  
  For Index = 0 To 3
    If Not (OpernoPopIValue(@ControlPoint(Index)\X) And OpernoPopIValue(@ControlPoint(Index)\Y))
      _OpernoArgumentError(6)
    EndIf
  Next
  
  PolyBezier_(Output, @ControlPoint(), 4)
EndProcedure
Procedure.i Std_Drawing_Polygon()
  Protected Count.i = OpernoArgumentCount() / 2, Index.i
  Protected Dim ControlPoint.Point(Count - 1)
  
  For Index = 0 To Count - 1
    OpernoPopIValue(@ControlPoint(Index)\X)
    OpernoPopIValue(@ControlPoint(Index)\Y)
  Next
  
  Polygon_(Output, @ControlPoint(), Count)
EndProcedure
Procedure.i Std_Drawing_Text()
  Protected X.i, Y.i, Size.i, Text.String, Angle.i, FontName.String
  Protected Font.i, PreviousFont.i, Range.RECT
  
  Angle      = 0
  Size       = 12
  FontName\S = "Consolas"
  
  Select OpernoArgumentCount()
    Case 3
      OpernoPopIValue(@X)
      OpernoPopIValue(@Y)
      OpernoPopSValue(@Text)
    Case 4
      OpernoPopIValue(@X)
      OpernoPopIValue(@Y)
      OpernoPopSValue(@Text)
      OpernoPopIValue(@Angle)
    Case 5
      OpernoPopIValue(@X)
      OpernoPopIValue(@Y)
      OpernoPopSValue(@Text)
      OpernoPopIValue(@Angle)
      OpernoPopIValue(@Size)
    Case 6
      OpernoPopIValue(@X)
      OpernoPopIValue(@Y)
      OpernoPopSValue(@Text)
      OpernoPopIValue(@Angle)
      OpernoPopIValue(@Size)
      OpernoPopSValue(@FontName)
  EndSelect
  
  Font = CreateFont_(Size, 0, Angle * 10, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, FontName\S)
  GetWindowRect_(WindowID(Window), @Range)
  PreviousFont = SelectObject_(Output, Font)
  SetTextAlign_(Output, #TA_BASELINE)
  SetBkMode_(Output, #TRANSPARENT)
  ExtTextOut_(Output, X, Y, 0, @Range, @Text\S, Len(Text\S), 0)
  SelectObject_(Output, PreviousFont)
  DeleteObject_(Font)
EndProcedure
Procedure.i Std_Drawing_PreviousPen()
  ProcedureReturn OpernoPushIValue(PreviousPen)
EndProcedure
Procedure.i Std_Drawing_OpenOutput()
  Protected Width.i, Height.i
  
  Width  = 500
  Height = 300
  
  OpernoPopIValue(@Width)
  OpernoPopIValue(@Height)
  
  Window = OpenWindow(#PB_Any, 0, 0, Width, Height, "", #PB_Window_SystemMenu|#PB_Window_ScreenCentered)
  Output = GetDC_(WindowID(Window))
EndProcedure
Procedure.i Std_Drawing_Wait()
  Protected Event.i, EventWindow.i
  
  Repeat
    Event       = WaitWindowEvent()
    EventWindow = EventWindow()
  Until EventWindow = Window And Event = #PB_Event_CloseWindow
  
  Output = 0
  CloseWindow(Window)
EndProcedure
Procedure.i Std_Drawing_DeletePen()
  Protected Pen.i
  
  If OpernoPopIValue(@Pen)
    DeleteObject_(Pen)
  Else
    _OpernoArgumentError(1)
  EndIf
EndProcedure



ProcedureDLL.i InitLibrary()
  *DrawingLibrary = OpernoCreateGroup("drawing", *OpernoStandardLibrary)
  OpernoExtractObject(*DrawingLibrary, ?Std_Drawing)
EndProcedure
ProcedureDLL.i DrawingLibrary()
  ProcedureReturn *DrawingLibrary
EndProcedure



DataSection
  Std_Drawing:
    Data.c #OpernoObject_Function : Data.s "createPen" ;{
      Data.i @Std_Drawing_CreatePen()
    ;}
    Data.c #OpernoObject_Function : Data.s "usePen" ;{
      Data.i @Std_Drawing_UsePen()
    ;}
    Data.c #OpernoObject_Function : Data.s "line" ;{
      Data.i @Std_Drawing_Line()
    ;}
    Data.c #OpernoObject_Function : Data.s "rectangle" ;{
      Data.i @Std_Drawing_Rectangle()
    ;}
    Data.c #OpernoObject_Function : Data.s "roundRectangle" ;{
      Data.i @Std_Drawing_RoundRectangle()
    ;}
    Data.c #OpernoObject_Function : Data.s "circle" ;{
      Data.i @Std_Drawing_Circle()
    ;}
    Data.c #OpernoObject_Function : Data.s "bezierCurve" ;{
      Data.i @Std_Drawing_BezierCurve()
    ;}
    Data.c #OpernoObject_Function : Data.s "polygon" ;{
      Data.i @Std_Drawing_Polygon()
    ;}
    Data.c #OpernoObject_Function : Data.s "text" ;{
      Data.i @Std_Drawing_Text()
    ;}
    Data.c #OpernoObject_Function : Data.s "previousPen" ;{
      Data.i @Std_Drawing_PreviousPen()
    ;}
    Data.c #OpernoObject_Function : Data.s "openOutput" ;{
      Data.i @Std_Drawing_OpenOutput()
    ;}
    Data.c #OpernoObject_Function : Data.s "wait" ;{
      Data.i @Std_Drawing_Wait()
    ;}
    Data.c #OpernoObject_Function : Data.s "deletePen" ;{
      Data.i @Std_Drawing_DeletePen()
    ;}
    Data.c #Null
EndDataSection
; IDE Options = PureBasic 4.40 Beta 5 (Windows - x86)
; ExecutableFormat = Shared Dll
; CursorPosition = 130
; FirstLine = 112
; Folding = -----
; EnableXP
; Executable = ..\drawing.dll