;**************************************************************
;     NALTOR
;**************************************************************
;   Copyright (c) 2010 Josef Sniatecki
;
; This program is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License
; As published by the Free Software Foundation; either version
; 3 of the License, Or (at your option) any later version.
;
; This program is distributed in the hope that it will be
; useful, but WITHOUT ANY WARRANTY; without even the implied
; warranty of MERCHANTABILITY Or FITNESS For A PARTICULAR
; PURPOSE. See the GNU General Public License For more details.
;
; You should have received a copy of the GNU General Public
; License along With this program;
; If Not, see "http://www.gnu.org/licenses/".
;**************************************************************



;******************************************
; Includes
;******************************************

IncludePath "..\..\..\..\Includes"
IncludeFile "Standard.pbi"
IncludeFile "Keyboard.pbi"

IncludePath ""
IncludeFile "Operno.pbi"



;******************************************
; Structures
;******************************************

Structure Entry
  Type.i
  Name.s
EndStructure
Structure Job
  *Object.OpernoObject
  
  *Refresh.Integer
  *Function.OpernoObject
  *Disabled.Integer
  
  RestTime.i
EndStructure



;******************************************
; Variables
;******************************************

Global *FileClass.OpernoObject
Global *FolderClass.OpernoObject



;******************************************
; Procedures
;******************************************

Procedure.s FolderIndex(Path.s)
  Protected Directory.i, NewList Folder.s(), Index.i = 1
  
  Directory = ExamineDirectory(#PB_Any, Path, "*.*")
  While NextDirectoryEntry(Directory)
    If DirectoryEntryType(Directory) = #PB_DirectoryEntry_Directory And DirectoryEntryName(Directory) <> "."
      AddElement(Folder())
      Folder() = DirectoryEntryName(Directory)
      OpernoOutput(0, "$cyan$" + RSet(Str(Index), 2) + " $gray$-$n$ " + Folder())
      Index + 1
    EndIf
  Wend
  FinishDirectory(Directory)
  
  Index = OpernoEvalI(OpernoInput("id"))
  If Index
    SelectElement(Folder(), Index - 1)
    ProcedureReturn Folder()
  Else
    ProcedureReturn ""
  EndIf
EndProcedure

Procedure.i PushFileValue(Name.s)
  Protected *This.OpernoValue
  
  *This          = OpernoPushValue(*FileClass)
  *This\String\s = Name
  
  ProcedureReturn *This
EndProcedure
Procedure.i CreateFileValue(Name.s)
  Protected *This.OpernoValue
  
  *This          = OpernoCreateValue(*FileClass)
  *This\String\s = Name
  
  ProcedureReturn *This
EndProcedure

Procedure.i PushFolderValue(Name.s)
  Protected *This.OpernoValue
  
  *This          = OpernoPushValue(*FolderClass)
  *This\String\s = Name
  
  ProcedureReturn *This
EndProcedure
Procedure.i CreateFolderValue(Name.s)
  Protected *This.OpernoValue
  
  *This          = OpernoCreateValue(*FolderClass)
  *This\String\s = Name
  
  ProcedureReturn *This
EndProcedure



;******************************************
; Operno classes
;******************************************

Procedure.i NALTOR_File__New()
  Protected *This.OpernoValue = OpernoThis()
  
  *This\String   = _New(String)
  *This\String\s = ""
  *This\Scope    | #OpernoValue_Binary
EndProcedure
Procedure.i NALTOR_File__Def()
  Protected *This.OpernoValue = OpernoThis()
  
  OpernoPopSValue(*This\String)
EndProcedure
Procedure.i NALTOR_File__Del()
  Protected *This.OpernoValue = OpernoThis()
  
  ClearStructure(*This\String, String)
  _Free(*This\String)
EndProcedure
Procedure.i NALTOR_File__Form()
  Protected *This.OpernoValue = OpernoThis()
  
  ProcedureReturn OpernoPushSValue(*This\String\s)
EndProcedure
Procedure.i NALTOR_File__FSet()
  Protected *This.OpernoValue = OpernoThis(), *Other.OpernoValue
  
  OpernoPopValue(@*Other)
  Select *Other\Class
    Case *OpernoStringClass
      OpernoSetValue(*Other, OpernoPushSValue(*This\String\s))
  EndSelect
  
  ProcedureReturn *Other
EndProcedure
Procedure.i NALTOR_File__Set()
  Protected *This.OpernoValue = OpernoThis(), *Other.OpernoValue
  
  OpernoPopValue(@*Other)
  Select *Other\Class
    Case *OpernoStringClass, *FileClass
      *This\String\s = *Other\String\s
  EndSelect
  
  ProcedureReturn *This
EndProcedure
Procedure.i NALTOR_File_Type()
  ProcedureReturn OpernoPushIValue(0)
EndProcedure
Procedure.i NALTOR_File_Name()
  Protected *This.OpernoValue = OpernoThis()
  
  ProcedureReturn OpernoPushSValue(GetFilePart(*This\String\s))
EndProcedure
Procedure.i NALTOR_File_Extension()
  Protected *This.OpernoValue = OpernoThis()
  
  ProcedureReturn OpernoPushSValue(GetExtensionPart(*This\String\s))
EndProcedure
Procedure.i NALTOR_File_Path()
  Protected *This.OpernoValue = OpernoThis()
  
  ProcedureReturn PushFolderValue(GetPathPart(*This\String\s))
EndProcedure
Procedure.i NALTOR_File_Rename()
  Protected *This.OpernoValue = OpernoThis(), Name.String
  
  If OpernoPopSValue(@Name)
    RenameFile(*This\String\S, Name\S)
    *This\String\S = GetPathPart(*This\String\S) + Name\S
  Else
    _OpernoArgumentError(1)
  EndIf
EndProcedure
Procedure.i NALTOR_File_Copy()
  Protected Copy.String, *This.OpernoValue = OpernoThis()
  
  If OpernoPopSValue(@Copy)
    CopyFile(*This\String\s, Copy\s)
  Else
    _OpernoArgumentError(1)
  EndIf
EndProcedure
Procedure.i NALTOR_File_Remove()
  Protected *This.String = OpernoMemory()
  
  DeleteFile(*This\s)
EndProcedure

Procedure.i NALTOR_Folder__New()
  Protected *This.OpernoValue = OpernoThis()
  
  *This\String   = _New(String)
  *This\String\s = ""
  *This\Scope    | #OpernoValue_Binary
EndProcedure
Procedure.i NALTOR_Folder__Def()
  Protected *This.OpernoValue = OpernoThis()
  
  OpernoPopSValue(*This\String)
EndProcedure
Procedure.i NALTOR_Folder__Del()
  Protected *This.OpernoValue = OpernoThis()
  
  ClearStructure(*This\String, String)
  _Free(*This\String)
EndProcedure
Procedure.i NALTOR_Folder__Form()
  Protected *This.OpernoValue = OpernoThis()
  
  ProcedureReturn OpernoPushSValue(*This\String\s)
EndProcedure
Procedure.i NALTOR_Folder__FSet()
  Protected *This.OpernoValue = OpernoThis(), *Other.OpernoValue
  
  OpernoPopValue(@*Other)
  Select *Other\Class
    Case *OpernoStringClass
      OpernoSetValue(*Other, OpernoPushSValue(*This\String\s))
  EndSelect
  
  ProcedureReturn *Other
EndProcedure
Procedure.i NALTOR_Folder__Set()
  Protected *This.OpernoValue = OpernoThis(), *Other.OpernoValue
  
  OpernoPopValue(@*Other)
  Select *Other\Class
    Case *OpernoStringClass, *FolderClass
      *This\String\s = *Other\String\s
  EndSelect
  
  ProcedureReturn *This
EndProcedure
Procedure.i NALTOR_Folder__Add()
  Protected *Value.OpernoValue, *This.String = OpernoMemory()
  
  OpernoPopValue(@*Value)
  Select *Value\Class
    Case *OpernoStringClass
      If Left(*Value\String\s, 1) = "\"
        ProcedureReturn PushFolderValue(*This\s + *Value\String\s)
      Else
        ProcedureReturn PushFileValue(*This\s + *Value\String\s)
      EndIf
  EndSelect
  
  ProcedureReturn #Null
EndProcedure
Procedure.i NALTOR_Folder_Type()
  ProcedureReturn OpernoPushIValue(1)
EndProcedure
Procedure.i NALTOR_Folder_Name()
  Protected *This.OpernoValue = OpernoThis()
  
  ProcedureReturn OpernoPushSValue(StringField(*This\String\s, CountString(*This\String\s, "\"), "\"))
EndProcedure
Procedure.i NALTOR_Folder_Down()
  Protected Path.String, Result.s
  Protected *This.OpernoValue = OpernoThis()
  
  If OpernoPopSValue(@Path)
    Result = *This\String\s + Path\s
  Else
    Result = *This\String\s + FolderIndex(*This\String\S)
  EndIf
  
  If Left(Result, 1) <> "\"
    Result + "\"
  EndIf
  
  ProcedureReturn PushFolderValue(Result)
EndProcedure
Procedure.i NALTOR_Folder_Up()
  Protected Result.s
  Protected *This.OpernoValue = OpernoThis()
  
  Result = *This\String\S
  Result = RemoveString(Result, StringField(Result, CountString(Result, "\"), "\") + "\")
  
  ProcedureReturn PushFolderValue(Result)
EndProcedure
Procedure.i NALTOR_Folder_Copy()
  Protected Copy.String, *This.OpernoValue = OpernoThis()
  
  If OpernoPopSValue(@Copy)
    CopyDirectory(*This\String\s, Copy\s, "*.*", #PB_FileSystem_Recursive)
  Else
    _OpernoArgumentError(1)
  EndIf
EndProcedure
Procedure.i NALTOR_Folder_Files()
  Protected Path.s, Directory.i, *Result.OpernoValue
  Protected *This.OpernoValue = OpernoThis()
  
  Path = *This\String\s
  
  Directory = ExamineDirectory(#PB_Any, Path, "*.*")
  If Directory
    *Result = OpernoPushQueueValue()
    While NextDirectoryEntry(Directory)
      Select DirectoryEntryType(Directory)
        Case #PB_DirectoryEntry_File
          OpernoAppendItem(*Result, CreateFileValue(Path + DirectoryEntryName(Directory)))
        Case #PB_DirectoryEntry_Directory
          If DirectoryEntryName(Directory) <> "." And DirectoryEntryName(Directory) <> ".."
            OpernoAppendItem(*Result, CreateFolderValue(Path + DirectoryEntryName(Directory) + "\"))
          EndIf
      EndSelect
    Wend
    FinishDirectory(Directory)
    
    ProcedureReturn *Result
  EndIf
EndProcedure
Procedure.i NALTOR_Folder_Explore()
  Protected *This.String = OpernoMemory()
  
  RunProgram("explorer", #Q + *This\s + #Q, "")
EndProcedure

;******************************************
; Operno functions
;******************************************

Procedure.i NALTOR_Environment_Set()
  Protected Name.String, Value.String
  
  If OpernoPopSValue(@Name) And OpernoPopSValue(@Value)
    SetEnvironmentVariable(Name\s, Value\s)
  Else
    _OpernoArgumentError(2)
  EndIf
EndProcedure
Procedure.i NALTOR_Environment_Get()
  Protected Name.String
  
  If OpernoPopSValue(@Name)
    ProcedureReturn OpernoPushSValue(GetEnvironmentVariable(Name\s))
  Else
    _OpernoArgumentError(1)
  EndIf
EndProcedure
Procedure.i NALTOR_Environment_All()
  Protected *Result.OpernoValue, *Name.OpernoValue, *Value.OpernoValue, *SubQueue.OpernoValue
  
  *Result = OpernoPushQueueValue()
  
  ExamineEnvironmentVariables()
  While NextEnvironmentVariable()
    *Name  = OpernoCreateSValue(EnvironmentVariableName())
    *Value = OpernoCreateSValue(EnvironmentVariableValue())
    
    *SubQueue = OpernoCreateQueueValue()
    OpernoAppendItem(*SubQueue, *Name)
    OpernoAppendItem(*SubQueue, *Value)
    
    OpernoAppendItem(*Result, *SubQueue)
  Wend
  
  ProcedureReturn *Result
EndProcedure
Procedure.i NALTOR_Environment_ShowAll()
  ExamineEnvironmentVariables()
  While NextEnvironmentVariable()
    OpernoOutput(0, EnvironmentVariableName())
  Wend
EndProcedure

Procedure.i NALTOR_Clipboard_Set()
  Protected Text.String
  
  If OpernoPopSValue(@Text)
    SetClipboardText(Text\s)
  Else
    _OpernoArgumentError(1)
  EndIf
EndProcedure
Procedure.i NALTOR_Clipboard_Get()
  ProcedureReturn OpernoPushSValue(GetClipboardText())
EndProcedure

Procedure.i NALTOR_Directory_Desktop()
  ProcedureReturn PushFolderValue(GetSystemPath(#CSIDL_DESKTOPDIRECTORY))
EndProcedure
Procedure.i NALTOR_Directory_Home()
  ProcedureReturn PushFolderValue(GetSystemPath(#CSIDL_PROFILE))
EndProcedure
Procedure.i NALTOR_Directory_Documents()
  ProcedureReturn PushFolderValue(GetSystemPath(#CSIDL_PERSONAL))
EndProcedure
Procedure.i NALTOR_Directory_Programs()
  ProcedureReturn PushFolderValue(GetSystemPath(#CSIDL_PROGRAM_FILES))
EndProcedure
Procedure.i NALTOR_Directory_DoUp()
  Protected Path.s
  
  Path = GetCurrentDirectory()
  Path = RemoveString(Path, StringField(Path, CountString(Path, "\"), "\") + "\")
  SetCurrentDirectory(Path)
  
  ProcedureReturn PushFolderValue(GetCurrentDirectory())
EndProcedure
Procedure.i NALTOR_Directory_DoDown()
  Protected Path.String, Directory.i, NewList Folder.s()
  
  If OpernoPopSValue(@Path)
    SetCurrentDirectory(GetCurrentDirectory() + Path\s)
  Else
    SetCurrentDirectory(GetCurrentDirectory() + FolderIndex(GetCurrentDirectory()))
  EndIf
  
  ProcedureReturn PushFolderValue(GetCurrentDirectory())
EndProcedure
Procedure.i NALTOR_Directory_Select()
  Protected Directory.i, NewList Entry.Entry(), Index.i = 1
  
  Directory = ExamineDirectory(#PB_Any, GetCurrentDirectory(), "*.*")
  While NextDirectoryEntry(Directory)
    If DirectoryEntryName(Directory) <> "."
      AddElement(Entry())
      Entry()\Type = DirectoryEntryType(Directory)
      Entry()\Name = DirectoryEntryName(Directory)
      If Entry()\Type = #PB_DirectoryEntry_Directory
        OpernoOutput(0, "$cyan$" + RSet(Str(Index), 2) + " $gray$-$n$ [" + Entry()\Name + "]")
      Else
        OpernoOutput(0, "$cyan$" + RSet(Str(Index), 2) + " $gray$-$n$ " + Entry()\Name)
      EndIf
      Index + 1
    EndIf
  Wend
  FinishDirectory(Directory)
  
  Index = OpernoEvalI(OpernoInput("id"))
  If Index
    SelectElement(Entry(), Index - 1)
    If Entry()\Type = #PB_DirectoryEntry_Directory
      SetCurrentDirectory(GetCurrentDirectory() + Entry()\Name)
      ProcedureReturn PushFolderValue(GetCurrentDirectory())
    Else
      ProcedureReturn PushFileValue(GetCurrentDirectory() + Entry()\Name)
    EndIf
  EndIf
EndProcedure
Procedure.i NALTOR_Directory_Set()
  Protected Path.String
  
  If OpernoPopSValue(@Path)
    SetCurrentDirectory(Path\s)
  Else
    _OpernoArgumentError(1)
  EndIf
EndProcedure
Procedure.i NALTOR_Directory_Get()
  ProcedureReturn PushFolderValue(GetCurrentDirectory())
EndProcedure

Procedure.i NALTOR_Run()
  Protected Path.String, *Arguments.OpernoValue, Arguments.s
  
  If OpernoPopSValue(@Path)
    OpernoPopValue(@*Arguments)
    
    If *Arguments
      Select *Arguments\Class
        Case *OpernoStringClass
          Arguments = *Arguments\String\s
        Case *FileClass, *FolderClass
          Arguments = #Q + *Arguments\String\s + #Q
        Default
          _OpernoError("Invalid arguments specified.")
      EndSelect
    EndIf
    
    RunProgram(Path\s, Arguments, GetPathPart(Path\s))
  Else
    _OpernoArgumentError(1)
  EndIf
EndProcedure
Procedure.i NALTOR_Web()
  Protected URL.String
  
  If OpernoPopSValue(@URL)
    RunProgram(GetStandardBrowser(), #Q + URL\s + #Q, "")
  Else
    RunProgram(GetStandardBrowser())
  EndIf
EndProcedure

Procedure.i NALTOR_Link()
  Protected Text.String, Path.String, Code.s
  
  If OpernoPopSValue(@Text) And OpernoPopSValue(@Path)
    Code = "main\" + Text\s + "(){"                                                                 + #N
    Code + "  if(args:count){run(" + #Q + Path\s + #Q + ",pop)}else{run(" + #Q + Path\s + #Q + ")}" + #N
    Code + "}"
    OpernoEval(Code)
  Else
    _OpernoArgumentError(2)
  EndIf
EndProcedure
Procedure.i NALTOR_WebLink()
  Protected Text.String, URL.String, Code.s
  
  If OpernoPopSValue(@Text) And OpernoPopSValue(@URL)
    Code = "main\" + Text\s + "(){"         + #N
    Code + "  web(" + #Q + URL\s + #Q + ")" + #N
    Code + "}"
    OpernoEval(Code)
  Else
    _OpernoArgumentError(2)
  EndIf
EndProcedure

Procedure.i NALTOR_SelectFile()
  ProcedureReturn PushFileValue(OpenFileRequester("Open", GetCurrentDirectory(), "*.*", 0))
EndProcedure
Procedure.i NALTOR_SelectFolder()
  ProcedureReturn PushFolderValue(RemoveString(PathRequester("Open", GetCurrentDirectory()), GetCurrentDirectory()))
EndProcedure

Procedure.i NALTOR_Standby()
  Protected NewList Job.Job(), *Group.OpernoObject, *Job.OpernoObject, *Child.OpernoObject
  Protected StartTime.i, Delta.i
  
  *Group = OpernoGet("job")
  
  *Job = OpernoFirstChild(*Group)
  While *Job
    AddElement(Job())
    Job()\Object   = *Job
    Job()\RestTime = 500
    
    *Child = OpernoFirstChild(*Job)
    While *Child
      Select *Child\Name
        Case "refresh"
          Job()\Refresh  = *Child\Variable\Value\Integer
          Job()\RestTime = Job()\Refresh\i
        Case "disabled"
          Job()\Disabled = *Child\Variable\Value\Integer
        Case "action"
          Job()\Function = *Child
      EndSelect
      *Child = OpernoNextObject(*Child)
    Wend
    
    *Job = OpernoNextObject(*Job)
  Wend
  
  
  
  KeyPressedDown(#VK_RETURN)
  IsKeyPressing(#VK_CONTROL)
  
  Repeat
    ForEach Job()
      If Not Job()\Disabled Or Job()\Disabled\i = #False
        If Job()\RestTime = 0
          OpernoOutput(0, "running " + #Q + Job()\Object\Name + #Q + " ...")
          OpernoCall(Job()\Function)
          OpernoOutput(0, "finsihed.")
          If Job()\Refresh
            Job()\RestTime = Job()\Refresh\i
          Else
            Job()\RestTime = 500
          EndIf
        Else
          Job()\RestTime - 1
        EndIf
      EndIf
      Delay(1)
    Next
  Until KeyPressedDown(#VK_RETURN) And IsKeyPressing(#VK_CONTROL)
EndProcedure



;******************************************
; Data section
;******************************************

DataSection
  FileClass:
    Data.c #OpernoObject_Function : Data.s "type" ;{
      Data.i @NALTOR_File_Type()
    ;}
    Data.c #OpernoObject_Function : Data.s "name" ;{
      Data.i @NALTOR_File_Name()
    ;}
    Data.c #OpernoObject_Function : Data.s "ext" ;{
      Data.i @NALTOR_File_Extension()
    ;}
    Data.c #OpernoObject_Function : Data.s "path" ;{
      Data.i @NALTOR_File_Path()
    ;}
    Data.c #OpernoObject_Function : Data.s "rename" ;{
      Data.i @NALTOR_File_Rename()
    ;}
    Data.c #OpernoObject_Function : Data.s "copy" ;{
      Data.i @NALTOR_File_Copy()
    ;}
    Data.c #OpernoObject_Function : Data.s "rem" ;{
      Data.i @NALTOR_File_Remove()
    ;}
    Data.c #OpernoObject_Function : Data.s "_new" ;{
      Data.i @NALTOR_File__New()
    ;}
    Data.c #OpernoObject_Function : Data.s "_def" ;{
      Data.i @NALTOR_File__Def()
    ;}
    Data.c #OpernoObject_Function : Data.s "_del" ;{
      Data.i @NALTOR_File__Del()
    ;}
    Data.c #OpernoObject_Function : Data.s "_form" ;{
      Data.i @NALTOR_File__Form()
    ;}
    Data.c #OpernoObject_Function : Data.s "_set" ;{
      Data.i @NALTOR_File__Set()
    ;}
    Data.c #OpernoObject_Function : Data.s "_fSet" ;{
      Data.i @NALTOR_File__FSet()
    ;}
    Data.c #Null

  FolderClass:
    Data.c #OpernoObject_Function : Data.s "type" ;{
      Data.i @NALTOR_Folder_Type()
    ;}
    Data.c #OpernoObject_Function : Data.s "name" ;{
      Data.i @NALTOR_Folder_Name()
    ;}
    Data.c #OpernoObject_Function : Data.s "down" ;{
      Data.i @NALTOR_Folder_Down()
    ;}
    Data.c #OpernoObject_Function : Data.s "up" ;{
      Data.i @NALTOR_Folder_Up()
    ;}
    Data.c #OpernoObject_Function : Data.s "copy" ;{
      Data.i @NALTOR_Folder_Copy()
    ;}
    Data.c #OpernoObject_Function : Data.s "files" ;{
      Data.i @NALTOR_Folder_Files()
    ;}
    Data.c #OpernoObject_Function : Data.s "exp" ;{
      Data.i @NALTOR_Folder_Explore()
    ;}
    Data.c #OpernoObject_Function : Data.s "_new" ;{
      Data.i @NALTOR_Folder__New()
    ;}
    Data.c #OpernoObject_Function : Data.s "_def" ;{
      Data.i @NALTOR_Folder__Def()
    ;}
    Data.c #OpernoObject_Function : Data.s "_del" ;{
      Data.i @NALTOR_Folder__Del()
    ;}
    Data.c #OpernoObject_Function : Data.s "_form" ;{
      Data.i @NALTOR_Folder__Form()
    ;}
    Data.c #OpernoObject_Function : Data.s "_set" ;{
      Data.i @NALTOR_Folder__Set()
    ;}
    Data.c #OpernoObject_Function : Data.s "_fSet" ;{
      Data.i @NALTOR_Folder__FSet()
    ;}
    Data.c #OpernoObject_Function : Data.s "_add" ;{
      Data.i @NALTOR_Folder__Add()
    ;}
    Data.c #Null
  
  FunctionTable:
    Data.c #OpernoObject_Group : Data.s "dir" ;{
      Data.c #OpernoObject_Function : Data.s "desk" ;{
        Data.i @NALTOR_Directory_Desktop()
      ;}
      Data.c #OpernoObject_Function : Data.s "home" ;{
        Data.i @NALTOR_Directory_Home()
      ;}
      Data.c #OpernoObject_Function : Data.s "docs" ;{
        Data.i @NALTOR_Directory_Documents()
      ;}
      Data.c #OpernoObject_Function : Data.s "progs" ;{
        Data.i @NALTOR_Directory_Programs()
      ;}
      Data.c #OpernoObject_Function : Data.s "up" ;{
        Data.i @NALTOR_Directory_DoUp()
      ;}
      Data.c #OpernoObject_Function : Data.s "down" ;{
        Data.i @NALTOR_Directory_DoDown()
      ;}
      Data.c #OpernoObject_Function : Data.s "sel" ;{
        Data.i @NALTOR_Directory_Select()
      ;}
      Data.c #OpernoObject_Function : Data.s "set" ;{
        Data.i @NALTOR_Directory_Set()
      ;}
      Data.c #OpernoObject_Function : Data.s "get" ;{
        Data.i @NALTOR_Directory_Get()
      ;}
      Data.c #Null
    ;}
    Data.c #OpernoObject_Group : Data.s "env" ;{
      Data.c #OpernoObject_Function : Data.s "set" ;{
        Data.i @NALTOR_Environment_Set()
      ;}
      Data.c #OpernoObject_Function : Data.s "get" ;{
        Data.i @NALTOR_Environment_Get()
      ;}
      Data.c #OpernoObject_Function : Data.s "all" ;{
        Data.i @NALTOR_Environment_All()
      ;}
      Data.c #OpernoObject_Function : Data.s "showAll" ;{
        Data.i @NALTOR_Environment_ShowAll()
      ;}
      Data.c #Null
    ;}
    Data.c #OpernoObject_Group : Data.s "clip" ;{
      Data.c #OpernoObject_Function : Data.s "set" ;{
        Data.i @NALTOR_Clipboard_Set()
      ;}
      Data.c #OpernoObject_Function : Data.s "get" ;{
        Data.i @NALTOR_Clipboard_Get()
      ;}
      Data.c #Null
    ;}
    Data.c #OpernoObject_Function : Data.s "run" ;{
      Data.i @NALTOR_Run()
    ;}
    Data.c #OpernoObject_Function : Data.s "web" ;{
      Data.i @NALTOR_Web()
    ;}
    Data.c #OpernoObject_Function : Data.s "link" ;{
      Data.i @NALTOR_Link()
    ;}
    Data.c #OpernoObject_Function : Data.s "webLink" ;{
      Data.i @NALTOR_WebLink()
    ;}
    Data.c #OpernoObject_Function : Data.s "selFile" ;{
      Data.i @NALTOR_SelectFile()
    ;}
    Data.c #OpernoObject_Function : Data.s "selFolder" ;{
      Data.i @NALTOR_SelectFolder()
    ;}
    Data.c #OpernoObject_Function : Data.s "standby" ;{
      Data.i @NALTOR_Standby()
    ;}
    Data.c #OpernoObject_Group : Data.s "job" ;{
      Data.c #Null
    ;}
    Data.c #Null
EndDataSection



;******************************************
; Main
;******************************************

ProcedureDLL.i InitLibrary()
  *FileClass = OpernoCreateGroup("file", *OpernoStandardLibrary)
  OpernoExtractObject(*FileClass, ?FileClass)
  
  *FolderClass = OpernoCreateGroup("folder", *OpernoStandardLibrary)
  OpernoExtractObject(*FolderClass, ?FolderClass)
  
  OpernoExtractObject(*OpernoStandardLibrary, ?FunctionTable)
EndProcedure
ProcedureDLL.i FileClass()
  ProcedureReturn *FileClass
EndProcedure
ProcedureDLL.i FolderClass()
  ProcedureReturn *FolerClass
EndProcedure
; IDE Options = PureBasic 4.41 (Windows - x86)
; CursorPosition = 337
; FirstLine = 94
; Folding = AAACAEAAY9---------
; EnableXP
; Executable = ..\..\Applications\Naltor\Naltor.exe