;*************************************************************************************
;          ______  ______  ______  ______  __   __  ______
;         / __  / / __  / / ____/ / __  / /  | / / / __  /
;        / / / / / /_/ / / /___  / /_/ / /   |/ / / / / /
;       / / / / / ____/ / ____/ /    _/ / /|   / / / / /
;      / /_/ / / /     / /___  / /| |  / / |  / / /_/ /
;     /_____/ /_/     /_____/ /_/ |_| /_/  |_/ /_____/
;      A dynamic and object-oriented programming language
;
;     M A I N
;
;*************************************************************************************
;   (c) 2010 Josef Sniatecki
;
; This program is free software; you can redistribute it and/or modify it under the
; terms of the GNU General Public License As published by the Free Software Foundation;
; either version 3 of the License, Or (at your option) any later version.
;
; This program is distributed in the hope that it will be useful, but WITHOUT ANY
; WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
; PARTICULAR PURPOSE. See the GNU General Public License For more details.
;
; You should have received a copy of the GNU General Public License along With this
; program; if not, see "http://www.gnu.org/licenses/".
;*************************************************************************************




EnableExplicit

;*******************************************************
; Includes
;*******************************************************

IncludePath "..\..\Includes"
IncludeFile "Standard.pbi"

  Structure QueueItem_Memory
    StructureUnion
      *Memory
      *Value.Value
      *Object.Object
      *Token.Token
      *Term.Term
      *Item.QueueItem
    EndStructureUnion
  EndStructure
  #Templates_Queue_Memory = #True
IncludeFile "Queue.pbi"

IncludeFile "WindowScreenshot.pbi"
IncludeFile "EditorGadget.pbi"
IncludeFile "Cursor.pbi"
; IncludeFile "MemoryManager.pbi"

IncludePath #PB_Compiler_FilePath



;*******************************************************
; Macros
;*******************************************************

; State:
Macro _Error(Message, Action1 = , Action2 = , Action3 = )
  Error(Message)
  Action1
  Action2
  Action3
  ProcedureReturn
EndMacro
Macro _ErrorB(Message, Level = 1, Action1 = , Action2 = , Action3 = )
  Error(Message)
  Action1
  Action2
  Action3
  Break Level
EndMacro
Macro _ErrorG(Label, Message = "")
  Error(Message)
  Goto Label#_Error
EndMacro
Macro _ErrorP(Count)
  Error("The minimal argument count is " + Str(Count) + "!")
  ProcedureReturn
EndMacro
Macro _E(Message = "", Action1 = , Action2 = , Action3 = )
  If Error
    Error(Message)
    Action1
    Action2
    Action3
    ProcedureReturn
  EndIf
EndMacro
Macro _EB(Level = 1, Message = "")
  If Error
    Error(Message)
    Break Level
  EndIf
EndMacro
Macro _EG(Label, Message = "")
  If Error
    Error(Message)
    Goto Label#_Error
  EndIf
EndMacro
Macro _R(Decrement = 0, Action1 = , Action2 = , Action3 = )
  If Truncation
    CheckTruncation(Decrement)
    Action1
    Action2
    Action3
    ProcedureReturn
  EndIf
EndMacro
Macro _RB(Decrement = 0, Level = 1, Action1 = , Action2 = , Action3 = )
  If Truncation
    CheckTruncation(Decrement)
    Action1
    Action2
    Action3
    Break Level
  EndIf
EndMacro
; Token:
Macro _N()
  ;Next token:
  _Move(*Current\TokenItem, N)
EndMacro
Macro _P()
  ;Previous token:
  If *Current\TokenItem
    _Move(*Current\TokenItem, P)
  Else
    *Current\TokenItem = *Current\TokenQueue\Last
  EndIf
EndMacro
Macro _SN(Variable)
  ;Next token:
  Variable = *Current\TokenItem\Memory
  _Move(*Current\TokenItem, N)
EndMacro
Macro _SP(Variable)
  ;Previous token:
  Variable = *Current\TokenItem\Memory
  _Move(*Current\TokenItem, P)
EndMacro
Macro _S(Variable)
  Variable = *Current\TokenItem\Memory
EndMacro
; Misc:
Macro _ReturnDLLString(Value)
  DLLString = Value
  ProcedureReturn DLLString
EndMacro
Macro _CM(Name)
  ;Class method:
  "_" + _Q#Name#_Q
EndMacro



;*******************************************************
; Constants
;*******************************************************

; Source code:
#OpernoPath = #PB_Compiler_FilePath
#Version    = "5.50"
; Object:
Enumeration 1
  #Object_Group
  #Object_Function
  #Object_Variable
EndEnumeration
Enumeration
  #ClassMethod_New
  #ClassMethod_Define
  #ClassMethod_Delete
  #ClassMethod_IsTrue
  #ClassMethod_Set
  #ClassMethod_FriendSet
  #ClassMethod_Add
  #ClassMethod_FriendAdd
  #ClassMethod_Subtract
  #ClassMethod_Negative
  #ClassMethod_FriendSubtract
  #ClassMethod_Multiply
  #ClassMethod_FriendMultiply
  #ClassMethod_Divide
  #ClassMethod_FriendDivide
  #ClassMethod_Power
  #ClassMethod_FriendPower
  #ClassMethod_DoAdd
  #ClassMethod_FriendDoAdd
  #ClassMethod_DoSubtract
  #ClassMethod_FriendDoSubtract
  #ClassMethod_DoMultiply
  #ClassMethod_FriendDoMultiply
  #ClassMethod_DoDivide
  #ClassMethod_FriendDoDivide
  #ClassMethod_Equal
  #ClassMethod_Less
  #ClassMethod_FriendLess
  #ClassMethod_Greater
  #ClassMethod_FriendGreater
  #ClassMethod_Increment
  #ClassMethod_Decrement
  #ClassMethod_Format
  #ClassMethod_Arguments
  #ClassMethod_Vector
  #ClassMethod_Iterate
  #ClassMethod_FileRead
  #ClassMethod_FileWrite
EndEnumeration : #ClassMethodCount = #PB_Compiler_EnumerationValue - 1
Enumeration
  #Function_Address
  #Function_Block
EndEnumeration
; Types:
Enumeration
  #Type_Integer
  #Type_Float
  #Type_Quad
  #Type_Double
  #Type_String
  #Type_Bool
  #Type_Variant
EndEnumeration
; Value scopes:
#Value_Reference  = %000001
#Value_Binary     = %000010
#Value_Searchable = %000100
#Value_Literal    = %001000
#Value_Stack      = %010000
#Value_Variable   = %100000
; Token:
Enumeration
  #Token_Name
  #Token_Value
  #Token_Operator
  #Token_Control
  #Token_Seperator
  #Token_Block
  #Token_Keyword
EndEnumeration
; Token IDs:
Enumeration
  #Operator_Set
  #Operator_Add
  #Operator_Subtract
  #Operator_Multiply
  #Operator_Divide
  #Operator_Power
  #Operator_Root
  #Operator_DoAdd
  #Operator_DoSubtract
  #Operator_DoMultiply
  #Operator_DoDivide
  #Operator_Equal
  #Operator_Less
  #Operator_Greater
  #Operator_NotEqual
  #Operator_LessEqual
  #Operator_GreaterEqual
  #Operator_Increment
  #Operator_Decrement
  #Operator_Format
  #Operator_Input
  #Operator_Reference
EndEnumeration : #OperatorCount = #PB_Compiler_EnumerationValue - 1
Enumeration
  #Seperator_Command
  #Seperator_Argument
  #Seperator_Block
EndEnumeration
Enumeration
  #Bracket_Round
  #Bracket_Square
  #Bracket_Curly
EndEnumeration : #BracketCount = #PB_Compiler_EnumerationValue - 1
Enumeration
  #Control_Next
  #Control_Previous
  #Control_Type
  #Control_Class
  #Control_ObjectClass
  #Control_Address
  #Control_ECT
EndEnumeration
Enumeration
  #Keyword_Define
  #Keyword_IsDefined
  #Keyword_Delete
  #Keyword_Pop
  #Keyword_If
  #Keyword_Else
  #Keyword_ElseIf
  #Keyword_While
  #Keyword_Repeat
  #Keyword_For
  #Keyword_Switch
  #Keyword_And
  #Keyword_Or
  #Keyword_XOr
  #Keyword_Not
  #Keyword_Break
  #Keyword_Return
  #Keyword_Output
  #Keyword_End
EndEnumeration : #KeywordCount = #PB_Compiler_EnumerationValue - 1
; Term types:
Enumeration
  #Term_Number
  #Term_Indentifer
  #Term_Sum
  #Term_Negative
  #Term_Product
  #Term_Division
  #Term_Power
EndEnumeration
; HTML-generator tags:
Enumeration
  #HTMLGenerator_Tag_Default
  #HTMLGenerator_Tag_Number
  #HTMLGenerator_Tag_String
  #HTMLGenerator_Tag_Operator
  #HTMLGenerator_Tag_Bracket
  #HTMLGenerator_Tag_Character
  #HTMLGenerator_Tag_Seperator
  #HTMLGenerator_Tag_Keyword
  #HTMLGenerator_Tag_ClassMethod
  #HTMLGenerator_Tag_Symbol
  #HTMLGenerator_Tag_Comment
EndEnumeration : #HTMLGenerator_TagCount = #PB_Compiler_EnumerationValue - 1
; Break level states:
#Truncation_Return = -1
#Truncation_End    = -2
; Console:
Enumeration
  #Symbol_None
  #Symbol_Default
  #Symbol_ETC
  #Symbol_Point
  #Symbol_Output
  #Symbol_Input
  #Symbol_Error
  #Symbol_Message
EndEnumeration : #SymbolCount = #PB_Compiler_EnumerationValue - 1
#SymbolLength    = 3
#SymbolSeperator = " : "
#IndentLength    = 2
; Attribute konstants for functions:
#RunPath_Define = %01
#RunPath_Global = %10
; GUI:
Enumeration
  #GUIWindow
  #ViewerWindow
EndEnumeration
Enumeration
  #GUIMenu
  #GUIMenuI
EndEnumeration
Enumeration
  #GUIMenu_Operno_NewInstance
  #GUIMenu_Operno_Restart
  #GUIMenu_Operno_OpenConsole
  #GUIMenu_Operno_SaveRecord
  #GUIMenu_Operno_SelectFont
  #GUIMenu_Operno_Close
  #GUIMenu_Execute_Input
  #GUIMenu_Execute_InputPlusResult
  #GUIMenu_Execute_File
  #GUIMenu_Execute_ClipboardContents
  #GUIMenu_Execute_TokenCompiler
  #GUIMenu_Output_Copy
  #GUIMenu_Output_Clear
  #GUIMenu_Output_DisplayInput
  #GUIMenu_HTML_GenerateHilightedFile
  #GUIMenu_HTML_Configuration
  #GUIMenu_Tools_Editor
  #GUIMenu_Tools_Viewer
  #GUIMenu_Extras_About
  #GUIMenu_Extras_Help
  #GUIMenu_Extras_Tutorial
  #GUIMenu_Extras_Screenshot
  
  #GUIMenuI_Cut
  #GUIMenuI_Copy
  #GUIMenuI_Paste
  #GUIMenuI_SelectAll
  #GUIMenuI_ContinueInFile
  
  #Viewer_Tree
EndEnumeration
Enumeration
  #GUI_Output
  #GUI_Input
  #GUI_OutputFrame
  #GUI_InputFrame
EndEnumeration
Enumeration
  #GUIFont
EndEnumeration
; Master callback types:
Enumeration
  #Callback_Output
  #Callback_Input
  #Callback_ClearOutput
  #Callback_Wait
  #Callback_WaitForReturn
EndEnumeration



;*******************************************************
; Structures
;*******************************************************

Structure Object
  Type.i
  Name.s
  StructureUnion
    *Group.Group
    *Function.Function
    *Variable.Variable
    *Memory
  EndStructureUnion
  *Super.Object
  *Item.QueueItem
  *Reference.Object
EndStructure
Structure Group
  *ObjectQueue.Queue
  *ClassMethod[#ClassMethodCount + 1]
  *Value.Value
EndStructure
Structure Function
  Type.i
  StructureUnion
    *Address
    *TokenQueue.Queue
  EndStructureUnion
  *Class.Object
EndStructure
Structure Variable
  *Value.Value
EndStructure

Structure Value
  *Class.Object
  StructureUnion
    *Memory
    *Integer.Integer
    *Float.Float
    *Quad.Quad
    *Double.Double
    *String.String
    *StringPointer.StringPointer
    *Queue.Queue
    *Object.Object
    *Item.QueueItem
    *Std_CoordinateSystem.Std_CoordinateSystem
    *Std_Array.Std_Array
  EndStructureUnion
  Scope.i
  *Variable.Object
  *ReferenceQueue.Queue
  *Original.Value
EndStructure
Structure StringPointer
  *String
  *Character.Character
  *EndOfString
EndStructure
Structure Std_CoordinateSystem
  MinX.f
  MaxX.f
  MinY.f
  MaxY.f
  XFactor.f
  YFactor.f
  
  Image.i
  Width.i
  Height.i
EndStructure
Structure Std_Array
  *Entries
  Size.i
EndStructure

Structure Token
  Type.i
  StructureUnion
    Integer.i
    ID.i
    *Value.Value
    String.s
    Name.s
    Operator.i
    Seperator.i
    Control.i
    Keyword.i
    *Block.Token_Block
  EndStructureUnion
EndStructure
Structure Token_Block
  Bracket.i
  *TokenQueue.Queue
EndStructure
Structure Term
  Type.i
  
  StructureUnion
    Number.d
    Indentifer.s
    *Children.Queue
    *FirstChild.Term
  EndStructureUnion
  StructureUnion
    *SecondChild.Term
  EndStructureUnion
EndStructure

Structure Level
  *Previous.Level
  *TokenQueue.Queue
  *TokenItem.QueueItem
  *ArgumentQueue.Queue
  *Object.Object
  *This.Value
  *Stack.Stack
  *SharedStack.Stack
  *Super.Level
EndStructure
Structure Stack
  *Previous.Stack
  *Value.Value
EndStructure
Structure CompileTerm
  *TokenQueue.Queue
  *TokenItem.QueueItem
EndStructure

Structure HTMLGenerator_Tag
  Bold.i
  Italic.i
  Underline.i
  Color.i
EndStructure



;*******************************************************
; Declarations
;*******************************************************

Declare.i Error(Message.s)
Declare.i CheckTruncation(Decrement.i)
Declare.i FatalErrorCallback()


Declare.i OpenLevel(*TokenQueue.Queue, *ArgumentQueue.Queue, *Object.Object, *This.Value, SharedStack.i = #False, Super.i = #False)
Declare.i CloseLevel()
Declare.i ClearLevels()
Declare.i AddStackValue()
Declare.i ClearStack()

Declare.i SearchLevels(Name.s)


Declare.i KeywordID(Keyword.s)
Declare.i ClassMethodID(Method.s)
Declare.i OperatorID(Characters.s)
Declare.i BracketID(Bracket.s)

Declare.i StandardClass(Name.s)
Declare.s ClassName(*Class)


Declare.i CreateValue(*Class.Object, *SourceValue.Value = 0)
Declare.i CreateVValue()
Declare.i CreateIValue(Integer.i)
Declare.i CreateFValue(Float.f)
Declare.i CreateQValue(Quad.q)
Declare.i CreateDValue(Double.d)
Declare.i CreateBValue(State.i)
Declare.i CreateSValue(String.s)
Declare.i CreateSPValue(*String, *Character.Character, *EndOfString = 0)
Declare.i CreateQueueValue(*Queue.Queue = 0, Reference.i = #False)
Declare.i CreateBlockValue(*TokenQueue.Queue = 0, Reference.i = #False)
Declare.i CreatePValue(*Object.Object)
Declare.i CreateReferenceValue(*Original.Value)

Declare.i FreeValue(*Value.Value)
Declare.i FreeHardValue(*Value.Value)
Declare.i CopyValue(*Source.Value, *Destination.Value)
Declare.i DuplicateValue(*Value.Value)
Declare.s ValueName(*Value.Value)
Declare.s ValuePath(*Value.Value)

Declare.i PushValue(*Class.Object, *SourceValue.Value = 0)
Declare.i PushVValue()
Declare.i PushIValue(Integer.i)
Declare.i PushFValue(Float.f)
Declare.i PushQValue(Quad.q)
Declare.i PushDValue(Double.d)
Declare.i PushBValue(State.i)
Declare.i PushSValue(String.s)
Declare.i PushSPValue(*String, *Character.Character, *EndOfString = 0)
Declare.i PushQueueValue(*Queue.Queue = 0, Reference.i = #False)
Declare.i PushBlockValue(*TokenQueue.Queue = 0, Reference.i = #False)
Declare.i PushPValue(*Object.Object)
Declare.i PushOValue(*Object.Object, Reference.i = #False)
Declare.i PushReferenceValue(*Original.Value)
Declare.i PushDuplicatedValue(*Value.Value)

Declare.i ValueI(*Value.Value)
Declare.f ValueF(*Value.Value)
Declare.q ValueQ(*Value.Value)
Declare.d ValueD(*Value.Value)
Declare.i ValueB(*Value.Value)
Declare.s ValueS(*Value.Value)
Declare.i ValueP(*Value.Value)

Declare.i PopValue(*Result.Value)
Declare.i PopIValue(*Result.Integer)
Declare.i PopFValue(*Result.Float)
Declare.i PopQValue(*Result.Quad)
Declare.i PopDValue(*Result.Double)
Declare.i PopBValue(*Result.Integer)
Declare.i PopSValue(*Result.String)
Declare.i PopPValue(*Result.Integer)
Declare.i PopValueAddress(*Result.Integer)

Declare.i Arguments1(*Value1.Value)
Declare.i Arguments2(*Value1.Value, *Value2.Value)


Declare.i OperateValue(OperationID.i, *LHS.Value, *RHS.Value)
Declare.i OperateValueVectorial(Operation.i, *LHS.Value, *RHS.Value)

Declare.i OperateObjectAndObject(Operation.i, *LObject.Object, *RObject.Object, *ResultObject.Object)
Declare.i OperateObjectAndQueue(Operation.i, *LObject.Object, *RQueue.Queue, *ResultObject.Object)
Declare.i OperateObjectAndValue(Operation.i, *LObject.Object, *RValue.Value, *ResultObject.Object)
Declare.i OperateQueueAndObject(Operation.i, *LQueue.Queue, *RObject.Object, *ResultObject.Object)
Declare.i OperateValueAndObject(Operation.i, *LValue.Value, *RObject.Object, *ResultObject.Object)
Declare.i OperateQueueAndQueue(Operation.i, *LQueue.Queue, *RQueue.Queue, *ResultQueue.Queue)
Declare.i OperateQueueAndValue(Operation.i, *LQueue.Queue, *RValue.Value, *ResultQueue.Queue)
Declare.i OperateValueAndQueue(Operation.i, *LValue.Value, *RQueue.Queue, *ResultQueue.Queue)
Declare.i OperateObject(Operation.i, *Object.Object, *ResultObject.Object)
Declare.i OperateQueue(Operation.i, *Queue.Queue, *ResultQueue.Queue)

Declare.i CompareValue(*Value.Value)
Declare.i CompareObject(*Object.Object)
Declare.i CompareQueue(*Queue.Queue)

Declare.i SubtractOperator(*LHS.Value, *RHS.Value)
Declare.i RootOperator(*LHS.Value, *RHS.Value)
Declare.i NotEqualOperator(*LHS.Value, *RHS.Value)
Declare.i LessEqualOperator(*LHS.Value, *RHS.Value)
Declare.i GreaterEqualOperator(*LHS.Value, *RHS.Value)
Declare.i IncrementOperator(*LHS.Value, *RHS.Value)
Declare.i DecrementOperator(*LHS.Value, *RHS.Value)
Declare.i FormatOperator(*LHS.Value, *RHS.Value)
Declare.i InputOperator(*LHS.Value, *RHS.Value)

Declare.i IsValueTrue(*Value.Value)
Declare.i SetValue(*Value.Value, *Source.Value)
Declare.i AddValue(*Value.Value, *Other.Value)
Declare.i SubtractValue(*Value.Value, *Other.Value)
Declare.i MultiplyValue(*Value.Value, *Other.Value)
Declare.i DivideValue(*Value.Value, *Other.Value)
Declare.i IncrementValue(*Value.Value)
Declare.i DecrementValue(*Value.Value)
Declare.i ValueSum(*First.Value, *Second.Value)
Declare.i ValueDifference(*First.Value, *Second.Value)
Declare.i ValueProduct(*First.Value, *Second.Value)
Declare.i ValueQuotient(*First.Value, *Second.Value)
Declare.i ValuePower(*First.Value, *Second.Value)
Declare.i IsValueEqual(*Value.Value, *Other.Value)
Declare.i IsValueLess(*Value.Value, *Other.Value)
Declare.i IsValueGreater(*Value.Value, *Other.Value)
Declare.s FormatValue(*Value.Value)
Declare.i WriteValue(File.i, *Value.Value)
Declare.i ReadValue(File.i, *Value.Value)


Declare.i CreateObject(Type.i, Name.s, *Super.Object)
Declare.i CreateGroup(Name.s, *Super.Object = 0, *Value.Value = 0)
Declare.i CreateVirtualTable(TableSize.i)
Declare.i CreateFunction(Name.s, *Super.Object = 0)
Declare.i CreateAddressFunction(Name.s, *Address, *Super.Object = 0)
Declare.i CreateBlockFunction(Name.s, *TokenQueue.Queue, *Class.Object = 0, *Super.Object = 0)
Declare.i CreateVariable(Name.s, *Value.Value, *Super.Object = 0)
Declare.i CreateReferenceObject(Name.s, *Original.Object, *Super.Object = 0)

Declare.i FreeObject(*Object.Object)
Declare.i FreeGroup(*Group.Group)
Declare.i FreeFunction(*Function.Function)
Declare.i FreeVariable(*Variable.Variable)

Declare.s GroupVariableName(*Object.Object)
Declare.i PopObject(*Object.Object)
Declare.i ExtractObject(*Super.Object, *Memory.All)
Declare.i ImportGroup(*Source.Object, *Destination.Object)
Declare.s FormatObject(*Object.Object, Indent.i = 0)
Declare.i WriteObject(File.i, *Object.Object)
Declare.i ReadObject(File.i, *Object.Object)

Declare.i GetObject(*Super.Object, Path.s)
Declare.i FindObject(*Super.Object, Name.s)
Declare.i FindSuperObject(*Object.Object, *Super.Object)
Declare.s ObjectPath(*Object.Object)


Declare.i AddIntegerToken(*Queue.Queue, Type.i, Integer.i)
Declare.i AddValueToken(*Queue.Queue, Type.i, *Value.Value)
Declare.i AddStringToken(*Queue.Queue, Type.i, String.s)
Declare.i AddBlockToken(*Queue.Queue, Type.i, BracketID.i, *TokenQueue.Queue)
Declare.i FreeToken(*Token.Token)
Declare.i CopyToken(*Source.Token, *Destination.Token)

Declare.s FormatToken(*Token.Token)
Declare.s FormatTokenQueue(*Queue.Queue, Indent.i = 0)

Declare.i SaveTokenQueue(Filename.s, *TokenQueue.Queue)
Declare.i LoadTokenQueue(Filename.s, *TokenQueue.Queue)
Declare.i WriteTokenQueue(File.i, *TokenQueue.Queue)
Declare.i ReadTokenQueue(File.i, *TokenQueue.Queue)


Declare.i Parse(*TokenQueue.Queue, *Pointer.Character, SingleLine.i = #False)
Declare.i ParseFile(Name.s, *TokenQueue.Queue)
Declare.i ParseSymbol(*TokenQueue.Queue, Name.s, *Pointer.Character)
Declare.i ParseIfControl(*TokenQueue.Queue, *Pointer.Character)

Declare.i Parser_IgnoreNewLine(*Pointer.Character, *Line)
Declare.i Parser_IgnoreComment(*Pointer.Character)
Declare.i Parser_IgnoreString(*Pointer.Character)
Declare.i Parser_CountLevel(Code.s)
Declare.s Parser_GetCode(*Pointer.All, EnableArguments.i = #False)
Declare.i Parser_ScanBlock(*Pointer.All)
Declare.i Parser_GetArguments(*Pointer.All, EnableError.i = #True)
Declare.i Parser_GetName(*Pointer.Character)


Declare.i Run(Priority.i = 0)
Declare.i RunBlock()
Declare.i RunControlBlock()
Declare.i RunCommands()

Declare.i Call(*Object.Object, *ArgumentQueue.Queue = 0, *This.Value = 0, Result.i = #True)
Declare.i CallFast(*Super.Object, Name.s, *ArgumentQueue.Queue = 0, *This.Object = 0, Result.i = #True, EnableError.i = #True)
Declare.i CallClassMethod(*Class.Object, Method.i, *ArgumentQueue.Queue = 0, *This.Value = 0, Result.i = #True, EnableError.i = #True)
Declare.i RunArguments(*TokenQueue.Queue, CopyLiterals.i = #False)

Declare.i RunObjectPath(*Super.Object, Mode.i, Name.s, *This.Value = 0, *ResultThis.Integer = 0, EnableError.i = #True)
Declare.i RunSubObjects(*Super.Object)

Declare.i RunDefinition(Name.s, *Super.Object)
Declare.i RunGroupDefinition(*Object.Object)
Declare.i RunFunctionDefinition(*Object.Object)
Declare.i RunQueueDefinition(*Object.Object)

Declare.i RunObject(*Object.Object, *This.Value)
Declare.i RunClass()

Declare.i NextArgument()
Declare.i Ignore()

Declare.i Enumerate(*First.Value, *Last.Value)


Declare.i Execute(Code.s, *Object.Object = 0, SharedStack.i = #False)
Declare.i ExecuteI(Code.s, *Object.Object = 0, SharedStack.i = #False)
Declare.f ExecuteF(Code.s, *Object.Object = 0, SharedStack.i = #False)
Declare.q ExecuteQ(Code.s, *Object.Object = 0, SharedStack.i = #False)
Declare.d ExecuteD(Code.s, *Object.Object = 0, SharedStack.i = #False)
Declare.s ExecuteS(Code.s, *Object.Object = 0, SharedStack.i = #False)
Declare.i ExecuteCommands(Code.s, *Object.Object = 0, SharedStack.i = #False)
Declare.i ExecuteObject(Path.s, *Super.Object, Mode.i, *ResultThis.Integer = 0, EnableError.i = #True)
Declare.i ExecuteFile(Name.s)

Declare.i ImportLibrary(Name.s)


Declare.i Statement_If(*Value.Value = #Null)
Declare.i Statement_While(*Value.Value = #Null)
Declare.i Statement_Repeat()
Declare.i Statement_For(*Value.Value)
Declare.i Statement_RunForCommands(*StartTokenItem.QueueItem, *Value.Value)
Declare.i Statement_FreeFor(*ArgumentQueue.Queue, *StatementObject.Object, *IndentiferObject.Object, *IndentiferQueue.Queue)
Declare.i Statement_Switch(*Value.Value)
Declare.i Statement_SwitchBlock(*StartTokenItem.QueueItem, *Value.Value)
Declare.i Statement_SwitchCase(*Token.Token, *Value.Value)


Declare.i LoadOptions()
Declare.i SaveOptions()

Declare.i DisplayConsoleOutput(Text.s)
Declare.i DisplayOutput(Text.s)
Declare.s ConvertOutput(Text.s, Indent.i)
Declare.i OutputSymbol(Symbol.i)

Declare.i OutputT(Text.s, Indent.i = 0)
Declare.i OutputL(Symbol.i, Text.s = "", Indent.i = 0)
Declare.i OutputFL(Symbol.i, Text.s, Indent.i = 0)
Declare.i OutputN()
Declare.i OutputE()
Declare.i OutputHL()
Declare.i OutputFLW(Symbol.i, Text.s, Indent.i = 0)

Declare.s ReadInput()
Declare.s InputFL(Symbol.i, Indent.i = 0)
Declare.s InputFLT(Symbol.i, Text.s, Indent.i = 0)
Declare.i InputYN(Question.s, Indent.i = 0)
Declare.s InputC(Indent.i = 0)
Declare.i WaitForReturn()

Declare.i OpenUI(EnableHeader.i = #True)
Declare.i OpenPreview()

Declare.i Interface_Head()
Declare.i Interface_Startup()
Declare.i Interface_Execute()
Declare.i Interface_ExecuteClipboard()
Declare.i Interface_Compile()
Declare.i Interface_HTML()
Declare.i Interface_GenerateHelp(Path.s = "")
Declare.i Interface_Tutorial()
Declare.i Interface_OpenHelp()
Declare.i Interface_About()

Declare.i Wait(Milliseconds.i)

Declare.i InitRecord()
Declare.i AddRecord(Code.s)
Declare.i SaveRecord()


Declare.i OpenGUIWindow()
Declare.i OpenGUI()
Declare.i DisplayGUIOutput(Text.s)
Declare.s ReadGUIInput()


Declare.i OpenViewer()
Declare.i ExamineViewer(Event.i, Menu.i, Gadget.i, Type.i)
Declare.i UpdateViewer()
Declare.i UpdateViewerTree(*Object.Object, Level.i = 0)


Declare.i ExtractIArray(Array Integer.i(1), *Memory.Integer, *EndOfMemory)
Declare.i ExtractSArray(Array String.s(1), *Memory, *EndOfMemory)

Declare.i TurnColor(Color.i)
Declare.s CamelCase(Text.s)
Declare.s SpacedCase(Text.s)


Declare.i GenerateHTMLFile(SourceFilename.s, HTMLFilename.s)
Declare.i GenerateHTMLCode(File.i, *Pointer.Character)
Declare.i HTMLCodeStyles(File.i)
Declare.s HTMLString(String.s)
Declare.s GenerateHTMLString(String.s, Tag.i)
Declare.i ExtractHTMLGeneratorTags(Array Tag.HTMLGenerator_Tag(1), *Memory.Integer)
Declare.i HTMLGeneratorConfiguration()

Declare.i GenerateHelpIndex(SourceFileName.s, DestinationFileName.s)
Declare.i GenerateHelp(SourceFileName.s, DestinationFileName.s)


Declare.i StartTutorial(FileName.s)


Declare.i DimensionQueue(*Queue.Queue, *Class.Object, *DefinitionQueue.Queue, *DimensionItem.QueueItem = 0)

Declare.i MultiplyBlock(*Block.Queue, Times.i)

Declare.i PrepareReference(*Reference.Value, *Original.Value)
Declare.i UpdateReferences(*Original.Value)
Declare.i ResetReferences(*ReferenceQueue.Queue)
Declare.i FreeReference(*Reference.Value)


Declare.i FreeTerm(*Term.Term)
Declare.i CopyTerm(*SourceTerm.Term, *DestinationTerm.Term)

Declare.i CreateTermNumber(Number.d)
Declare.i CreateTermIndentifer(Name.s)
Declare.i CreateTermSum(*Children.Queue = #Null)
Declare.i CreateTermProduct(*Children.Queue = #Null)
Declare.i CreateTermDivision()
Declare.i CreateTermPower()

Declare.i CreateTermAddition(*A.Term, *B.Term)
Declare.i CreateTermMultiplication(*A.Term, *B.Term)

Declare.i CompileTerm(*Level.CompileTerm, Priority.i = 0)
Declare.i CompileTermByTokens(*TokenQueue.Queue)

Declare.i IsTermEqual(*A.Term, *B.Term)

Declare.i AddTerm(*A.Term, *B.Term)
Declare.i MultiplyTerm(*A.Term, *B.Term)

Declare.i RestFactors(*AllFactors.Queue, *Factors.Queue)
Declare.i CopyFactors(*Factors.Queue)

Declare.i SimplifyTermF(*Term.Term)
Declare.i SimplifyTerm(*Term.Term)

Declare.s FormatTerm(*Term.Term, *Parent.Term = #Null)



;*******************************************************
; Variables
;*******************************************************

; DLL:
Prototype.i StringCallback(String.s, *Result)
Global DLLString.s
Global StringCallback.StringCallback

ProcedureDLL.i AttachProcess(Instance.i)
  CompilerIf #EnableDLL = #False
    CompilerIf #PB_Compiler_Debugger = #False
      SetCurrentDirectory(GetPathPart(ProgramFilename()))
    CompilerElse
      SetCurrentDirectory(#PB_Compiler_FilePath)
    CompilerEndIf
  CompilerEndIf
  
  IncludeFile "Sources\Initializations.pbi"
EndProcedure

CompilerIf #EnableDLL = #False
  AttachProcess(0)
CompilerEndIf



;*******************************************************
; Sources
;*******************************************************

IncludeFile "Sources\Managment.pbi"
IncludeFile "Sources\Level.pbi"
IncludeFile "Sources\IDs.pbi"
IncludeFile "Sources\Value.pbi"
IncludeFile "Sources\Operations.pbi"
IncludeFile "Sources\Object.pbi"
IncludeFile "Sources\Token.pbi"
IncludeFile "Sources\Parser.pbi"
IncludeFile "Sources\Core.pbi"
IncludeFile "Sources\Executers.pbi"
IncludeFile "Sources\Statements.pbi"
IncludeFile "Sources\Library.pbi"
IncludeFile "Sources\Classes.pbi"
IncludeFile "Sources\Interface.pbi"
IncludeFile "Sources\Hilighter.pbi"
IncludeFile "Sources\GUI.pbi"
IncludeFile "Sources\Viewer.pbi"
IncludeFile "Sources\Misc.pbi"
IncludeFile "Sources\HTMLGenerator.pbi"
IncludeFile "Sources\Tutorial.pbi"

IncludeFile "Sources\Term.pb"

IncludeFile "Sources\Exports.pbi"
IncludeFile "Sources\FunctionTable.pbi"



;*******************************************************
; Main
;*******************************************************

CompilerIf #EnableDLL = #False
  Global Parameter.s
  
  ; Commandline functions:
  Parameter = ProgramParameter(0)
  Select Parameter
    Case ""
      OpenPreview()
      OpenGUI()
    Case "cns"
      OpenUI()
    Case "compile"
      ; Generate a binary token-file:
      ParseFile(ProgramParameter(1), Main\TokenQueue) : OutputE()
      If ProgramParameter(2)
        SaveTokenQueue(ProgramParameter(2), Main\TokenQueue)
      Else
        SaveTokenQueue(StringField(ProgramParameter(1), 1, ".") + ".otc", Main\TokenQueue)
      EndIf
    Case "generatehelp"
      ; Generate a syntax-hilighted HTML-file:
      If ProgramParameter(2)
        GenerateHTMLFile(ProgramParameter(1), ProgramParameter(2))
      Else
        GenerateHTMLFile(ProgramParameter(1), StringField(ProgramParameter(1), 1, ".") + ".html")
      EndIf
    Case "nopreview"
      ; Open the GUI without a preview:
      OpenGUI()
    Default
      ; Execute the given file:
      ExecuteFile(Parameter)
      ; Ask for the IDE, if an error an not an "end" occurred:
      If Not Truncation And (GUI Or Console Or Error)
        OutputE()
        ; Print only a horinzontal line, if any output occurred!
        OutputHL()
        If Console
          OpenUI()
        Else
          OpenGUI()
        EndIf
      EndIf
  EndSelect
  
  SaveOptions()
  End
CompilerEndIf



;*******************************************************
; Data sections
;*******************************************************

DataSection
  Keywords:
    Data.s "def"
    Data.s "defined?"
    Data.s "del"
    Data.s "pop"
    Data.s "if", "else", "elseIf"
    Data.s "while"
    Data.s "repeat"
    Data.s "for"
    Data.s "switch"
    Data.s "and", "or", "xor", "not"
    Data.s "break"
    Data.s "return"
    Data.s "out"
    Data.s "exit"
  EndKeywords:
  
  ClassMethods:
    Data.s _CM(new)
    Data.s _CM(def)
    Data.s _CM(del)
    Data.s _CM(true?)
    Data.s _CM(set), _CM(fSet)
    Data.s _CM(add), _CM(fAdd)
    Data.s _CM(sub), _CM(neg), _CM(fSub)
    Data.s _CM(mul), _CM(fMul)
    Data.s _CM(div), _CM(fDiv)
    Data.s _CM(pow), _CM(fPow)
    Data.s _CM(add!), _CM(fAdd!)
    Data.s _CM(sub!), _CM(fSub!)
    Data.s _CM(mul!), _CM(fMul!)
    Data.s _CM(div!), _CM(fDiv!)
    Data.s _CM(eq)
    Data.s _CM(le), _CM(fLe)
    Data.s _CM(gr), _CM(fGr)
    Data.s _CM(inc), _CM(dec)
    Data.s _CM(form), _CM(args), _CM(vec), _CM(it)
    Data.s _CM(fileRead), _CM(fileWrite)
  EndClassMethods:
  
  Operators:
    Data.s "="
    Data.s "+", "-"
    Data.s "*", "/"
    Data.s "**", "//"
    Data.s "+=", "-=", "*=", "/="
    Data.s "==", "<", ">"
    Data.s "<>", "<=", ">="
    Data.s "++", "--"
    Data.s "<<", ">>"
    Data.s "->"
  EndOperators:
  OperatorFunctions:
    Data.i #ClassMethod_Set
    Data.i #ClassMethod_Add, @SubtractOperator()
    Data.i #ClassMethod_Multiply, #ClassMethod_Divide
    Data.i #ClassMethod_Power, @RootOperator()
    Data.i #ClassMethod_DoAdd, #ClassMethod_DoSubtract, #ClassMethod_DoMultiply, #ClassMethod_DoDivide
    Data.i #ClassMethod_Equal, #ClassMethod_Less, #ClassMethod_Greater
    Data.i @NotEqualOperator(), @LessEqualOperator(), @GreaterEqualOperator()
    Data.i @IncrementOperator(), @DecrementOperator()
    Data.i @FormatOperator(), @InputOperator()
    Data.i #None
  EndOperatorFunctions:
  OperatorMethods:
    Data.i #ClassMethod_Set
    Data.i #ClassMethod_Add, #ClassMethod_Subtract
    Data.i #ClassMethod_Multiply, #ClassMethod_Divide
    Data.i #ClassMethod_Power, #None
    Data.i #ClassMethod_DoAdd, #ClassMethod_DoSubtract, #ClassMethod_DoMultiply, #ClassMethod_DoDivide
    Data.i #ClassMethod_Equal, #ClassMethod_Less, #ClassMethod_Greater
    Data.i #None, #None, #None
    Data.i #ClassMethod_Increment, #ClassMethod_Decrement
    Data.i #ClassMethod_Format, #None
    Data.i #None
  EndOperatorMethods:
  OperatorFriendMethods:
    Data.i #ClassMethod_FriendSet
    Data.i #ClassMethod_FriendAdd, #ClassMethod_FriendSubtract
    Data.i #ClassMethod_FriendMultiply, #ClassMethod_FriendPower
    Data.i #None, #None, #None, #None
    Data.i #ClassMethod_Equal, #ClassMethod_FriendLess, #ClassMethod_FriendGreater
    Data.i #None, #None
    Data.i #None, #None
    Data.i #None
  EndOperatorFriendMethods:
  OperatorPriorities:
    Data.i 2          ;"="
    Data.i 5, 5       ;"+" , "-"
    Data.i 6, 6       ;"*" , "/"
    Data.i 7, 7       ;"**", "//"
    Data.i 2, 2, 2, 2 ;"+=", "-=", "*=", "/="
    Data.i 1, 1, 1    ;"==", "<" , ">"
    Data.i 1, 1, 1    ;"<>", "<=", ">="
    Data.i 8, 8       ;"++", "--"
    Data.i 4, 9       ;"<<", ">>"
    Data.i 0          ;"->"
    
    ;Die 3 wird ausgelassen, weil manche Schlsselwrter ber
    ;diese Prioritt verfgen.
  EndOperatorPriorities:
  OperatorLeftValues:
    Data.i 1          ;"="
    Data.i 1, 1       ;"+" , "-"
    Data.i 1, 1       ;"*" , "/"
    Data.i 1, 1       ;"**", "//"
    Data.i 1, 1, 1, 1 ;"+=", "-=", "*=", "/="
    Data.i 1, 1, 1    ;"==", "<" , ">"
    Data.i 1, 1, 1    ;"<>", "<=", ">="
    Data.i 0, 0       ;"++", "--"
    Data.i 1, 0       ;"<<", ">>"
    Data.i 1          ;"->"
  EndOperatorLeftValues:
  OperatorCompare:
    Data.i 0          ;"="
    Data.i 0, 0       ;"+" , "-"
    Data.i 0, 0       ;"*" , "/"
    Data.i 0, 0       ;"**", "//"
    Data.i 0, 0, 0, 0 ;"+=", "-=", "*=", "/="
    Data.i 1, 1, 1    ;"==", "<" , ">"
    Data.i 1, 1, 1    ;"<>", "<=", ">="
    Data.i 0, 0       ;"++", "--"
    Data.i 0, 0       ;"<<", ">>"
    Data.i 0          ;"->"
  EndOperatorCompare:
  
  Brackets:
    Data.s "("
    Data.s "["
    Data.s "{"
  EndBrackets:
  ClosingBrackets:
    Data.s ")"
    Data.s "]"
    Data.s "}"
  EndClosingBrackets:
  
  Symbols:
    Data.s ""    ;None
    Data.s " > " ;Default
    Data.s "..." ;etc
    Data.s " - " ;Point
    Data.s ">>>" ;Output
    Data.s "<<<" ;Input
    Data.s "<!>" ;Error
    Data.s "-->" ;Message
  EndSymbols:
  
  HTMLGenerator_StandardTags:
    ; Default:
    Data.i #False
    Data.i #False
    Data.i #False
    Data.i $000000
    ; Number:
    Data.i #False
    Data.i #False
    Data.i #False
    Data.i $808000
    ; String:
    Data.i #False
    Data.i #False
    Data.i #False
    Data.i $008000
    ; Operator:
    Data.i #False
    Data.i #False
    Data.i #False
    Data.i $0000FF
    ; Brackets:
    Data.i #True
    Data.i #False
    Data.i #False
    Data.i $000000
    ; Control:
    Data.i #False
    Data.i #False
    Data.i #False
    Data.i $000000
    ; Seperator
    Data.i #True
    Data.i #False
    Data.i #False
    Data.i $000000
    ; Keyword:
    Data.i #True
    Data.i #False
    Data.i #False
    Data.i $FF0000
    ; Symbol:
    Data.i #False
    Data.i #False
    Data.i #False
    Data.i $0080FF
    ; Comment:
    Data.i #False
    Data.i #True
    Data.i #False
    Data.i $808080
  EndHTMLGenerator_StandardTags:
  HTMLGenerator_TagNames:
    Data.s "Default"
    Data.s "Number"
    Data.s "String"
    Data.s "Operator"
    Data.s "Brackets"
    Data.s "Control"
    Data.s "Seperator"
    Data.s "Keyword"
    Data.s "Class method"
    Data.s "Symbol"
    Data.s "Comment"
  EndHTMLGenerator_TagNames:
  HTMLGenerator_Examples:
    Data.s "any text"
    Data.s "123.6"
    Data.s #Q + "Hello!" + #Q
    Data.s "+ - * /"
    Data.s "() [] {}"
    Data.s "\ : . @"
    Data.s "; ,"
    Data.s "define"
    Data.s "_new"
    Data.s "#symbol"
    Data.s "'Ignored"
  EndHTMLGenerator_Examples:
EndDataSection
; IDE Options = PureBasic 4.41 (Windows - x86)
; CursorPosition = 464
; FirstLine = 358
; Folding = BMAIA9
; EnableXP
; UseIcon = ..\..\..\Bilder\Icons\Operno 5-00.ico
; Executable = Operno.exe
; CurrentDirectory = ..\Operno 6\
; CompileSourceDirectory