;**************************************************************
;     Block (class)
;
; Autohr: Josef Sniatecki
; Creation date: 12.04.2009
;**************************************************************
;   Copyright (c) 2009 Josef Sniatecki
;
; This program is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License
; As published by the Free Software Foundation; either version
; 3 of the License, Or (at your option) any later version.
;
; This program is distributed in the hope that it will be
; useful, but WITHOUT ANY WARRANTY; without even the implied
; warranty of MERCHANTABILITY Or FITNESS For A PARTICULAR
; PURPOSE. See the GNU General Public License For more details.
;
; You should have received a copy of the GNU General Public
; License along With this program;
; If Not, see "http://www.gnu.org/licenses/".
;**************************************************************



Procedure.i MultiplyBlock(*Block.Queue, Times.i)
  Protected *Result.Queue, Index.i
  
  *Result = CreateQueue(@FreeToken())
  If Times = 0
    AddIntegerToken(*Result, #Token_Seperator, #Seperator_Block)
  Else
    For Index = 1 To Times
      If Index > 1
        *Result\Last\Token\Seperator = #Seperator_Command
      EndIf
      CopyQueue(*Block, *Result, @CopyToken(), *Result\Last)
    Next
  EndIf
  ProcedureReturn PushBlockValue(*Result)
EndProcedure

Procedure.i Std_Block_Call()
  Protected *Result.Value
  
  *Current\Super = *Current\Previous\Super
  
  OpenLevel(*Current\This\Queue, 0, 0, 0, #True)
    *Result = RunBlock()
  CloseLevel()
  
  ProcedureReturn *Result
EndProcedure
Procedure.i Std_Block_CallIn()
  Protected *Object.Object, *Result.Value
  
  If PopPValue(@*Object)
    *Current\Super = *Current\Previous\Super
    
    OpenLevel(*Current\This\Queue, 0, *Object, 0, #True)
      *Result = RunBlock()
    CloseLevel()
    
    ProcedureReturn *Result
  Else
    _ErrorP(1)
  EndIf
EndProcedure
Procedure.i Std_Block_Tokens()
  Protected *Item.QueueItem, *Result.Queue
  
  *Result = CreateQueue(@FreeValue())
  *Item   = *Current\This\Queue\First
  While *Item
    AppendQueueItem(*Result, CreateTokenValue(*Item))
    _Move(*Item, N)
  Wend
  
  ProcedureReturn PushQueueValue(*Result)
EndProcedure
Procedure.i Std_Block_Size()
  ProcedureReturn PushIValue(*Current\This\Queue\Count-1)
EndProcedure
Procedure.i Std_Block__New()
  *Current\This\Queue = CreateQueue(@FreeToken())
  *Current\This\Scope | #Value_Binary
  AddIntegerToken(*Current\This\Queue, #Token_Seperator, #Seperator_Block)
EndProcedure
Procedure.i Std_Block__Del()
  FreeQueue(*Current\This\Queue)
EndProcedure
Procedure.i Std_Block__IsTrue()
  If *Current\This\Queue\Count > 1
    ProcedureReturn PushBValue(#True)
  Else
    ProcedureReturn PushBValue(#False)
  EndIf
EndProcedure
Procedure.i Std_Block__Set()
  Protected *Value.Value, *ThisQueue.Queue = *Current\This\Queue
  
  PopValueAddress(@*Value)
  Select *Value\Class
    Case *BlockClass
      ClearQueue(*ThisQueue)
      If *Value\Queue\Count > 1
        CopyQueue(*Value\Queue, *ThisQueue, @CopyToken())
      EndIf
    Case *StringClass
      ClearQueue(*ThisQueue)
      Parse(*ThisQueue, @*Value\String\S) : _E()
      *ThisQueue\Last\Token\Seperator = #Seperator_Block
    Case *PointerClass, *ObjectClass
      If *Value\Object\Type = #Object_Function
        ClearQueue(*ThisQueue)
        AddValueToken(*ThisQueue, #Token_Value, CreatePValue(*Value\Object))
        AddIntegerToken(*ThisQueue, #Token_Control, #Control_Next)
        AddIntegerToken(*ThisQueue, #Token_Seperator, #Seperator_Block)
      EndIf
  EndSelect
  ProcedureReturn *Value
EndProcedure
Procedure.i Std_Block__Add()
  Protected *Value.Value, *Result.Queue
  
  PopValueAddress(@*Value)
  Select *Value\Class
    Case *BlockClass
      ; Copy the current block:
      *Result = CreateQueue(@FreeToken())
      CopyQueue(*Current\This\Queue, *Result, @CopyToken())
      If *Result\Count > 1
        *Result\Last\Token\Seperator = #Seperator_Command
      Else
        FreeLastQueueItem(*Current\This\Queue)
      EndIf
      ; Copy the second block into the current copy:
      CopyQueue(*Value\Queue, *Result, @CopyToken(), *Result\Last)
      ProcedureReturn PushBlockValue(*Result)
    Case *StringClass
      ; Copy the current block:
      *Result = CreateQueue(@FreeToken())
      CopyQueue(*Current\This\Queue, *Result, @CopyToken())
      If *Result\Count > 1
        *Result\Last\Token\Seperator = #Seperator_Command
      Else
        FreeLastQueueItem(*Current\This\Queue)
      EndIf
      ; Parse the code direct to the current copy:
      Parse(*Result, @*Value\String\S) : _E("", FreeQueue(*Result))
      *Result\Last\Token\Seperator = #Seperator_Block
      ProcedureReturn PushBlockValue(*Result)
  EndSelect
EndProcedure
Procedure.i Std_Block__Mul()
  Protected *Value.Value, *Result.Value, Index.i
  
  PopValueAddress(@*Value)
  Select *Value\Class
    Case *IntegerClass
      ProcedureReturn MultiplyBlock(*Current\This\Queue, *Value\Integer\I)
  EndSelect
EndProcedure
Procedure.i Std_Block__DoAdd()
  Protected *Value.Value, *TokenQueue.Queue = *Current\This\Queue
  
  PopValueAddress(@*Value)
  Select *Value\Class
    Case *BlockClass
      ; Copy the second block into the current copy:
      If *TokenQueue\Count > 1
        *TokenQueue\Last\Token\Seperator = #Seperator_Command
      EndIf
      CopyQueue(*Value\Queue, *TokenQueue, @CopyToken(), *TokenQueue\Last)
      ProcedureReturn *Value
    Case *StringClass
      ; Copy the current block:
      If *TokenQueue\Count > 1
        *TokenQueue\Last\Token\Seperator = #Seperator_Command
      EndIf
      ; Parse the code direct to the current copy:
      Parse(*TokenQueue, @*Value\String\S) : _E()
      *TokenQueue\Last\Token\Seperator = #Seperator_Block
      ProcedureReturn *Value
  EndSelect
EndProcedure
Procedure.i Std_Block__Form()
  Protected String.s
  
  If *Current\This\Queue\Count > 1
    
    String = FormatTokenQueue(*Current\This\Queue, 1)
    If FindString(String, #N, 1)
      ProcedureReturn PushSValue("{" + #N + Space(#IndentLength) + String + #N + "}")
    Else
      ProcedureReturn PushSValue(String)
    EndIf
  Else
    ProcedureReturn PushSValue("{}")
  EndIf
EndProcedure



DataSection
  BlockClass:
    Data.c #Object_Function : Data.s "call" ;{
      Data.i @Std_Block_Call()
    ;}
    Data.c #Object_Function : Data.s "callIn" ;{
      Data.i @Std_Block_CallIn()
    ;}
    Data.c #Object_Function : Data.s "tokens" ;{
      Data.i @Std_Block_Tokens()
    ;}
    Data.c #Object_Function : Data.s "size" ;{
      Data.i @Std_Block_Size()
    ;}
    Data.c #Object_Function : Data.s _CM(new) ;{
      Data.i @Std_Block__New()
    ;}
    Data.c #Object_Function : Data.s _CM(del) ;{
      Data.i @Std_Block__Del()
    ;}
    Data.c #Object_Function : Data.s _CM(true?) ;{
      Data.i @Std_Block__IsTrue()
    ;}
    Data.c #Object_Function : Data.s _CM(set) ;{
      Data.i @Std_Block__Set()
    ;}
    Data.c #Object_Function : Data.s _CM(add) ;{
      Data.i @Std_Block__Add()
    ;}
    Data.c #Object_Function : Data.s _CM(mul) ;{
      Data.i @Std_Block__Mul()
    ;}
    Data.c #Object_Function : Data.s _CM(add!) ;{
      Data.i @Std_Block__DoAdd()
    ;}
    Data.c #Object_Function : Data.s _CM(form) ;{
      Data.i @Std_Block__Form()
    ;}
    Data.c #Object_Function : Data.s _CM(it) ;{
      Data.i @Std_Block_Tokens()
    ;}
    Data.c #False
EndDataSection
; IDE Options = PureBasic 4.41 (Windows - x86)
; CursorPosition = 242
; FirstLine = 207
; Folding = -----
; EnableXP
; CompileSourceDirectory