;**************************************************************
;     Quad (class)
;
; Autor: Josef Sniatecki
; Creation date: 08.04.2009
;**************************************************************
;   (c) 2009 Josef Sniatecki
;
; This program is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License
; As published by the Free Software Foundation; either version
; 3 of the License, Or (at your option) any later version.
;
; This program is distributed in the hope that it will be
; useful, but WITHOUT ANY WARRANTY; without even the implied
; warranty of MERCHANTABILITY Or FITNESS For A PARTICULAR
; PURPOSE. See the GNU General Public License For more details.
;
; You should have received a copy of the GNU General Public
; License along With this program;
; If Not, see "http://www.gnu.org/licenses/".
;**************************************************************



Procedure.i Std_Quad_Abs()
  If *Current\This\Quad\Q < 0
    ProcedureReturn PushQValue(-*Current\This\Quad\Q)
  Else
    ProcedureReturn PushQValue(*Current\This\Quad\Q)
  EndIf
EndProcedure
Procedure.i Std_Quad_DoAbs()
  If *Current\This\Quad\Q < 0
    _Inv(*Current\This\Quad\Q)
  EndIf
  ProcedureReturn *Current\This
EndProcedure
Procedure.i Std_Quad__New()
  *Current\This\Quad  = _New(Quad)
  *Current\This\Scope | #Value_Binary
EndProcedure
Procedure.i Std_Quad__Def()
  PopQValue(*Current\This\Quad)
EndProcedure
Procedure.i Std_Quad__Del()
  FreeMemory(*Current\This\Memory)
EndProcedure
Procedure.i Std_Quad__IsTrue()
  If *Current\This\Quad\Q
    ProcedureReturn PushBValue(#True)
  Else
    ProcedureReturn PushBValue(#False)
  EndIf
EndProcedure
Procedure.i Std_Quad__Set()
  Protected *Value.Value
  
  PopValueAddress(@*Value)
  Select *Value\Class
    Case *IntegerClass, *BooleanClass
      *Current\This\Quad\Q = *Value\Integer\I
    Case *FloatClass
      *Current\This\Quad\Q = *Value\Float\F
    Case *QuadClass
      *Current\This\Quad\Q = *Value\Quad\Q
    Case *DoubleClass
      *Current\This\Quad\Q = *Value\Double\D
    Case *StringClass
      *Current\This\Quad\Q = Val(*Value\String\S)
  EndSelect
  ProcedureReturn *Value
EndProcedure
Procedure.i Std_Quad__Add()
  Protected *Value.Value
  
  PopValueAddress(@*Value)
  Select *Value\Class
    Case *IntegerClass, *BooleanClass
      ProcedureReturn PushQValue(*Current\This\Quad\Q + *Value\Integer\I)
    Case *FloatClass
      ProcedureReturn PushQValue(*Current\This\Quad\Q + *Value\Float\F)
    Case *QuadClass
      ProcedureReturn PushQValue(*Current\This\Quad\Q + *Value\Quad\Q)
    Case *DoubleClass
      ProcedureReturn PushDValue(*Current\This\Quad\Q + *Value\Double\D)
    Case *StringClass
      ProcedureReturn PushSValue(Space(*Current\This\Quad\Q) + *Value\String\S)
  EndSelect
EndProcedure
Procedure.i Std_Quad__Sub()
  Protected *Value.Value
  
  PopValueAddress(@*Value)
  Select *Value\Class
    Case *IntegerClass, *BooleanClass
      ProcedureReturn PushQValue(*Current\This\Quad\Q - *Value\Integer\I)
    Case *FloatClass
      ProcedureReturn PushQValue(*Current\This\Quad\Q - *Value\Float\F)
    Case *QuadClass
      ProcedureReturn PushQValue(*Current\This\Quad\Q - *Value\Quad\Q)
    Case *DoubleClass
      ProcedureReturn PushDValue(*Current\This\Quad\Q - *Value\Double\D)
  EndSelect
EndProcedure
Procedure.i Std_Quad__Neg()
  ProcedureReturn PushFValue(-*Current\This\Quad\Q)
EndProcedure
Procedure.i Std_Quad__Mul()
  Protected *Value.Value
  
  PopValueAddress(@*Value)
  Select *Value\Class
    Case *IntegerClass, *BooleanClass
      ProcedureReturn PushQValue(*Current\This\Quad\Q * *Value\Integer\I)
    Case *FloatClass
      ProcedureReturn PushQValue(*Current\This\Quad\Q * *Value\Float\F)
    Case *QuadClass
      ProcedureReturn PushQValue(*Current\This\Quad\Q * *Value\Quad\Q)
    Case *DoubleClass
      ProcedureReturn PushDValue(*Current\This\Quad\Q * *Value\Double\D)
    Case *StringClass
      ProcedureReturn PushSValue(MStr(*Value\String\S, *Current\This\Quad\Q))
  EndSelect
EndProcedure
Procedure.i Std_Quad__Div()
  Protected *Value.Value
  
  PopValueAddress(@*Value)
  Select *Value\Class
    Case *IntegerClass
      ProcedureReturn PushQValue(*Current\This\Quad\Q / *Value\Integer\I)
    Case *FloatClass
      ProcedureReturn PushQValue(*Current\This\Quad\Q / *Value\Float\F)
    Case *QuadClass
      ProcedureReturn PushQValue(*Current\This\Quad\Q / *Value\Quad\Q)
    Case *DoubleClass
      ProcedureReturn PushDValue(*Current\This\Quad\Q / *Value\Double\D)
  EndSelect
EndProcedure
Procedure.i Std_Quad__Pow()
  Protected *Value.Value
  
  PopValueAddress(@*Value)
  Select *Value\Class
    Case *IntegerClass
      ProcedureReturn PushQValue(Pow(*Current\This\Quad\Q, *Value\Integer\I))
    Case *FloatClass
      ProcedureReturn PushQValue(Pow(*Current\This\Quad\Q, *Value\Float\F))
    Case *QuadClass
      ProcedureReturn PushQValue(Pow(*Current\This\Quad\Q, *Value\Quad\Q))
    Case *DoubleClass
      ProcedureReturn PushDValue(Pow(*Current\This\Quad\Q, *Value\Double\D))
  EndSelect
EndProcedure
Procedure.i Std_Quad__DoAdd()
  Protected *Value.Value
  
  PopValueAddress(@*Value)
  Select *Value\Class
    Case *IntegerClass, *BooleanClass
      *Current\This\Quad\Q + *Value\Integer\I
    Case *FloatClass
      *Current\This\Quad\Q + *Value\Float\F
    Case *QuadClass
      *Current\This\Quad\Q + *Value\Quad\Q
    Case *DoubleClass
      *Current\This\Quad\Q + *Value\Double\D
  EndSelect
  ProcedureReturn *Value
EndProcedure
Procedure.i Std_Quad__DoSub()
  Protected *Value.Value
  
  PopValueAddress(@*Value)
  Select *Value\Class
    Case *IntegerClass, *BooleanClass
      *Current\This\Quad\Q - *Value\Integer\I
    Case *FloatClass
      *Current\This\Quad\Q - *Value\Float\F
    Case *QuadClass
      *Current\This\Quad\Q - *Value\Quad\Q
    Case *DoubleClass
      *Current\This\Quad\Q - *Value\Double\D
  EndSelect
  ProcedureReturn *Value
EndProcedure
Procedure.i Std_Quad__DoMul()
  Protected *Value.Value
  
  PopValueAddress(@*Value)
  Select *Value\Class
    Case *IntegerClass, *BooleanClass
      *Current\This\Quad\Q * *Value\Integer\I
    Case *FloatClass
      *Current\This\Quad\Q * *Value\Float\F
    Case *QuadClass
      *Current\This\Quad\Q * *Value\Quad\Q
    Case *DoubleClass
      *Current\This\Quad\Q * *Value\Double\D
  EndSelect
  ProcedureReturn *Value
EndProcedure
Procedure.i Std_Quad__DoDiv()
  Protected *Value.Value
  
  PopValueAddress(@*Value)
  Select *Value\Class
    Case *IntegerClass, *BooleanClass
      *Current\This\Quad\Q / *Value\Integer\I
    Case *FloatClass
      *Current\This\Quad\Q / *Value\Float\F
    Case *QuadClass
      *Current\This\Quad\Q / *Value\Quad\Q
    Case *DoubleClass
      *Current\This\Quad\Q / *Value\Double\D
  EndSelect
  ProcedureReturn *Value
EndProcedure
Procedure.i Std_Quad__Eq()
  Protected *Value.Value
  
  PopValueAddress(@*Value)
  Select *Value\Class
    Case *IntegerClass, *BooleanClass
      If *Current\This\Quad\Q = *Value\Integer\I
        ProcedureReturn PushBValue(#True)
      Else
        ProcedureReturn PushBValue(#False)
      EndIf
    Case *FloatClass
      If *Current\This\Quad\Q = *Value\Float\F
        ProcedureReturn PushBValue(#True)
      Else
        ProcedureReturn PushBValue(#False)
      EndIf
    Case *QuadClass
      If *Current\This\Quad\Q = *Value\Quad\Q
        ProcedureReturn PushBValue(#True)
      Else
        ProcedureReturn PushBValue(#False)
      EndIf
    Case *DoubleClass
      If *Current\This\Float\F = *Value\Double\D
        ProcedureReturn PushBValue(#True)
      Else
        ProcedureReturn PushBValue(#False)
      EndIf
  EndSelect
EndProcedure
Procedure.i Std_Quad__Le()
  Protected *Value.Value
  
  PopValueAddress(@*Value)
  Select *Value\Class
    Case *IntegerClass, *BooleanClass
      If *Current\This\Quad\Q < *Value\Integer\I
        ProcedureReturn PushBValue(#True)
      Else
        ProcedureReturn PushBValue(#False)
      EndIf
    Case *FloatClass
      If *Current\This\Quad\Q < *Value\Float\F
        ProcedureReturn PushBValue(#True)
      Else
        ProcedureReturn PushBValue(#False)
      EndIf
    Case *QuadClass
      If *Current\This\Quad\Q < *Value\Quad\Q
        ProcedureReturn PushBValue(#True)
      Else
        ProcedureReturn PushBValue(#False)
      EndIf
    Case *DoubleClass
      If *Current\This\Quad\Q < *Value\Double\D
        ProcedureReturn PushBValue(#True)
      Else
        ProcedureReturn PushBValue(#False)
      EndIf
  EndSelect
EndProcedure
Procedure.i Std_Quad__Gr()
  Protected *Value.Value
  
  PopValueAddress(@*Value)
  Select *Value\Class
    Case *IntegerClass, *BooleanClass
      If *Current\This\Quad\Q > *Value\Integer\I
        ProcedureReturn PushBValue(#True)
      Else
        ProcedureReturn PushBValue(#False)
      EndIf
    Case *FloatClass
      If *Current\This\Quad\Q > *Value\Float\F
        ProcedureReturn PushBValue(#True)
      Else
        ProcedureReturn PushBValue(#False)
      EndIf
    Case *QuadClass
      If *Current\This\Quad\Q > *Value\Quad\Q
        ProcedureReturn PushBValue(#True)
      Else
        ProcedureReturn PushBValue(#False)
      EndIf
    Case *DoubleClass
      If *Current\This\Quad\Q > *Value\Double\D
        ProcedureReturn PushBValue(#True)
      Else
        ProcedureReturn PushBValue(#False)
      EndIf
  EndSelect
EndProcedure
Procedure.i Std_Quad__Inc()
  *Current\This\Quad\Q + 1
  ProcedureReturn *Current\This
EndProcedure
Procedure.i Std_Quad__Dec()
  *Current\This\Quad\Q - 1
  ProcedureReturn *Current\This
EndProcedure
Procedure.i Std_Quad__Form()
  ProcedureReturn PushSValue(Str(*Current\This\Quad\Q))
EndProcedure

DataSection
  QuadClass:
    Data.c #Object_Function : Data.s "abs" ;{
      Data.i @Std_Quad_Abs()
    ;}
    Data.c #Object_Function : Data.s "abs!" ;{
      Data.i @Std_Quad_DoAbs()
    ;}
    Data.c #Object_Function : Data.s _CM(new) ;{
      Data.i @Std_Quad__New()
    ;}
    Data.c #Object_Function : Data.s _CM(def) ;{
      Data.i @Std_Quad__Def()
    ;}
    Data.c #Object_Function : Data.s _CM(del) ;{
      Data.i @Std_Quad__Del()
    ;}
    Data.c #Object_Function : Data.s _CM(true?) ;{
      Data.i @Std_Quad__IsTrue()
    ;}
    Data.c #Object_Function : Data.s _CM(set) ;{
      Data.i @Std_Quad__Set()
    ;}
    Data.c #Object_Function : Data.s _CM(add) ;{
      Data.i @Std_Quad__Add()
    ;}
    Data.c #Object_Function : Data.s _CM(sub) ;{
      Data.i @Std_Quad__Sub()
    ;}
    Data.c #Object_Function : Data.s _CM(neg) ;{
      Data.i @Std_Quad__Neg()
    Data.s "negative"
    ;}
    Data.c #Object_Function : Data.s _CM(mul) ;{
      Data.i @Std_Quad__Mul()
    ;}
    Data.c #Object_Function : Data.s _CM(div) ;{
      Data.i @Std_Quad__Div()
    ;}
    Data.c #Object_Function : Data.s _CM(pow) ;{
      Data.i @Std_Quad__Pow()
    ;}
    Data.c #Object_Function : Data.s _CM(add!) ;{
      Data.i @Std_Quad__DoAdd()
    ;}
    Data.c #Object_Function : Data.s _CM(sub!) ;{
      Data.i @Std_Quad__DoSub()
    ;}
    Data.c #Object_Function : Data.s _CM(mul!) ;{
      Data.i @Std_Quad__DoMul()
    ;}
    Data.c #Object_Function : Data.s _CM(div!) ;{
      Data.i @Std_Quad__DoDiv()
    ;}
    Data.c #Object_Function : Data.s _CM(eq) ;{
      Data.i @Std_Quad__Eq()
    ;}
    Data.c #Object_Function : Data.s _CM(le) ;{
      Data.i @Std_Quad__Le()
    ;}
    Data.c #Object_Function : Data.s _CM(gr) ;{
      Data.i @Std_Quad__Gr()
    ;}
    Data.c #Object_Function : Data.s _CM(inc) ;{
      Data.i @Std_Quad__Inc()
    ;}
    Data.c #Object_Function : Data.s _CM(dec) ;{
      Data.i @Std_Quad__Dec()
    ;}
    Data.c #Object_Function : Data.s _CM(form) ;{
      Data.i @Std_Quad__Form()
    ;}
    Data.c #False
EndDataSection
; IDE Options = PureBasic 4.40 (Windows - x86)
; CursorPosition = 1
; Folding = --------
; EnableXP