;**************************************************************
;     Token (class)
;
; Author: Josef Sniatecki
; Creation date: 21.02.2010
;**************************************************************
;   Copyright (c) 2010 Josef Sniatecki
;
; This program is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License
; As published by the Free Software Foundation; either version
; 3 of the License, Or (at your option) any later version.
;
; This program is distributed in the hope that it will be
; useful, but WITHOUT ANY WARRANTY; without even the implied
; warranty of MERCHANTABILITY Or FITNESS For A PARTICULAR
; PURPOSE. See the GNU General Public License For more details.
;
; You should have received a copy of the GNU General Public
; License along With this program;
; If Not, see "http://www.gnu.org/licenses/".
;**************************************************************



Procedure.i Std_Token_Type()
  If *Current\This\Item
    ProcedureReturn PushIValue(*Current\This\Item\Token\Type)
  Else
    _Error("Item is null.")
  EndIf
EndProcedure
Procedure.i Std_Token_Next()
  If *Current\This\Item
    ProcedureReturn PushTokenValue(*Current\This\Item\N)
  Else
    _Error("Item is null.")
  EndIf
EndProcedure
Procedure.i Std_Token_Previous()
  If *Current\This\Item
    ProcedureReturn PushTokenValue(*Current\This\Item\P)
  Else
    _Error("Item is null.")
  EndIf
EndProcedure
Procedure.i Std_Token_Indentifer()
  Protected *Item.QueueItem = *Current\This\Item
  
  If *Item
    If *Item\Token\Type = #Token_Name
      ProcedureReturn PushSValue(*Item\Token\Name)
    Else
      _Error("Only allowed for indentifers.")
    EndIf
  Else
    _Error("Item is null.")
  EndIf
EndProcedure
Procedure.i Std_Token_Value()
  Protected *Item.QueueItem = *Current\This\Item
  
  If *Item
    If *Item\Token\Type = #Token_Value
      ProcedureReturn *Item\Token\Value
    Else
      _Error("Only allowed for values.")
    EndIf
  Else
    _Error("Item is null.")
  EndIf
EndProcedure
Procedure.i Std_Token_Operator()
  Protected *Item.QueueItem = *Current\This\Item
  
  If *Item
    If *Item\Token\Type = #Token_Operator
      ProcedureReturn PushSValue(Operator(*Item\Token\Operator))
    Else
      _Error("Only allowed for operators.")
    EndIf
  Else
    _Error("Item is null.")
  EndIf
EndProcedure
Procedure.i Std_Token_Block()
  Protected *Item.QueueItem = *Current\This\Item
  
  If *Item
    If *Item\Token\Type = #Token_Block
      ProcedureReturn PushBlockValue(*Item\Token\Block\TokenQueue, #True)
    Else
      _Error("Only allowed for blocks!")
    EndIf
  Else
    _Error("Item is null!")
  EndIf
EndProcedure
Procedure.i Std_Token__New()
  *Current\This\Scope | #Value_Binary
EndProcedure
Procedure.i Std_Token__Def()
  Protected *Value.Value
  
  If PopValueAddress(@*Value)
    If *Value\Class = *BlockClass
      *Current\This\Item = *Value\Queue\First
    Else
      _Error("Only blocks are allowed!")
    EndIf
  Else
    _ErrorP(1)
  EndIf
EndProcedure
Procedure.i Std_Token__Set()
  Protected *Value.Value
  
  PopValueAddress(@*Value)
  Select *Value\Class
    Case *TokenClass
      *Current\This\Item = *Value\Item
    Default
      ProcedureReturn #Null
  EndSelect
  
  ProcedureReturn *Value
EndProcedure
Procedure.i Std_Token__IsTrue()
  If *Current\This\Item
    ProcedureReturn PushBValue(#True)
  Else
    ProcedureReturn PushBValue(#False)
  EndIf
EndProcedure
Procedure.i Std_Token__Inc()
  If *Current\This\Item
    *Current\This\Item = *Current\This\Item\N
    ProcedureReturn *Current\This
  Else
    _Error("Item is null.")
  EndIf
EndProcedure
Procedure.i Std_Token__Dec()
  If *Current\This\Item
    *Current\This\Item = *Current\This\Item\P
    ProcedureReturn *Current\This
  Else
    _Error("Item is null.")
  EndIf
EndProcedure
Procedure.i Std_Token__Form()
  If *Current\This\Item
    ProcedureReturn PushSValue(FormatToken(*Current\This\Item\Token))
  Else
    _Error("Item is null.")
  EndIf
EndProcedure



DataSection
  TokenClass:
    Data.c #Object_Function : Data.s "type" ;{
      Data.i @Std_Token_Type()
    ;}
    Data.c #Object_Function : Data.s "next" ;{
      Data.i @Std_Token_Next()
    ;}
    Data.c #Object_Function : Data.s "prev" ;{
      Data.i @Std_Token_Previous()
    ;}
    Data.c #Object_Function : Data.s "indentifer" ;{
      Data.i @Std_Token_Indentifer()
    ;}
    Data.c #Object_Function : Data.s "value" ;{
      Data.i @Std_Token_Value()
    ;}
    Data.c #Object_Function : Data.s "operator" ;{
      Data.i @Std_Token_Operator()
    ;}
    Data.c #Object_Function : Data.s "block" ;{
      Data.i @Std_Token_Block()
    ;}
     Data.c #Object_Function : Data.s _CM(new) ;{
      Data.i @Std_Token__New()
    ;}
    Data.c #Object_Function : Data.s _CM(def) ;{
      Data.i @Std_Token__Def()
    ;}
    Data.c #Object_Function : Data.s _CM(set) ;{
      Data.i @Std_Token__Set()
    ;}
    Data.c #Object_Function : Data.s _CM(true?) ;{
      Data.i @Std_Token__IsTrue()
    ;}
    Data.c #Object_Function : Data.s _CM(inc) ;{
      Data.i @Std_Token__Inc()
    ;}
    Data.c #Object_Function : Data.s _CM(dec) ;{
      Data.i @Std_Token__Dec()
    ;}
    Data.c #Object_Function : Data.s _CM(form) ;{
      Data.i @Std_Token__Form()
    ;}
    Data.c #Null
EndDataSection
; IDE Options = PureBasic 4.41 (Windows - x86)
; CursorPosition = 190
; FirstLine = 165
; Folding = -----
; EnableXP