;**************************************************************
;     Coordinate System
;
; Autor: Josef Sniatecki
; Creation date: 06.02.2010
;**************************************************************
;   (c) 2010 Josef Sniatecki
;
; This program is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License
; As published by the Free Software Foundation; either version
; 3 of the License, Or (at your option) any later version.
;
; This program is distributed in the hope that it will be
; useful, but WITHOUT ANY WARRANTY; without even the implied
; warranty of MERCHANTABILITY Or FITNESS For A PARTICULAR
; PURPOSE. See the GNU General Public License For more details.
;
; You should have received a copy of the GNU General Public
; License along With this program;
; If Not, see "http://www.gnu.org/licenses/".
;**************************************************************



Procedure.i CoordinateSystemDrawBackground(*System.Std_CoordinateSystem, BackgroundColor.i, AxeColor.i, GridColor.i)
  Protected Index.i, Length.i, K.f
  
  StartDrawing(ImageOutput(*System\Image))
    Box(0, 0, *System\Width, *System\Height, BackgroundColor)
    
    Length = *System\Width / *System\XFactor
    For Index = -Length To Length
      K = (-*System\MinX + Index) * *System\XFactor
      Line(K, 0, 1, *System\Height, GridColor)
    Next
    Length = *System\Height / *System\YFactor
    For Index = -Length To Length
      K = (-*System\MinY + Index) * *System\YFactor
      Line(0, K, *System\Width, 1, GridColor)
    Next
    
    Line(0, -*System\MinY * *System\YFactor, *System\Width, 1, AxeColor)
    Line(-*System\MinX * *System\XFactor, 0, 1, *System\Height, AxeColor)
  StopDrawing()
EndProcedure
Procedure.i CoordinateSystemFactors(*System.Std_CoordinateSystem)
  *System\XFactor = *System\Width /(*System\MaxX - *System\MinX)
  *System\YFactor = *System\Height/(*System\MaxY - *System\MinY)
EndProcedure
Procedure.i CoordinateSystemDrawGraph(*System.Std_CoordinateSystem, *Function.Queue, Color.i, Thickness.i)
  Protected X.i, Y.i, YFloat.f, PreviousX.i, PreviousY.i, *Value.Value
  Protected DC.i, Pen.i, PreviousPen.i
  Protected Steps.i
  
  Protected *Object.Object, *XVariable.Object, *X.Float

  *Object = CreateGroup("_graph")
    *XVariable = CreateVariable("x", CreateFValue(0), *Object)
    *X         = *XVariable\Variable\Value\Float
  
  
  ; Start drawing:
  OpenLevel(*Function, 0, *Object, 0)
  
  DC = StartDrawing(ImageOutput(*System\Image))
    If Thickness
      Pen   = CreatePen_(#PS_SOLID, Thickness, Color)
      Steps = 1
    Else
      Pen   = CreatePen_(#PS_DOT, 1, Color)
      Steps = 5
    EndIf
    PreviousPen = SelectObject_(DC, Pen)
    
    While X < *System\Width
      *X\F   = *System\MinX + X/*System\XFactor
      *Value = Run() : _EB()
      If *Value
        YFloat = ValueF(*Value) : _EB()
      Else
        _ErrorB("Invalid function used for plotting.")
      EndIf
      ClearStack()
      
      *Current\TokenItem = *Current\TokenQueue\First
      Y = (*System\MaxY - YFloat)**System\YFactor
      
      ; Draw part:
      If X <> 0
        MoveToEx_(DC, PreviousX, Int(PreviousY), #Null)
      Else
        MoveToEx_(DC, 0, Y, #Null)
      EndIf
      If Abs(PreviousY - Y) < *System\Height
        LineTo_(DC, X, Y)
      EndIf
      
      PreviousY = Y
      PreviousX = X
      X         + Steps
    Wend
    
    DeleteObject_(Pen)
    SelectObject_(DC, PreviousPen)
  StopDrawing()
  
  CloseLevel()
  
  
  
  FreeQueue(*Function)
  FreeObject(*Object)
EndProcedure

Procedure.i Std_CoordSys_Graph()
  Protected *Function.Queue = CreateQueue(@FreeToken()), Color.i, Thickness.i
  
  If PopBlockValue(*Function)
    ; Predefinitions:
    Color     = $000000
    Thickness = 2
    
    ; Get optional parameters:
    PopIValue(@Color)
    PopIValue(@Thickness)
    
    CoordinateSystemDrawGraph(*Current\This\Std_CoordinateSystem, *Function, Color, Thickness)
  Else
    _ErrorP(1)
  EndIf
EndProcedure
Procedure.i Std_CoordSys_Point()
  Protected X.f, Y.f, Color.i
  Protected *System.Std_CoordinateSystem = *Current\This\Memory
  
  If PopFValue(@X) And PopFValue(@Y)
    Color = $FF0000
    
    PopIValue(@Color)
  
    StartDrawing(ImageOutput(*System\Image))
      Circle((X - *System\MinX)**System\XFactor, (*System\MaxY - Y)**System\YFactor, 3, Color)
    StopDrawing()
  Else
    _ErrorP(2)
  EndIf
EndProcedure
Procedure.i Std_CoordSys_DrawBackground()
  Protected BackgroundColor.i, AxeColor.i, GridColor.i
  
  BackgroundColor = $FFFFFF
  AxeColor        = $808080
  GridColor       = $E0E0E0
  
  PopIValue(@BackgroundColor)
  PopIValue(@AxeColor)
  PopIValue(@GridColor)
  
  CoordinateSystemDrawBackground(*Current\This\Std_CoordinateSystem, BackgroundColor, AxeColor, GridColor)
EndProcedure
Procedure.i Std_CoordSys_DrawUnits()
  Protected Color.i, Length.i, Index.i, *System.Std_CoordinateSystem = *Current\This\Memory
  
  Color = $00808080
  
  PopIValue(@Color)
  
  StartDrawing(ImageOutput(*System\Image))
    DrawingMode(#PB_2DDrawing_Transparent)
    
    Length = *System\Width / *System\XFactor - 1
    For Index = 1 To Length
      If *System\MinX + Index <> 0
        DrawText(Index * *System\XFactor - 5, (-*System\MinY)**System\YFactor + 2, Str(*System\MinX + Index), Color)
      EndIf
    Next
    Length = *System\Height / *System\YFactor - 1
    For Index = 1 To Length
      If *System\MinY + Index <> 0
        DrawText((-*System\MinX)**System\XFactor + 2, Index * *System\YFactor - 5, Str(*System\MinY + Index), Color)
      EndIf
    Next
  StopDrawing()
EndProcedure
Procedure.i Std_CoordSys_Open()
  Protected *System.Std_CoordinateSystem = *Current\This\Memory
  Protected Window.i, ImageGadget.i
  Protected Event.i
  
  If GUI
    Window = OpenWindow(#PB_Any, #PB_Ignore, #PB_Ignore, *System\Width, *System\Height, "System", #PB_Window_ScreenCentered|#PB_Window_SystemMenu, WindowID(#GUIWindow))
  Else
    Window = OpenWindow(#PB_Any, #PB_Ignore, #PB_Ignore, *System\Width, *System\Height, "System", #PB_Window_ScreenCentered|#PB_Window_SystemMenu)
  EndIf
    ImageGadget = ImageGadget(#PB_Any, 0, 0, *System\Width, *System\Height, ImageID(*System\Image))
  
  Repeat
    Event = WaitWindowEvent()
    If EventWindow() = Window
      Select Event
        Case #WM_KEYDOWN
          If EventwParam() = #VK_RETURN
            Break
          EndIf
        Case #PB_Event_CloseWindow
          Break
      EndSelect
    EndIf
  ForEver
  
  CloseWindow(Window)
EndProcedure
Procedure.i Std_CoordSys_Save()
  Protected Name.String
  
  If PopSValue(@Name)
    If Not SaveImage(*Current\This\Std_CoordinateSystem\Image, Name\S)
      _Error("Can not save coordinate system: " + ValueName(*Current\This) + " in " + #Q + Name\S + #Q)
    EndIf
  Else
    _ErrorP(1)
  EndIf
EndProcedure
Procedure.i Std_CoordSys__New()
  Protected *System.Std_CoordinateSystem
  
  *System = _New(Std_CoordinateSystem)
  
  *Current\This\Std_CoordinateSystem = *System
  *Current\This\Scope|#Value_Binary
  
  *System\MinX = -10
  *System\MinY = -10
  *System\MaxX =  10
  *System\MaxY =  10
  
  *System\Width  = 400
  *System\Height = 400
  
  CoordinateSystemFactors(*System)
  *System\Image = CreateImage(#PB_Any, *System\Width, *System\Height)
  CoordinateSystemDrawBackground(*System, $FFFFFF, $808080, $E0E0E0)
EndProcedure
Procedure.i Std_CoordSys__Def()
  Protected *System.Std_CoordinateSystem = *Current\This\Memory
  Protected Index.i, Length.i
  
  Select *Current\ArgumentQueue\Count
    Case 2
      PopIValue(@*System\Width)
      PopIValue(@*System\Height)
      *System\MinX = -10
      *System\MinY = -10
    Case 4
      PopIValue(@*System\Width)
      PopIValue(@*System\Height)
      PopFValue(@*System\MaxX)
      PopFValue(@*System\MaxY)
      *System\MaxX / 2
      *System\MaxY / 2
      *System\MinX = -*System\MaxX
      *System\MinY = -*System\MaxY
    Case 6
      PopIValue(@*System\Width)
      PopIValue(@*System\Height)
      PopFValue(@*System\MinX)
      PopFValue(@*System\MinY)
      PopFValue(@*System\MaxX)
      PopFValue(@*System\MaxY)
  EndSelect
  
  If *System\Image
    FreeImage(*System\Image)
  EndIf
  
  CoordinateSystemFactors(*System)
  *System\Image = CreateImage(#PB_Any, *System\Width, *System\Height)
  CoordinateSystemDrawBackground(*System, $FFFFFF, $808080, $E0E0E0)
EndProcedure
Procedure.i Std_CoordSys__Del()
  Protected *System.Std_CoordinateSystem = *Current\This\Memory
  
  If *System\Image
    FreeImage(*System\Image)
  EndIf
  _Free(*System)
EndProcedure

; IDE Options = PureBasic 4.41 (Windows - x86)
; CursorPosition = 81
; FirstLine = 60
; Folding = ---
; EnableXP