;**************************************************************
;     Core
;
; Author: Josef Sniatecki
; Creation date: 03.04.2009
;**************************************************************
;   (c) 2009 Josef Sniatecki
;
; This program is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License
; As published by the Free Software Foundation; either version
; 3 of the License, Or (at your option) any later version.
;
; This program is distributed in the hope that it will be
; useful, but WITHOUT ANY WARRANTY; without even the implied
; warranty of MERCHANTABILITY Or FITNESS For A PARTICULAR
; PURPOSE. See the GNU General Public License For more details.
;
; You should have received a copy of the GNU General Public
; License along With this program;
; If Not, see "http://www.gnu.org/licenses/".
;**************************************************************



Structure Run
  *Token.Token
  *Value.Value
  *Object.Object
  StructureUnion
    *Queue.Queue
    *Value2.Value
    *Object2.Object
    Integer.i
    String.s
  EndStructureUnion
  StructureUnion
    *Value3.Value
    *Object3.Object
    *Queue2.Queues
    *This.Value
    Integer2.i
    String2.s
  EndStructureUnion
EndStructure



Procedure.i Run(Priority.i = 0)
  Protected Run.Run
  
  With Run
  
  Repeat
    _SN(\Token)
    Select \Token\Type
      Case #Token_Name
        ; Normal accessing to an object:
        If Not \Value
          If Priority = 0
            \Object = RunObjectPath(*Current\Object, #RunPath_Define|#RunPath_Global, \Token\Name, 0, @\This)
          Else
            \Object = RunObjectPath(*Current\Object, #RunPath_Global, \Token\Name, 0, @\This)
          EndIf
          _E()
          \Value  = RunObject(\Object, \This) : _E()
        Else
          ; Multiplication:
          _P()
          \Value2 = Run(6) : _E()
          \Value  = OperateValue(#Operator_Multiply, \Value, \Value2) : _E()
        EndIf
      Case #Token_Value
        If Not \Value
          ; Save the address of the value:
          \Value = \Token\Value
        Else
          ; Multiplication:
          _P()
          \Value2 = Run(6) : _E()
          \Value  = OperateValue(#Operator_Multiply, \Value, \Value2) : _E()
        EndIf
      Case #Token_Operator
        If Priority > 0 And Priority => OperatorPriority(\Token\Operator)
          ; The operator has a low priority:
          _P()
          Break
        Else
          ; Operator with a greater priority than actually:
          \Value2 = Run(OperatorPriority(\Token\Operator))        : _E() : _RB()
          \Value  = OperateValue(\Token\Operator, \Value, \Value2) : _E() : _RB()
        EndIf
      Case #Token_Control
        Select \Token\Control
          Case #Control_Next
            ; Accessing to child objects of the current value:
            If \Value
              If \Value\Scope&#Value_Searchable
                \This   = \Value
                \Object = RunObjectPath(\Value\Memory, #RunPath_Define, "", \Value, @\This)
              Else
                _Error("A value does not contain a searchable memroy: " + ValuePath(\Value))
              EndIf
            Else
              \Object = RunObjectPath(Main\Object, #RunPath_Define|#RunPath_Global, "", 0, @\This)
            EndIf
            _E()
            \Value = RunObject(\Object, \This) : _E()
          Case #Control_Class
            ; Accessing to class objects of the current value:
            If \Value
              \Object = RunObjectPath(\Value\Class, 0, "", \Value, @\This) : _E()
            ElseIf *Current\This
              \Object = RunObjectPath(*Current\This\Class, 0, "", *Current\This, @\This)
            ElseIf *Current\Object And *Current\Object\Group\Value
              \Object = RunObjectPath(*Current\Object\Group\Value\Class, 0, "", *Current\Object\Group\Value, @\This) : _E()
            Else
              _Error("No value specified for " + #Q + ":" + #Q + "!")
            EndIf
            \Value = RunObject(\Object, \This) : _E()
          Case #Control_Previous
            ; Accessing to the super object of the current value:
            If \Value
              If \Value\Scope&#Value_Searchable And \Value\Object\Super
                \Object = RunObjectPath(\Value\Object\Super, #RunPath_Define|#RunPath_Global, "", 0, @\This) : _E()
              Else
                _Error("This object does not have a parent object: " + ObjectPath(\Value\Object))
              EndIf
            Else
              _Error("No value specified for " + #Q + "\\" + #Q + "!")
            EndIf
            \Value = RunObject(\Object, \This) : _E()
          Case #Control_Type
            ; Convert the current value to the specified class:
            If \Value
              \Object = RunClass() : _E()
              If \Object <> \Value\Class
                \Value = SetValue(PushValue(\Object), \Value)
              EndIf
            Else
              _Error("No value defined for a convertion.")
            EndIf
          Case #Control_Address
            ; Get the address of the object:
            \Value = PushPValue(Abs(RunObjectPath(*Current\Object, #RunPath_Define|#RunPath_Global, "")))
            ; "Abs" is called, because new defined objects are returned by a negative address.
          Case #Control_ECT
            ; Make an enumerated queue:
            If Priority <= 2
              \Value2 = Run(3) : _E()
              If \Value And \Value2
                \Value = Enumerate(\Value, \Value2) : _E()
                \Value = PushQueueValue(\Value)
              Else
                _Error("Only one value used in an enumeration!")
              EndIf
            Else
              _P()
              Break
            EndIf
        EndSelect
      Case #Token_Seperator
        Select \Token\Seperator
          Case #Seperator_Command
            If Priority
              _P()
            EndIf
            Break
          Case #Seperator_Argument
            If Priority > -1 And Priority < 3
              \Value2 = PushQueueValue(CreateQueue(@FreeValue()))
              If \Value\Scope&#Value_Literal
                AppendQueueItem(\Value2\Queue, DuplicateValue(\Value))
              Else
                AppendQueueItem(\Value2\Queue, CreateReferenceValue(\Value))
              EndIf
              Repeat
                \Value = Run(3) : _E()
                If \Value
                  If \Value\Scope&#Value_Literal
                    AppendQueueItem(\Value2\Queue, DuplicateValue(\Value))
                  Else
                    AppendQueueItem(\Value2\Queue, CreateReferenceValue(\Value))
                  EndIf
                EndIf
              Until Not NextArgument()
              \Value = \Value2
            Else
              _P()
              Break
            EndIf
          Case #Seperator_Block
            _P()
            Break
        EndSelect
      Case #Token_Block
        Select \Token\Block\Bracket
          Case #Bracket_Round
            If Not \Value
              OpenLevel(\Token\Block\TokenQueue, 0, 0, 0, #True)
                \Value = Run(-1)
              CloseLevel()
            ElseIf \Value\Class = *ObjectClass
              ; Fast definition:
              \Value = PushValue(\Value\Object)
              \Queue = RunArguments(\Token\Block\TokenQueue)
              If \Queue\Count
                CallClassMethod(\Value\Class, #ClassMethod_Define, \Queue, \Value, #False) : _E()
              ElseIf \Value\Class = *ObjectClass
                _SN(\Token)
                If \Token\Type = #Token_Block And \Token\Block\Bracket = #Bracket_Curly
                  OpenLevel(\Token\Block\TokenQueue, 0, \Value\Object, 0)
                    RunGroupDefinition(\Value\Object)
                  CloseLevel()
                Else
                  _P()
                EndIf
              EndIf
            Else
              ; Fast multiplication:
              OpenLevel(\Token\Block\TokenQueue, 0, 0, 0, #True)
                \Value2 = Run() : _E()
                \Value = OperateValue(#Operator_Multiply, \Value, \Value2)
              CloseLevel()
            EndIf
            _E() : _RB()
          Case #Bracket_Square
            If \Value
              ; Allows an access by the class method "_args":
              \Value = CallClassMethod(\Value\Class, #ClassMethod_Arguments, RunArguments(\Token\Block\TokenQueue), \Value)
            Else
              ; Create a new queue:
              \Value2 = RunArguments(\Token\Block\TokenQueue, #True) : _E()
              \Value  = PushQueueValue(\Value2)
            EndIf
          Case #Bracket_Curly
            If \Value
              If \Value\Class = *ObjectClass
                OpenLevel(\Token\Block\TokenQueue, 0, \Value\Object, \Value, #True)
                  \Value = RunBlock()
                CloseLevel()
              Else
                \Object = CreateGroup("_block")
                  CreateVariable("value", CreateReferenceValue(\Value), \Object)
                OpenLevel(\Token\Block\TokenQueue, 0, \Object, \Value, #True)
                  \Value = RunBlock()
                CloseLevel()
                FreeObject(\Object)
              EndIf
              _E() : _RB()
            Else
              ; Create a new block:
              \Value = PushBlockValue(CopyQueue(\Token\Block\TokenQueue, CreateQueue(@FreeToken()), @CopyToken()))
            EndIf
        EndSelect
      Case #Token_Keyword
        Select \Token\Keyword
          Case #Keyword_Define
            ; Define a object protected form global and standard objects:
            If \Value And \Value\Scope&#Value_Searchable
              ; Use another parent object:
              \Object2 = \Value\Memory
            Else
              ; Use the current object as parent:
              \Object2 = *Current\Object
            EndIf
            _SN(\Token)
            If \Token\Type = #Token_Block And \Token\Block\Bracket = #Bracket_Square
              ; Save all new objects in a queue:
              \Value = PushQueueValue(CreateQueue(@FreeValue()))
              OpenLevel(\Token\Block\TokenQueue, 0, 0, 0, #True)
                Repeat
                  \Object = Abs(RunObjectPath(\Object2, #RunPath_Define, "")) : _E("in " + #Q + "def[...]" + #Q + ".")
                  \Value3 = RunObject(\Object, 0)                             : _E("in " + #Q + "def[...]" + #Q + ".")
                  AppendQueueItem(\Value\Queue, CreateReferenceValue(\Value3))
                Until Not NextArgument()
              CloseLevel()
            Else
              ; Define objects without a queue:
              _P()
              Repeat
                RunObjectPath(\Object2, #RunPath_Define, "") : _E("in " + #Q + "def" + #Q + ".")
              Until Not NextArgument()
            EndIf
          Case #Keyword_IsDefined
            ; Check a object protected form global and standard objects:
            If \Value And \Value\Scope&#Value_Searchable
              ; Use another parent object:
              \Object2 = \Value\Memory
            Else
              ; Use the current object as parent:
              \Object2 = *Current\Object
            EndIf
            \Value = PushBValue(#True)
            Repeat
              If Not RunObjectPath(\Object2, #RunPath_Global, "", 0, 0, #False)
                \Value\Integer\I = #False
              EndIf
            Until Not NextArgument()
          Case #Keyword_Delete
            ; Delete specified object of a parent object:
            If \Value And \Value\Scope&#Value_Searchable
              ; Use another parent object:
              \Object2 = \Value\Memory
              Repeat
                \Object = RunObjectPath(\Object2, 0, "") : _E("in " + #Q + "del" + #Q + ".")
                FreeObject(\Object)
              Until Not NextArgument()
            Else
              ; Use the current object as parent object:
              \Object2 = *Current\Object
              Repeat
                \Object = RunObjectPath(\Object2, #RunPath_Global, "") : _E("in " + #Q + "del" + #Q + ".")
                FreeObject(\Object)
              Until Not NextArgument()
            EndIf
          Case #Keyword_Pop
            ; Pop values of the argument queue:
            Select *Current\TokenItem\Token\Type
              Case #Token_Name, #Token_Block
                ; Save the first argument into a variable:
                If \Value And \Value\Scope&#Value_Searchable
                  ; Use another parent object:
                  \Object2 = \Value\Memory
                Else
                  ; Use the current object as parent object:
                  \Object2 = *Current\Object
                EndIf
                \Value = PushBValue(#True)
                Repeat
                  \Object = RunObjectPath(\Object2, #RunPath_Define, "", @\This) : _E("in " + #Q + "pop" + #Q + ".")
                  If \Value\Integer\I
                    If Not PopValue(RunObject(\Object, \This))
                      \Value\Integer\I = #False
                    EndIf
                  EndIf
                Until Not NextArgument()
              Default
                ; Return the first argument:
                If \Value And \Value\Scope&#Value_Searchable
                  PopObject(\Value\Object)
                Else
                  If Not PopValueAddress(@\Value)
                    \Value = PushBValue(#False)
                  EndIf
                EndIf
            EndSelect
          Case #Keyword_If
            ; Start if-controll:
            \Value = Statement_If(\Value) : _E("In a " + #Q + "if" + #Q + " statement.") : _RB()
          Case #Keyword_Else
            If Priority <= 2
              If \Value And IsValueTrue(\Value)
                Ignore()
              Else
                \Value = Run(-1) : _E()
              EndIf
            Else
              _P()
              Break
            EndIf
          Case #Keyword_While
            ; Start while-controll:
            Statement_While(\Value) : _E("In a " + #Q + "while" + #Q + " loop.") : _RB()
            \Value = 0
          Case #Keyword_Repeat
            ; Start repeat-controll:
            \Value = Statement_Repeat() : _E("In a " + #Q + "repeat" + #Q + " loop.") : _RB()
          Case #Keyword_For
            If Priority <= 2
              \Value = Statement_For(\Value) : _E("In a " + #Q + "for" + #Q + " loop.") : _RB()
            Else
              _P()
              Break
            EndIf
          Case #Keyword_Switch
            \Value = Statement_Switch(\Value) : _E("In a " + #Q + "switch" + #Q + " statement.") : _RB()
          Case #Keyword_And
            ; and-operator for expressions:
            If Priority <= 0
              If IsValueTrue(\Value)
                If IsValueTrue(Run(-1))
                  \Value = PushBValue(#True)
                Else
                  \Value = PushBValue(#False)
                EndIf
              Else
                ; If the first value is false, the second will be ignored:
                \Value = PushBValue(#False)
                Ignore()
              EndIf
            Else
              _P()
              Break
            EndIf
          Case #Keyword_Or
            If Priority <= 0
              If IsValueTrue(\Value)
                ; If the first value if true, the second will be ignored:
                \Value = PushBValue(#True)
                Ignore()
              ElseIf IsValueTrue(Run(-1))
                \Value = PushBValue(#True)
              Else
                \Value = PushBValue(#False)
              EndIf
            Else
              _P()
              Break
            EndIf
          Case #Keyword_XOr
            If Priority <= 0
              If IsValueTrue(\Value) XOr IsValueTrue(Run(-1))
                \Value = PushBValue(#True)
              Else
                \Value = PushBValue(#False)
              EndIf
            Else
              _P()
            EndIf
          Case #Keyword_Not
            If IsValueTrue(Run(-1))
              \Value = PushBValue(#False)
            Else
              \Value = PushBValue(#True)
            EndIf
          Case #Keyword_Break
            _SN(\Token)
            If \Token\Type <> #Token_Seperator
              _P()
              Truncation = ValueI(Run()) : _E()
            Else
              Truncation = 1
            EndIf
            Break
          Case #Keyword_Return
            _SN(\Token)
            If \Token\Type <> #Token_Seperator
              _P()
              \Value = Run() : _E()
            EndIf
            Truncation = #Truncation_Return
            Break
          Case #Keyword_Output
            ; Output the following values:
            If \Value And \Value\Scope&#Value_Searchable
              OpenLevel(0, 0, \Value\Object, 0, #True)
                Repeat
                  \Value  = Run(-1)             : _E()
                  \String = FormatValue(\Value) : _E()
                  OutputFL(#Symbol_Output, \String)
                Until Not NextArgument()
              CloseLevel()
            Else
              Repeat
                \Value  = Run(-1)             : _E()
                \String = FormatValue(\Value) : _E()
                OutputFL(#Symbol_Output, \String)
              Until Not NextArgument()
            EndIf
            \Value = 0
          Case #Keyword_End
            Truncation = #Truncation_End
            Break
        EndSelect
    EndSelect
  ForEver
  
  ProcedureReturn \Value
  
  EndWith
EndProcedure
Procedure.i RunBlock()
  Protected *Result.Value, *Token.Token
  
  If *Current\TokenQueue\Count
    Repeat
      *Result = Run() : _RB()
      *Token = *Current\TokenItem\Token
      If *Token\Type = #Token_Seperator And *Token\Seperator = #Seperator_Block
        ProcedureReturn *Result
      Else
        ClearStack()
      EndIf
    ForEver
  Else
    ProcedureReturn #Null
  EndIf
  
  ProcedureReturn *Result
EndProcedure
Procedure.i RunControlBlock()
  Protected *Token.Token, *Result.Value
  
  _SN(*Token)
  If *Token\Type = #Token_Block
    Select *Token\Block\Bracket
      Case #Bracket_Round
        OpenLevel(*Token\Block\TokenQueue, 0, 0, 0, #True)
          *Result = Run()
        CloseLevel()
      Case #Bracket_Curly
        OpenLevel(*Token\Block\TokenQueue, 0, 0, 0, #True)
          *Result = RunBlock()
        CloseLevel()
      Default
        _Error("Invalid brackets used for a control block: " + #Q + Bracket(*Token\Block\Bracket) + "..." + ClosingBracket(*Token\Block\Bracket) + #Q)
    EndSelect
    ProcedureReturn *Result
  Else
    _Error("A block is missing for a control.")
  EndIf
EndProcedure
Procedure.i RunCommands()
  ; Evaluates several lines. 
  
  Protected *Result.Value
  
  While *Current\TokenItem
    ClearStack()
    *Result = Run() : _E() : _RB()
  Wend
  ProcedureReturn *Result
EndProcedure

Procedure.i Call(*Object.Object, *ArgumentQueue.Queue = 0, *This.Value = 0, Result.i = #True)
  Protected *Result.Value, *Value.Value, *Function.Object, *PushedFunction.Value
  
  Select *Object\Function\Type
    Case #Function_Address
      ; Call binary function:
      OpenLevel(0, *ArgumentQueue, 0, *This, #True, #True)
        *Result = _Call(*Object\Function\Address)
      CloseLevel()
    Case #Function_Block
      ; Construct a self made value:
      If *This And *Object = *Object\Super\Group\ClassMethod[#ClassMethod_New]
        *This\Memory = CreateGroup("value", 0, *This)
        *This\Scope|#Value_Searchable
      EndIf
      ; Push a buffer for the result:
      If Result
        If *Object\Function\Class
          *Result = PushValue(*Object\Function\Class)
        EndIf
      EndIf
      ; Build function object:
      *Function = CreateGroup(*Object\Name)
      ; Push it into the stack:
      *PushedFunction = PushOValue(*Function)
      If *This
        CreateVariable("this", CreateReferenceValue(*This), *Function)
      EndIf
      CreateReferenceObject("self" , *Function    , *Function)
      CreateReferenceObject("super", *Object\Super, *Function)
      ; Call block:
      If *Object\Function\Class
        OpenLevel(*Object\Function\TokenQueue, *ArgumentQueue, *Function, *This, #False, #True)
      Else
        OpenLevel(*Object\Function\TokenQueue, *ArgumentQueue, *Function, *This, #True, #True)
      EndIf
        *Value = RunBlock()
        If Truncation = #Truncation_Return
          Truncation = 0
        EndIf
        If Result And *Value
          If *Object\Function\Class And *Value\Class <> *VariantClass
            ; Make a copy:
            SetValue(*Result, *Value)
          ElseIf *Value\Class <> *VariantClass
            *Result = *Value
          Else
            *Result = *PushedFunction
          EndIf
        Else
          *Result = *PushedFunction
        EndIf
      CloseLevel()
  EndSelect
  
  _E("Occurred in: " + ObjectPath(*Object) + "()")
  ProcedureReturn *Result
EndProcedure
Procedure.i CallFast(*Super.Object, Name.s, *ArgumentQueue.Queue = 0, *This.Object = 0, Result.i = #True, EnableError.i = #True)
  Protected *Object.Object
  
  *Object = FindObject(*Super, Name)
  If *Object
    ; Function found:
    ProcedureReturn Call(*Object, *ArgumentQueue, *This, Result)
  Else
    ; Function not found:
    FreeQueue(*ArgumentQueue)
    If EnableError
      _Error("Function not found: " + #Q + ObjectPath(*Super) + "\" + Name + #Q)
    Else
      ProcedureReturn PushVValue()
    EndIf
  EndIf
EndProcedure
Procedure.i CallClassMethod(*Class.Object, Method.i, *ArgumentQueue.Queue = 0, *This.Value = 0, Result.i = #True, EnableError.i = #True)
  If *Class\Group\ClassMethod[Method]
    ; Class method is vaild:
    ProcedureReturn Call(*Class\Group\ClassMethod[Method], *ArgumentQueue, *This, Result)
  Else
    ; Class method don't exist:
    If *ArgumentQueue > 0
      FreeQueue(*ArgumentQueue)
    EndIf
    If EnableError
      _Error("The class don't contains a method: " + #Q + ClassName(*Class) + "\" + ClassMethod(Method) + #Q)
    Else
      ProcedureReturn #False
    EndIf
  EndIf
EndProcedure
Procedure.i RunArguments(*TokenQueue.Queue, CopyLiterals.i = #False)
  ; This function extracts all arguments of a token queue.
  ; The result is a list of calculated arguments.
  
  Protected *ArgumentQueue.Queue, *Token.Token
  Protected *Value.Value
  
  *ArgumentQueue = CreateQueue(#Queue_SaveMemory)
  
  OpenLevel(*TokenQueue, 0, 0, 0, #True)
    Repeat
      _SN(*Token)
      If *Token\Type = #Token_Seperator
        If *Token\Seperator = #Seperator_Block
          ; End the extracting of arguments:
          Break
        ElseIf *Token\Seperator = #Seperator_Argument
          ; Execute next argument:
          *Value = Run(-1) : _E("", FreeQueue(*ArgumentQueue))
          If CopyLiterals And *Value\Scope&#Value_Literal
            *Value = PushDuplicatedValue(*Value)
          EndIf
          AppendQueueItem(*ArgumentQueue, *Value)
        EndIf
      Else
        _P()
        ; Execute the first argument and put the result it in the list:
        *Value = Run(-1) : _E("", FreeQueue(*ArgumentQueue))
        If CopyLiterals And *Value\Scope&#Value_Literal
          *Value = PushDuplicatedValue(*Value)
        EndIf
        AppendQueueItem(*ArgumentQueue, *Value)
      EndIf
    ForEver
  CloseLevel()
  
  ProcedureReturn *ArgumentQueue
EndProcedure

Procedure.i RunObjectPath(*Super.Object, Mode.i, Name.s, *This.Value = 0, *ResultThis.Integer = 0, EnableError.i = #True)
  ; By this function can objects searched by
  ; a list of tokens.
  
  Protected *Object.Object
  Protected *Token.Token, *Value.Value, *Level.Object
  
  Repeat
    If *Super\Type = #Object_Group
      If Not Name
        _SN(*Token)
        Select *Token\Type
          Case #Token_Name
            ; Find the object by a given name:
            Name = *Token\Name
            *Object = FindObject(*Super, Name)
          Case #Token_Block
            Select *Token\Block\Bracket
              Case #Bracket_Round
                OpenLevel(*Token\Block\TokenQueue, 0, 0, 0, #True)
                  *Value = Run() : _E()
                CloseLevel()
                Select *Value\Class
                  Case *IntegerClass
                    ; Get a object by an specified index:
                    If Not SelectQueueItem(*Super\Group\ObjectQueue, *Value\Integer\I, @*Object)
                      ; Missing objects get a general name:
                      Name = "child_" + Str(*Value\Integer\I)
                    EndIf
                  Case *StringClass
                    ; Find the object by a variable string/name:
                    Name = *Value\String\S
                    *Object = FindObject(*Super, Name)
                  Case *PointerClass
                    ; Set the current object to the given address:
                    *Object = *Value\Object
                    *Super = *Object\Super
                EndSelect
              Case #Bracket_Square
                OpenLevel(*Token\Block\TokenQueue, 0, 0, 0, #True)
                  *Object = RunSubObjects(*Super) : _E()
                CloseLevel()
                ProcedureReturn *Object
              Default
                _Error("Invalid brackets used: " + "...\" + Bracket(*Token\Block\Bracket) + "..." + ClosingBracket(*Token\Block\Bracket))
            EndSelect
          Default
            _P()
            ProcedureReturn *Super
        EndSelect
      Else
        *Object = FindObject(*Super, Name)
      EndIf
    Else
      *Object = *Super
    EndIf
    
    If Not *Object
      If Mode&#RunPath_Global And *Super = *Current\Object
        If *Current\This And *Current\This\Scope&#Value_Searchable And *Current\This\Object\Type = #Object_Group
          *Object = FindObject(*Current\This\Object, Name)
        EndIf
        If Not *Object
          ; Search in lower levels:
          *Object = SearchLevels(Name)
        EndIf
        If Not *Object And *Current\Super
          *Object = FindObject(*Current\Super\Object, Name)
          If Not *Object And *Current\Super\This And *Current\Super\This\Scope&#Value_Searchable And *Current\This <> *Current\Super\This And *Current\Super\This\Object\Type = #Object_Group
            *Object = FindObject(*Current\Super\This\Object, Name)
          EndIf
        EndIf
        If Not *Object And *Current\Object <> Main\Object
          *Object = FindObject(Main\Object, Name)
        EndIf
        If Not *Object And *Current\Object <> *StandardLibrary
          *Object = FindObject(*StandardLibrary, Name)
        EndIf
      EndIf
      If Not *Object And Mode&#RunPath_Define
        ; Define the missing object:
        *Object = -RunDefinition(Name, *Super) : _E("In a definition: " + ObjectPath(*Super) + "\" + Name)
        ProcedureReturn *Object
      EndIf
      If Not *Object
        Break
      EndIf
    EndIf
    
    Name = ""
    
    Select *Object\Type
      Case #Object_Variable
        _SN(*Token)
        Select *Token\Type
          Case #Token_Control
            Select *Token\Control
              Case #Control_Next
                *This = *Object\Variable\Value
                If *This\Scope&#Value_Searchable
                  *Super = *This\Memory
                Else
                  _Error("The value does not contain a searchable memory: " + ObjectPath(*Object))
                EndIf
              Case #Control_Class
                *This  = *Object\Variable\Value
                *Super = *This\Class
                Mode&~(#RunPath_Define)
              Case #Control_ObjectClass
                ; Get the object class:
                *This  = PushOValue(*Object, #True)
                *Super = *This\Class
                Mode&~(#RunPath_Define)
              Default
                _P()
                Break
            EndSelect
          Default
            _P()
            Break
        EndSelect
      Case #Object_Group
        _SN(*Token)
        Select *Token\Type
          Case #Token_Control
            Select *Token\Control
              Case #Control_Next
                ; Change current super object to the child object:
                *Super = *Object
              Case #Control_Previous
                ; Change the current object to the super object of the child:
                If *Object\Super
                  *Super = *Object\Super
                Else
                  _Error("This object does not have a parent: " + ObjectPath(*Object))
                EndIf
              Case #Control_Class, #Control_ObjectClass
                ; Get the object class:
                *This = PushOValue(*Object, #True)
                *Super = *This\Class
                Mode&~(#RunPath_Define)
              Default
                _P()
                Break
            EndSelect
          Default
            _P()
            Break
        EndSelect
      Case #Object_Function
        _SN(*Token)
        Select *Token\Type
          Case #Token_Control
            Select *Token\Control
              Case #Control_ObjectClass
                ; Get the object class:
                *This = PushOValue(*Object, #True)
                *Super = *This\Class
                Mode&~(#RunPath_Define)
              Default
                _P()
                Break
            EndSelect
          Default
            _P()
            Break
        EndSelect
    EndSelect
    
    *Object = #Null
  ForEver
  
  If *Object
    If *ResultThis
      *ResultThis\I = *This
    EndIf
    ProcedureReturn *Object
  ElseIf EnableError
    If *This
      _Error("Classmethod not defined:" + #N + "- value:  " + ValuePath(*This) + #N + "- method: " + ObjectPath(*Super) + "\" + Name)
    Else
      _Error("Object not defined: " + ObjectPath(*Super) + "\" + Name)
    EndIf
  EndIf
EndProcedure
Procedure.i RunSubObjects(*Super.Object)
  Protected *Token.Token, *Object.Object, *ResultObject.Object
  
  *ResultObject = CreateGroup("")
  
  _SN(*Token)
  Repeat
    If *Token\Type = #Token_Seperator
      If *Token\Seperator <> #Seperator_Argument
        PushOValue(*ResultObject)
        ProcedureReturn *ResultObject
      EndIf
    Else
      _P()
      *Object = RunObjectPath(*Super, 0, "") : _E()
      CreateReferenceObject(*Object\Name, *Object, *ResultObject)
    EndIf
    _SN(*Token)
  ForEver
EndProcedure

Procedure.i RunDefinition(Name.s, *Super.Object)
  Protected *Token.Token, *Value.Value, *Object.Object, *Class.Object
  Protected *Queue.Queue
  
  _SN(*Token)
  Select *Token\Type
    Case #Token_Operator
      Select *Token\Operator
        Case #Operator_Set
          ; Variable definition by an expression (type conclusion):
          *Value = Run(-1) : _E()
          If *Value
            *Object = CreateVariable(Name, DuplicateValue(*Value), *Super)
          Else
            _Error("No value defined.")
          EndIf
        Case #Operator_Reference
          ; Create a reference to a specified object:
          *Object = Abs(RunObjectPath(*Current\Object, #RunPath_Define|#RunPath_Global, "", 0, 0)) : _E() ;Original
          *Object = CreateReferenceObject(Name, *Object, *Super)                                          ;Reference
        Case #Operator_Input
          *Value = Execute(InputFLT(#Symbol_Input, Name), 0, #True) : _E()
          If *Value
            *Object = CreateVariable(Name, DuplicateValue(*Value), *Super)
          Else
            _Error("No value typed.")
          EndIf
        Case #Operator_Multiply
          ; Define a null pointer:
          *Object = CreateVariable(Name, CreatePValue(0), *Super)
        Default
          _Error("Invalid operator used: " + #Q + Operator(*Token\Operator) + #Q)
      EndSelect
    Case #Token_Value
      ; Fast variable definition:
      *Object = CreateVariable(Name, DuplicateValue(*Token\Value), *Super)
    Case #Token_Control
      Select *Token\Control
        Case #Control_Type
          *Class = RunClass() : _E()
          If *Class
            *Object = CreateVariable(Name, CreateValue(*Class), *Super)
          Else
            _Error("Class is not given for a definition: " + #Q + ObjectPath(*Super) + "\" + Name + #Q)
          EndIf
          _SN(*Token)
          If *Token\Type = #Token_Block And *Token\Block\Bracket = #Bracket_Round
            ; Fast definition:
            *Queue = RunArguments(*Token\Block\TokenQueue) : _E()
            CallClassMethod(*Class, #ClassMethod_Define, *Queue, *Object\Variable\Value, #False)
          Else
            _P()
          EndIf
          If *Object\Variable\Value\Scope&#Value_Searchable
            _SN(*Token)
            If *Token\Type = #Token_Block And *Token\Block\Bracket = #Bracket_Curly
              ; Open a definition block:
              OpenLevel(*Token\Block\TokenQueue, 0, *Object\Variable\Value\Object, 0)
                RunBlock()
              CloseLevel()
            Else
              _P()
            EndIf
          EndIf
          _SN(*Token)
          If *Token\Type = #Token_Operator And *Token\Operator = #Operator_Set
            SetValue(*Object\Variable\Value, Run(-1))
          Else
            _P()
          EndIf
        Default
          _Error("Invalid control used after an undefined object: " + #Q + ObjectPath(*Super) + "\" + Name + FormatToken(*Token) + #Q)
      EndSelect
    Case #Token_Block
      Select *Token\Block\Bracket
        Case #Bracket_Round
          ; Create a new function:
          *Object = CreateBlockFunction(Name, 0, 0, *Super)
          RunFunctionDefinition(*Object) : _E()
        Case #Bracket_Curly
          ; Define a new group:
          *Object = CreateGroup(Name, *Super)
          If *Token\Block\TokenQueue\Count > 1
            ; Open a definition block:
            OpenLevel(*Token\Block\TokenQueue, 0, *Object, 0)
              RunGroupDefinition(*Object)
            CloseLevel()
          EndIf
        Case #Bracket_Square
          If *Token\Block\TokenQueue\Count
            *Object = CreateVariable(Name, 0, *Super)
            RunQueueDefinition(*Object) : _E("In a definition: " + #Q + ObjectPath(*Super) + "\" + Name + #Q, FreeObject(*Object))
          Else
            *Object = CreateVariable(Name, CreateQueueValue(), *Super)
          EndIf
      EndSelect
    Default
      _P()
  EndSelect
  
  If Not *Object
    *Object = CreateVariable(Name, CreateVValue(), *Super)
  EndIf
  
  ProcedureReturn *Object
EndProcedure
Procedure.i RunGroupDefinition(*Object.Object)
  Protected *Token.Token, *Child.Object
  
  Repeat
    _SN(*Token)
    Select *Token\Type
      Case #Token_Seperator
        Select *Token\Seperator
          Case #Seperator_Block
            ProcedureReturn
          Case #Seperator_Argument
            _SN(*Token)
            CreateReferenceObject(*Token\Name, *Child, *Object)
        EndSelect
      Default
        _P()
        *Child = Abs(RunObjectPath(*Object, #RunPath_Define, "")) : _E()
    EndSelect
  ForEver
EndProcedure
Procedure.i RunFunctionDefinition(*Object.Object)
  Protected *TokenQueue.Queue, *Token.Token, *DestinationToken.Token
  
  *TokenQueue                 = CreateQueue(@FreeToken())
  *Object\Function\TokenQueue = *TokenQueue
  
  ; Copy argument queue:
  *Token = *Current\TokenItem\P\Memory
  If *Token\Block\TokenQueue\Count > 1
    AddIntegerToken(*TokenQueue, #Token_Keyword, #Keyword_Pop)
    CopyQueue(*Token\Block\TokenQueue, *TokenQueue, @CopyToken(), *TokenQueue\Last)
    *TokenQueue\Last\Token\Seperator = #Seperator_Command
  EndIf
  ; Scopes:
  _SN(*Token)
  If *Token\Type = #Token_Control And *Token\Control = #Control_Type
    ; Get a result-class:
    *Object\Function\Class = RunClass() : _EG(RunFunctionDefinition, "For the result class of a function: " + #Q + ObjectPath(*Object) + #Q)
    _SN(*Token)
  EndIf
  ; Copy commands:
  If *Token\Type = #Token_Block
    ; Copy a whole block:
    If *Token\Block\Bracket = #Bracket_Curly
      CopyQueue(*Token\Block\TokenQueue, *TokenQueue, @CopyToken(), *TokenQueue\Last)
    Else
      _ErrorG(RunFunctionDefinition, "Invalid brackets used for a function: " + #Q + Bracket(*Token\Block\Bracket) + "..." + ClosingBracket(*Token\Block\Bracket) + #Q + ", function = " + #Q + ObjectPath(*Object) + #Q)
    EndIf
  ElseIf *Token\Type <> #Token_Seperator
    ; Copy a single line:
    If *Token\Type = #Token_Operator And *Token\Operator = #Operator_Set
      _SN(*Token)
    EndIf
    Repeat
      *DestinationToken = _New(Token)
      CopyToken(*Token, *DestinationToken)
      AppendQueueItem(*TokenQueue, *DestinationToken)
      _SN(*Token)
    Until *Token\Type = #Token_Seperator And (*Token\Seperator = #Seperator_Command Or *Token\Seperator = #Seperator_Block)
    AddIntegerToken(*TokenQueue, #Token_Seperator, #Seperator_Block)
    _P()
  Else
    _ErrorG(RunFunctionDefinition, "Function invalid defined: " + ObjectPath(*Object))
  EndIf
  
  ProcedureReturn *Object
  
  ; If an error occured:
  RunFunctionDefinition_Error:
    FreeObject(*Object)
    ProcedureReturn #False
EndProcedure
Procedure.i RunQueueDefinition(*Object.Object)
  Protected *Token.Token, *ArgumentQueue.Queue, *Item.QueueItem
  Protected *Class.Object, *DefinitionQueue.Queue, Integer.i
  
  *ArgumentQueue = RunArguments(*Current\TokenItem\P\Token\Block\TokenQueue) : _EG(RunQueueDefinition)
  If *ArgumentQueue\Count
    _SN(*Token)
    If *Token\Type = #Token_Control And *Token\Control = #Control_Type
      *Class = RunClass() : _EG(RunQueueDefinition)
      _SN(*Token)
      If *Token\Type = #Token_Block And *Token\Block\Bracket = #Bracket_Round
        *DefinitionQueue = RunArguments(*Token\Block\TokenQueue) : _EG(RunQueueDefinition)
      Else
        _P()
      EndIf 
    Else
      *Class = *VariantClass
      _P()
    EndIf
    
    *Item = *ArgumentQueue\First
    While *Item
      If *Item\Value\Class <> *IntegerClass
        Integer = ValueI(*Item\Value) : _EG(RunQueueDefinition)
        *Item\Value = PushIValue(Integer)
      EndIf
      _Move(*Item, N)
    Wend
    
    *Object\Variable\Value = CreateQueueValue()
    DimensionQueue(*Object\Variable\Value\Queue, *Class, *DefinitionQueue, *ArgumentQueue\First) : _EG(RunQueueDefinition)
    If *DefinitionQueue
      FreeQueue(*DefinitionQueue)
    EndIf
  Else
    *Object\Variable\Value = CreateQueueValue()
  EndIf
  
  FreeQueue(*ArgumentQueue)
  ProcedureReturn #True
  
  RunQueueDefinition_Error:
    If *ArgumentQueue
      FreeQueue(*ArgumentQueue)
    EndIf
    If *DefinitionQueue
      FreeQueue(*DefinitionQueue)
    EndIf
    ProcedureReturn #False
EndProcedure

Procedure.i RunObject(*Object.Object, *This.Value)
  ; This function gets the value of a given object.
  
  Protected *Token.Token, *Result.Value, *ResultQueue.Queue, *ResultObject.Object
  Protected *ArgumentQueue.Queue
  
  If *Object > 0
    ; Normal:
    Select *Object\Type
      Case #Object_Variable
        ; Return the value of the variable:
        ProcedureReturn *Object\Variable\Value
      Case #Object_Function
        ; Call the given function:
        _SN(*Token)
        If *Token\Type = #Token_Block And *Token\Block\Bracket = #Bracket_Round
          ; Call with a queue of arguments:
          Repeat
            *ArgumentQueue = RunArguments(*Token\Block\TokenQueue) : _EB(1, "In an argument queue: " + #Q + ObjectPath(*Object) + #Q)
            *Result = Call(*Object, *ArgumentQueue, *This)         : _EB()
            
            _SN(*Token)
            
            If *Token\Type = #Token_Block And *Token\Block\Bracket = #Bracket_Curly
              If *Result\Scope&#Value_Searchable
                OpenLevel(*Token\Block\TokenQueue, 0, *Result\Object, 0, #True)
                  RunBlock()
                CloseLevel()
              Else
                *ResultObject = CreateGroup("_block", 0, *Result)
                  CreateVariable("value", CreateReferenceValue(*Result), *ResultObject)
                OpenLevel(*Token\Block\TokenQueue, 0, *ResultObject, 0, #True)
                  RunBlock()
                CloseLevel()
              EndIf
              _EB(1, "In a result-block: " + #Q + ObjectPath(*Object) + #Q)
              *Result = 0
              _SN(*Token)
            EndIf
            
            If *Token\Type = #Token_Block And *Token\Block\Bracket = #Bracket_Round
              If *Result
                If Not *ResultQueue
                  *ResultQueue = CreateQueue(@FreeValue())
                EndIf
                AppendQueueItem(*ResultQueue, CreateReferenceValue(*Result))
              EndIf
              Continue
            Else
              _P()
              If *ResultQueue
                If *Result
                  AppendQueueItem(*ResultQueue, CreateReferenceValue(*Result))
                EndIf
              EndIf
              Break
            EndIf
          ForEver
          ; If an error occurred:
          If Error
            If *ResultQueue
              FreeQueue(*ResultQueue)
            EndIf
            ProcedureReturn
          EndIf
          ; Return the real result:
          If *ResultQueue
            ProcedureReturn PushQueueValue(*ResultQueue)
          ElseIf *Result
            ProcedureReturn *Result
          Else
            ProcedureReturn 0
          EndIf
        Else
          ; Call without arguments, but with a possible argument sharing:
          _P()
          ProcedureReturn Call(*Object, 0, *This)
        EndIf
      Default
        ProcedureReturn PushOValue(*Object, #True)
    EndSelect
  Else
    ; New defined object value:
    _Inv(*Object)
    Select *Object\Type
      Case #Object_Variable
        ; Return the value of the variable:
        ProcedureReturn *Object\Variable\Value
    EndSelect
    ProcedureReturn PushOValue(*Object, #True)
  EndIf
EndProcedure
Procedure.i RunClass()
  Protected *Token.Token, *Class.Object
  
  _SN(*Token)
  Select *Token\Type
    Case #Token_Name
      *Class = StandardClass(*Token\Name)
      If Not *Class
        *Class = RunObjectPath(*Current\Object, #RunPath_Global, *Token\Name, 0, 0, #False)
        If Not *Class
          _Error("Class not found: " + #Q + ObjectPath(*Current\Object) + "\" + *Token\Name + #Q)
        ElseIf *Class\Type <> #Object_Group
          _Error("Only groups are as classes allowed: " + #Q + ObjectPath(*Class) + #Q)
        EndIf
      EndIf
    Case #Token_Block
      Select *Token\Block\Bracket
        Case #Bracket_Round
          OpenLevel(*Token\Block\TokenQueue, 0, 0, 0, #True)
            *Class = ValueP(Run())
          CloseLevel()
          _E()
          If *Class\Type <> #Object_Group
            _Error("Only groups are as classes allowed: " + #Q + ObjectPath(*Class) + #Q)
          EndIf
        Default
          _Error("Invalid brackets used for classing: " + #Q + Bracket(*Token\Block\Bracket) + "..." + ClosingBracket(*Token\Block\Bracket) + #Q)
      EndSelect
  EndSelect
  
  ProcedureReturn *Class
EndProcedure

Procedure.i NextArgument()
  Protected *Token.Token
  
  _SN(*Token)
  If *Token\Type = #Token_Seperator And *Token\ID = #Seperator_Argument
    ; An argument is following:
    ProcedureReturn #True
  Else
    ; No arguments are following:
    _P()
    ProcedureReturn #False
  EndIf
EndProcedure
Procedure.i Ignore()
  Protected *Token.Token
  
  Repeat
    _SN(*Token)
    If *Token\Type = #Token_Seperator
      _P()
      ProcedureReturn
    EndIf
  ForEver
EndProcedure

Procedure.i Enumerate(*First.Value, *Last.Value)
  Protected *ResultQueue.Queue, FirstInteger.i, LastInteger.i, *Index.Value
  
  If *First\Class = *Last\Class
    *ResultQueue = CreateQueue(@FreeValue())
    
    If *First\Class = *IntegerClass
      FirstInteger = *First\Integer\I
      LastInteger  = *Last\Integer\I
      
      ; Enumeration with integers:
      If FirstInteger < LastInteger
        ; Forwards:
        Repeat
          AppendQueueItem(*ResultQueue, CreateIValue(FirstInteger))
          FirstInteger + 1
        Until FirstInteger > LastInteger
      Else
        ; Backwards:
        Repeat
          AppendQueueItem(*ResultQueue, CreateIValue(FirstInteger))
          FirstInteger - 1
        Until FirstInteger < LastInteger
      EndIf
    Else
      ; Enumeration with unknow classes:
      *Index = DuplicateValue(*First) : _E()
      If IsValueLess(*First, *Last)
        ; Forwards:
        Repeat
          AppendQueueItem(*ResultQueue, DuplicateValue(*Index))
          IncrementValue(*Index) : _E()
        Until IsValueGreater(*Index, *Last)
      Else
        ; Backwards:
        Repeat
          AppendQueueItem(*ResultQueue, DuplicateValue(*Index))
          DecrementValue(*Index) : _E()
        Until IsValueLess(*Index, *Last)
      EndIf
    EndIf
    
    ProcedureReturn *ResultQueue
  Else
    _Error("Only two equal classes are for enumerations allowed: " + #Q + ClassName(*First\Class) + #Q + "..." + #Q + ClassName(*Last\Class) + #Q)
  EndIf
EndProcedure
; IDE Options = PureBasic 4.41 (Windows - x86)
; CursorPosition = 553
; FirstLine = 526
; Folding = ----
; EnableXP
; CompileSourceDirectory