;**************************************************************
;     Initializations
;
; Author: Josef Sniatecki
; Creation date: 03.04.2009
;**************************************************************
;   Copyright (c) 2009 Josef Sniatecki
;
; This program is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License
; As published by the Free Software Foundation; either version
; 3 of the License, Or (at your option) any later version.
;
; This program is distributed in the hope that it will be
; useful, but WITHOUT ANY WARRANTY; without even the implied
; warranty of MERCHANTABILITY Or FITNESS For A PARTICULAR
; PURPOSE. See the GNU General Public License For more details.
;
; You should have received a copy of the GNU General Public
; License along With this program;
; If Not, see "http://www.gnu.org/licenses/".
;**************************************************************



; Levels:
Global Main.Level
Global Preprocessor.Level
Global *Current.Level = @Main

; States:
Global Error.i
Global ErrorMessage.s
Global Truncation.i

; Keywords:
Global Dim Keyword.s(#KeywordCount)
  ExtractSArray(Keyword(), ?Keywords, ?EndKeywords)

; Class methods:
Global Dim ClassMethod.s(#ClassMethodCount)
  ExtractSArray(ClassMethod(), ?ClassMethods, ?EndClassMethods)

; Operators:
Global Dim Operator.s(#OperatorCount)
Global Dim OperatorFunction.i(#OperatorCount)
Global Dim OperatorMethod.i(#OperatorCount)
Global Dim OperatorFriendMethod.i(#OperatorCount)
Global Dim OperatorPriority.i(#OperatorCount)
Global Dim OperatorLeftValue.i(#OperatorCount)
Global Dim OperatorCompare.i(#OperatorCount)

ExtractSArray(Operator()            , ?Operators            , ?EndOperators)
ExtractIArray(OperatorFunction()    , ?OperatorFunctions    , ?EndOperatorFunctions)
ExtractIArray(OperatorMethod()      , ?OperatorMethods      , ?EndOperatorMethods)
ExtractIArray(OperatorFriendMethod(), ?OperatorFriendMethods, ?EndOperatorFriendMethods)
ExtractIArray(OperatorPriority()    , ?OperatorPriorities   , ?EndOperatorPriorities)
ExtractIArray(OperatorLeftValue()   , ?OperatorLeftValues   , ?EndOperatorLeftValues)
ExtractIArray(OperatorCompare()     , ?OperatorCompare      , ?EndOperatorCompare)

; Brackets:
Global Dim Bracket.s(#BracketCount)
  ExtractSArray(Bracket(), ?Brackets, ?EndBrackets)
Global Dim ClosingBracket.s(#BracketCount)
  ExtractSArray(ClosingBracket(), ?ClosingBrackets, ?EndClosingBrackets)

; Symbols:
Global Dim Symbol.s(#SymbolCount)
  ExtractSArray(Symbol(), ?Symbols, ?EndSymbols)

; Main:
Main\TokenQueue    = CreateQueue(@FreeToken()) ;Allocate an empty token queue
Main\Object        = CreateGroup("main")       ;Create the parent object for all other objects
Main\ArgumentQueue = CreateQueue()             ;Create empty argument queue

CreateReferenceObject("main", Main\Object, Main\Object) ;Create a reference to itself

; Classes:
Global *StandardClassLibrary.Object = CreateGroup("stdClass")
  Global *IntegerClass.Object       = CreateGroup("integer"      , *StandardClassLibrary)
  Global *FloatClass.Object         = CreateGroup("float"        , *StandardClassLibrary)
  Global *QuadClass.Object          = CreateGroup("quad"         , *StandardClassLibrary)
  Global *DoubleClass.Object        = CreateGroup("double"       , *StandardClassLibrary)
  Global *BooleanClass.Object       = CreateGroup("boolean"      , *StandardClassLibrary)
  Global *StringClass.Object        = CreateGroup("string"       , *StandardClassLibrary)
  Global *StringPointerClass.Object = CreateGroup("stringPointer", *StandardClassLibrary)
  Global *QueueClass.Object         = CreateGroup("queue"        , *StandardClassLibrary)
  Global *ArrayClass.Object         = CreateGroup("array"        , *StandardClassLibrary)
  Global *BlockClass.Object         = CreateGroup("block"        , *StandardClassLibrary)
  Global *TokenClass.Object         = CreateGroup("token"        , *StandardClassLibrary)
  Global *PointerClass.Object       = CreateGroup("pointer"      , *StandardClassLibrary)
  Global *ObjectClass.Object        = CreateGroup("object"       , *StandardClassLibrary)
  Global *VariantClass.Object       = CreateGroup("variant"      , *StandardClassLibrary)
  Global *ReferenceClass.Object     = CreateGroup("reference"    , *StandardClassLibrary)

ExtractObject(*IntegerClass      , ?IntegerClass)
ExtractObject(*FloatClass        , ?FloatClass)
ExtractObject(*QuadClass         , ?QuadClass)
ExtractObject(*DoubleClass       , ?DoubleClass)
ExtractObject(*BooleanClass      , ?BooleanClass)
ExtractObject(*StringClass       , ?StringClass)
ExtractObject(*StringPointerClass, ?StringPointerClass)
ExtractObject(*QueueClass        , ?QueueClass)
ExtractObject(*ArrayClass        , ?ArrayClass)
ExtractObject(*BlockClass        , ?BlockClass)
ExtractObject(*TokenClass        , ?TokenClass)
ExtractObject(*PointerClass      , ?PointerClass)
ExtractObject(*ObjectClass       , ?ObjectClass)
ExtractObject(*VariantClass      , ?VariantClass)
ExtractObject(*ReferenceClass    , ?ReferenceClass)

; Standard-library:
Global *StandardLibrary.Object = CreateGroup("std")
CreateReferenceObject("std", *StandardLibrary, Main\Object)
ExtractObject(*StandardLibrary, ?StandardLibrary)
ImportGroup(*StandardClassLibrary, *StandardLibrary)

Global *FileStreamClass.Object = GetObject(*StandardLibrary, "fileStream")
Global *ProgramClass.Object    = GetObject(*StandardLibrary, "program")

; Preprocessor:
Global *PreprocessorObject.Object = CreateGroup("pre")
CreateReferenceObject("pre", *PreprocessorObject, *StandardLibrary)
ExtractObject(*PreprocessorObject, ?PreprocessorObject)

; Output & Input:
Global Console.i
Global GUI.i
Global *MasterCallback
Global OutputSymbols.i = #Enabled

; Options & Interface:
  ; User:
  Global User_Name.s
  Global User_Language.s
  Global User_New.i
  ; Preview:
  Global Preview_Display.i
  Global Preview_Font.s
  Global Preview_FontSize.i
  ; Console:
  Global Console_FrontColor.i
  Global Console_BackColor.i
  ; GUI:
  Global GUI_DisplayInput.i
  Global GUI_FrontColor.i
  Global GUI_BackColor.i
  Global GUI_Font.s
  Global GUI_FontSize.i
  Global GUI_Width.i
  Global GUI_Height.i
  Global GUI_Coloring.i = #True
  ; Misc:
  Global Record.s

; HTML-Generator:
Global HTMLGenerator_Font.s
Global Dim HTMLGenerator_Tag.HTMLGenerator_Tag(#HTMLGenerator_TagCount)
Global Dim HTMLGenerator_TagName.s(#HTMLGenerator_TagCount)
  ExtractSArray(HTMLGenerator_TagName(), ?HTMLGenerator_TagNames, ?EndHTMLGenerator_TagNames)
Global Dim HTMLGenerator_Example.s(#HTMLGenerator_TagCount)
  ExtractSArray(HTMLGenerator_Example(), ?HTMLGenerator_Examples, ?EndHTMLGenerator_Examples)

; Load options:
LoadOptions()
InitRecord()

; Error:
CompilerIf #PB_Compiler_Debugger = #False
  OnErrorCall(@FatalErrorCallback())
CompilerEndIf
; IDE Options = PureBasic 4.41 (Windows - x86)
; CursorPosition = 105
; FirstLine = 81
; EnableXP