;**************************************************************
;     Level
;
; Autor: Josef Sniatecki
; Creation date: 08.04.2009
;**************************************************************
;   (c) 2009 Josef Sniatecki
;
; This program is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License
; As published by the Free Software Foundation; either version
; 3 of the License, Or (at your option) any later version.
;
; This program is distributed in the hope that it will be
; useful, but WITHOUT ANY WARRANTY; without even the implied
; warranty of MERCHANTABILITY Or FITNESS For A PARTICULAR
; PURPOSE. See the GNU General Public License For more details.
;
; You should have received a copy of the GNU General Public
; License along With this program;
; If Not, see "http://www.gnu.org/licenses/".
;**************************************************************



Procedure.i OpenLevel(*TokenQueue.Queue, *ArgumentQueue.Queue, *Object.Object, *This.Value, SharedStack.i = #False, Super.i = #False)
  Protected *Previous.Level
  
  ; Allocate new level:
  *Previous         = *Current
  *Current          = _New(Level)
  *Current\Previous = *Previous
  ; Init token queue:
  If *TokenQueue
    *Current\TokenQueue = *TokenQueue
    *Current\TokenItem = *Current\TokenQueue\First
  Else
    *Current\TokenQueue = *Previous\TokenQueue
    *Current\TokenItem = *Previous\TokenItem
  EndIf
  ; Init argument queue:
  If *ArgumentQueue > 0
    *Current\ArgumentQueue = *ArgumentQueue
  ElseIf *ArgumentQueue
    *Current\ArgumentQueue = CreateQueue(#Queue_SaveMemory)
  Else
    *Current\ArgumentQueue = *Previous\ArgumentQueue
  EndIf
  ; Init object:
  If *Object
    *Current\Object = *Object
  Else
    *Current\Object = *Previous\Object
  EndIf
  ; Init "this" value:
  If *This
    *Current\This = *This
  Else
    *Current\This = *Previous\This
  EndIf
  ; Init the stack:
  If SharedStack
    If *Previous\Stack
      *Current\SharedStack = *Previous\Stack
    Else
      *Current\SharedStack = #True
    EndIf
    *Current\Stack = *Previous\Stack
  EndIf
  ; Init super object:
  If Super
    *Current\Super = *Current
  Else
    *Current\Super = *Previous\Super
  EndIf
  
  ProcedureReturn *Current
EndProcedure
Procedure.i CloseLevel()
  Protected *Previous.Level
  
  ; Get previous level:
  *Previous = *Current\Previous
  ; Change the current token queue:
  If *Current\TokenQueue = *Previous\TokenQueue
    *Previous\TokenItem = *Current\TokenItem
  EndIf
  ; Free the current argument queue:
  If *Current\ArgumentQueue <> *Previous\ArgumentQueue
    FreeQueue(*Current\ArgumentQueue)
  EndIf
  ; Clear the current stack:
  If *Current\SharedStack
    *Previous\Stack = *Current\Stack
  Else
    ClearStack()
  EndIf
  ; Change the current level:
  _Free(*Current)
  *Current = *Previous
  
  ProcedureReturn *Current
EndProcedure
Procedure.i ClearLevels()
  While *Current <> @Main
    CloseLevel()
  Wend
EndProcedure
Procedure.i AddStackValue()
  Protected *Stack.Stack
  
  *Stack             = _New(Stack)
  *Stack\Previous    = *Current\Stack
  *Stack\Value       = _New(Value)
  *Stack\Value\Scope = #Value_Stack
  *Current\Stack     = *Stack
  
  ProcedureReturn *Stack\Value
EndProcedure
Procedure.i ClearStack()
  Protected *Stack.Stack
  
  *Stack = *Current\Stack
  While *Stack And *Stack <> *Current\SharedStack
    FreeHardValue(*Stack\Value)
    _Move(*Stack, Previous)
    _Free(*Current\Stack)
    *Current\Stack = *Stack
  Wend
EndProcedure

Procedure.i SearchLevels(Name.s)
  Protected *Object.Object, *Level.Level, *Previous.Level
  
  *Level    = *Current\Previous
  *Previous = *Current
  
  Repeat
    If *Level = @Main Or *Level = *Current\Super
      ProcedureReturn #Null
    ElseIf *Previous\Object <> *Level\Object
      *Object = FindObject(*Level\Object, Name)
      If *Object
        ProcedureReturn *Object
      EndIf
    EndIf
    *Previous = *Level
    _Move(*Level, Previous)
  ForEver
EndProcedure

; IDE Options = PureBasic 4.40 (Windows - x86)
; CursorPosition = 1
; Folding = --
; EnableXP
; CompileSourceDirectory