;**************************************************************
;     Library
;
; Autor: Josef Sniatecki
; Creation date: 05.04.2009
;**************************************************************
;   (c) 2009 Josef Sniatecki
;
; This program is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License
; As published by the Free Software Foundation; either version
; 3 of the License, Or (at your option) any later version.
;
; This program is distributed in the hope that it will be
; useful, but WITHOUT ANY WARRANTY; without even the implied
; warranty of MERCHANTABILITY Or FITNESS For A PARTICULAR
; PURPOSE. See the GNU General Public License For more details.
;
; You should have received a copy of the GNU General Public
; License along With this program;
; If Not, see "http://www.gnu.org/licenses/".
;**************************************************************
; Here are all intern functions and some classes of operno.
; Each object is listed in a group, named "std". But this
; group is declared as global. So you can access any object
; only by it's name.



;******************************************
; Functions
;******************************************

; Value:
Procedure.i Std_Value_Class()
  Protected *Value.Value
  
  If PopValueAddress(@*Value)
    ProcedureReturn PushPValue(*Value\Class)
  EndIf
EndProcedure
Procedure.i Std_Value_Duplicate()
  Protected *Value.Value
  
  If PopValueAddress(@*Value)
    ProcedureReturn DuplicateValue(*Value)
  EndIf
EndProcedure
Procedure.i Std_Value_SetReference()
  Protected *Value.Value, *This.Value
  
  If PopValueAddress(@*This)
    If *This\Scope&#Value_Reference
      If PopValueAddress(@*Value)
        If *This\Original
          FreeReference(*This)
        EndIf
        PrepareReference(*This, *Value)
        *This\Class    = *Value\Class
        *This\Memory   = *Value\Memory
        *This\Scope    = *Value\Scope|#Value_Reference
        *This\Variable = *Value\Variable
      EndIf
    Else
      _Error("Only references can use " + #Q + "setReference()" + #Q + "!")
    EndIf
  EndIf
EndProcedure
Procedure.i Std_Value_Swap()
  Protected *First.Value, *Second.Value, *Temp.Value
  
  PopValueAddress(@*First)
  PopValueAddress(@*Second)
  
  *Temp = DuplicateValue(*First)
  SetValue(*First, *Second)
  SetValue(*Second, *Temp)
  
  FreeValue(*Temp)
EndProcedure
Procedure.i Std_Value_SwapHeaders()
  Protected *First.Value, *Second.Value
  
  PopValueAddress(@*First)
  PopValueAddress(@*Second)
  
  Swap *First\Class, *Second\Class
  Swap *First\Memory, *Second\Memory
  Swap *First\Scope, *Second\Scope
EndProcedure
Procedure.i Std_Value_Name()
  Protected *Value.Value
  
  If PopValueAddress(@*Value)
    ProcedureReturn PushSValue(ValueName(*Value))
  Else
    _ErrorP(1)
  EndIf
EndProcedure
Procedure.i Std_Value_Variable()
  Protected *Value.Value
  
  If PopValueAddress(@*Value)
    If *Value\Variable
      ProcedureReturn PushPValue(*Value\Variable)
    Else
      ProcedureReturn PushPValue(#Null)
    EndIf
  Else
    _ErrorP(1)
  EndIf
EndProcedure

; General:
Procedure.i Std_Arguments()
  ProcedureReturn PushQueueValue(*Current\ArgumentQueue, #True)
EndProcedure
Procedure.i Std_Yield()
  Protected *TokenItem.QueueItem, *Token.Token, *Result.Value
  
  If *Current\Previous And *Current\Previous\Super
    *TokenItem = *Current\Previous\Super\Previous\TokenItem
    *Token     = *TokenItem\Memory
    
    If *Token\Type = #Token_Block And *Token\Block\Bracket = #Bracket_Curly
      *Result = PushBlockValue(*Token\Block\TokenQueue, #True)
      *Current\Previous\Super\Previous\TokenItem = *TokenItem\N
    Else
      *Result = PushBlockValue()
    EndIf
    
    ProcedureReturn *Result
  Else
    _Error("Only allowed within a function!")
  EndIf
EndProcedure

; Math:
Procedure.i Std_Math_Sin()
  Protected Angle.d
  
  If PopDValue(@Angle)
    ProcedureReturn PushDValue(Sin(Angle))
  Else
    _ErrorP(1)
  EndIf
EndProcedure
Procedure.i Std_Math_Cos()
  Protected Angle.d
  
  If PopDValue(@Angle)
    ProcedureReturn PushDValue(Cos(Angle))
  Else
    _ErrorP(1)
  EndIf
EndProcedure
Procedure.i Std_Math_Tan()
  Protected Angle.d
  
  If PopDValue(@Angle)
    ProcedureReturn PushDValue(Tan(Angle))
  Else
    _ErrorP(1)
  EndIf
EndProcedure
Procedure.i Std_Math_Mod()
  Protected *Number1.Value, Divisor.i
  
  If PopValueAddress(@*Number1) And PopIValue(@Divisor)
    Select *Number1\Class
      Case *IntegerClass
        ProcedureReturn PushIValue(*Number1\Integer\I % Divisor)
      Case *QuadClass
        ProcedureReturn PushQValue(*Number1\Quad\Q % Divisor)
      Default
        _Error("The first argument must be an integer or quad.")
    EndSelect
  Else
    _ErrorP(2)
  EndIf
EndProcedure
Procedure.i Std_Math_Log()
  Protected Number.f, Base.f
  
  Select *Current\ArgumentQueue\Count
    Case 1
      PopFValue(@Number)
      ProcedureReturn PushFValue(Log(Number))
    Case 2
      PopFValue(@Number)
      PopFValue(@Base)
      ProcedureReturn PushFValue(Log(Number) / Log(Base))
    Case 0
      _ErrorP(1)
  EndSelect
EndProcedure
Procedure.i Std_Math_ASin()
  Protected Value.d
  
  If PopDValue(@Value)
    ProcedureReturn PushDValue(ASin(Value))
  Else
    _ErrorP(1)
  EndIf
EndProcedure
Procedure.i Std_Math_ACos()
  Protected Value.d
  
  If PopDValue(@Value)
    ProcedureReturn PushDValue(ACos(Value))
  Else
    _ErrorP(1)
  EndIf
EndProcedure
Procedure.i Std_Math_ATan()
  Protected Value.d
  
  If PopDValue(@Value)
    ProcedureReturn PushDValue(ATan(Value))
  Else
    _ErrorP(1)
  EndIf
EndProcedure
Procedure.i Std_Math_Factorial()
  Protected N.i, Result.i
  
  If PopIValue(@N)
    Result = 1
    While N > 1
      Result * N
      N - 1
    Wend
    ProcedureReturn PushIValue(Result)
  Else
    _ErrorP(1)
  EndIf
EndProcedure
Procedure.i Std_Math_Sgn()
  Protected Float.f
  
  If PopFValue(@Float)
    If Float < 0
      ; Negative/Less than 0:
      ProcedureReturn PushIValue(-1)
    ElseIf Float
      ; Positive/Greater than 0:
      ProcedureReturn PushIValue(1)
    Else
      ; Neutral/Equal to 0:
      ProcedureReturn PushIValue(0)
    EndIf
  Else
    _ErrorP(1)
  EndIf
EndProcedure
Procedure.i Std_Math_GGT()
  Protected Number1.i, Number2.i
  
  If PopIValue(@Number1) And PopIValue(@Number2)
    ProcedureReturn PushIValue(GCD(Number1, Number2))
  Else
    _ErrorP(2)
  EndIf
EndProcedure
Procedure.i Std_Math_KGV()
  Protected Number1.i, Number2.i
  
  If PopIValue(@Number1) And PopIValue(@Number2)
    ProcedureReturn PushIValue(LCD(Number1, Number2))
  Else
    _ErrorP(2)
  EndIf
EndProcedure
Procedure.i Std_Math_Random()
  Protected *Value.Value, Minimum.i, Maximum.i
  
  Select *Current\ArgumentQueue\Count
    Case 0
      ProcedureReturn PushIValue(Random(1000000))
    Case 1
      PopValueAddress(@*Value)
      If *Value\Class = *QueueClass
        SelectQueueItem(*Value\Queue, Random(*Value\Queue\Count - 1), @*Value)
        ProcedureReturn *Value
      Else
        ProcedureReturn PushIValue(Random(ValueI(*Value)))
      EndIf
    Case 2
      PopIValue(@Minimum)
      PopIValue(@Maximum)
      ProcedureReturn PushIValue(Random(Maximum - Minimum) + Minimum)
  EndSelect
EndProcedure

; File stream:
Procedure.i Std_FileStream_Read()
  Protected *Class.Object, *Result.Value
  
  If PopPValue(@*Class)
    Select *Class
      Case *IntegerClass
        ProcedureReturn PushIValue(ReadInteger(*Current\This\Integer\I))
      Case *FloatClass
        ProcedureReturn PushFValue(ReadFloat(*Current\This\Integer\I))
      Case *QuadClass
        ProcedureReturn PushQValue(ReadQuad(*Current\This\Integer\I))
      Case *DoubleClass
        ProcedureReturn PushDValue(ReadDouble(*Current\This\Integer\I))
      Case *StringClass
        ProcedureReturn PushSValue(ReadNTString(*Current\This\Integer\I))
      Case *ObjectClass
        *Result = PushOValue(CreateGroup("value"))
        ReadObject(*Current\This\Integer\I, *Result)
        ProcedureReturn *Result
      Default
        *Result = PushValue(*Class)
        CallClassMethod(*Class, #ClassMethod_FileRead, Arguments1(*Current\This), *Result, #False)
        ProcedureReturn *Result
    EndSelect
  EndIf
EndProcedure
Procedure.i Std_FileStream_ReadIn()
  Protected *Value.Value, File.i = *Current\This\Integer\I
  
  While PopValueAddress(@*Value)
    ReadValue(File, *Value) : _E()
  Wend
  ProcedureReturn *Value
EndProcedure
Procedure.i Std_FileStream_Write()
  Protected *Value.Value, File.i = *Current\This\Integer\I
  
  While PopValueAddress(@*Value)
    WriteValue(File, *Value) : _E()
  Wend
  ProcedureReturn *Value
EndProcedure
Procedure.i Std_FileStream_ReadChar()
  ProcedureReturn PushIValue(ReadCharacter(*Current\This\Integer\I))
EndProcedure
Procedure.i Std_FileStream_WriteChar()
  Protected Character.i
  
  If PopIValue(@Character)
    WriteCharacter(*Current\This\Integer\I, Character)
  EndIf
EndProcedure
Procedure.i Std_FileStream_ReadLine()
  ProcedureReturn PushSValue(ReadString(*Current\This\Integer\I))
EndProcedure
Procedure.i Std_FileStream_WriteLine()
  Protected Line.String
  
  If PopSValue(@Line)
    WriteStringN(*Current\This\Integer\I, Line\S)
  EndIf
EndProcedure
Procedure.i Std_FileStream_IsFinished()
  If Eof(*Current\This\Integer\I)
    ProcedureReturn PushBValue(#True)
  Else
    ProcedureReturn PushBValue(#False)
  EndIf
EndProcedure
Procedure.i Std_FileStream_Lines()
  Protected *LineQueue.Queue, File.i = *Current\This\Integer\I
  
  *LineQueue = CreateQueue(@FreeValue())
  
  While Not Eof(File)
    AppendQueueItem(*LineQueue, CreateSValue(ReadString(File)))
  Wend
  
  ProcedureReturn PushQueueValue(*LineQueue)
EndProcedure
Procedure.i Std_FileStream_Create()
  Protected Filename.String
  
  If PopSValue(@Filename)
    *Current\This\Integer\I = CreateFile(#PB_Any, Filename\S)
  EndIf
EndProcedure
Procedure.i Std_FileStream_Open()
  Protected Filename.String
  
  If PopSValue(@Filename)
    *Current\This\Integer\I = ReadFile(#PB_Any, Filename\S)
  EndIf
EndProcedure
Procedure.i Std_FileStream_Close()
  CloseFile(*Current\This\Integer\I)
  *Current\This\Integer\I = 0
EndProcedure
Procedure.i Std_FileStream__New()
  *Current\This\Scope|#Value_Binary
  *Current\This\Integer = _New(Integer)
EndProcedure
Procedure.i Std_FileStream__Def()
  Protected Filename.String
  
  If PopSValue(@Filename)
    *Current\This\Integer\I = OpenFile(#PB_Any, Filename\S)
  EndIf
EndProcedure
Procedure.i Std_FileStream__Del()
  FreeMemory(*Current\This\Integer)
EndProcedure
Procedure.i Std_FileStream__IsTrue()
  If *Current\This\Integer\I
    ProcedureReturn PushBValue(#True)
  Else
    ProcedureReturn PushBValue(#False)
  EndIf
EndProcedure
Procedure.i Std_FileStream__Set()
  Protected *Value.Value
  
  PopValueAddress(@*Value)
  Select *Value\Class
    Case *IntegerClass, *Current\This\Class
      *Current\This\Integer\I = *Value\Integer\I
  EndSelect
EndProcedure

; System:
Procedure.i Std_System_Wait()
  Protected Time.i, StartTime.i
  
  If *Current\ArgumentQueue\Count
    PopIValue(@Time)
  Else
    Time = 16
  EndIf
  
  Wait(Time)
EndProcedure
Procedure.i Std_System_Beep()
  Protected Frequency.i,  Duration.i
  
  Select *Current\ArgumentQueue\Count
    Case 2
      PopIValue(@Frequency)
      PopIValue(@Duration)
    Case 1
      PopIValue(@Frequency)
      Duration = 500
    Default
      Frequency = 1000
      Duration = 500
  EndSelect
  Beep_(Frequency, Duration)
EndProcedure
Procedure.i Std_System_Message()
  Protected Title.String, Text.String, Flags.i
  
  Select *Current\ArgumentQueue\Count
    Case 1
      Title\S = ""
      PopSValue(@Text)  : _E()
    Case 2
      PopSValue(@Title) : _E()
      PopSValue(@Text)  : _E()
    Default
      PopSValue(@Title) : _E()
      PopSValue(@Text)  : _E()
      PopIValue(@Flags) : _E()
  EndSelect
  
  ProcedureReturn PushIValue(MessageRequester(Title\S, Text\S, Flags))
EndProcedure
Procedure.i Std_System_Input()
  Protected Title.String, Text.String, Input.String
  
  Select *Current\ArgumentQueue\Count
    Case 0
      Title\S = "Input"
      Text\S = "Enter a string:"
    Case 1
      Title\S = ""
      PopSValue(@Text)  : _E()
    Case 2
      PopSValue(@Title) : _E()
      PopSValue(@Text)  : _E()
    Default
      PopSValue(@Title) : _E()
      PopSValue(@Text)  : _E()
      PopSValue(@Input) : _E()
  EndSelect
  
  ProcedureReturn PushSValue(InputRequester(Title\S, Text\S, Input\S))
EndProcedure

; Program:
Procedure.i Std_Program_Await()
  WaitProgram(*Current\This\Integer\I)
EndProcedure
Procedure.i Std_Program_Close()
  Protected Program.i = *Current\This\Integer\I
  
  If Program And IsProgram(Program)
    KillProgram(Program)
    CloseProgram(Program)
  EndIf
  *Current\This\Integer\I = #Null
EndProcedure
Procedure.i Std_Program_Run()
  Protected Name.String, Parameters.String
  
  If PopSValue(@Name)
    PopSValue(@Parameters)
    
    RunProgram(Name\S, Parameters\S, "")
  EndIf
EndProcedure
Procedure.i Std_Program__New()
  *Current\This\Scope|#Value_Binary
  *Current\This\Integer = _New(Integer)
EndProcedure
Procedure.i Std_Program__DefAndOpen()
  Protected Name.String, Parameter.String
  
  Select *Current\ArgumentQueue\Count
    Case 1
      PopSValue(@Name)
      Parameter\S = ""
    Case 2
      PopSValue(@Name)
      PopSValue(@Parameter)
  EndSelect
  
  *Current\This\Integer\I = RunProgram(Name\S, Parameter\S, "", #PB_Program_Open)
EndProcedure
Procedure.i Std_Program__Del()
  Protected Program.i = *Current\This\Integer\I
  
  If Program And IsProgram(Program)
    KillProgram(Program)
    CloseProgram(Program)
  EndIf
  FreeMemory(*Current\This\Integer)
EndProcedure
Procedure.i Std_Program__IsTrue()
  If *Current\This\Integer\I And ProgramRunning(*Current\This\Integer\I)
    ProcedureReturn PushBValue(#True)
  Else
    ProcedureReturn PushBValue(#False)
  EndIf
EndProcedure
Procedure.i Std_Program__Set()
  Protected *Value.Value
  
  PopValueAddress(@*Value)
  Select *Value\Class
    Case *IntegerClass, *Current\This\Class
      *Current\This\Integer\I = *Value\Integer\I
    Case *StringClass
      *Current\This\Integer\I = RunProgram(*Value\String\S)
  EndSelect
EndProcedure

; Special:
Procedure.i Std_Import()
  Protected Library.i, Name.String
  
  While PopSValue(@Name)
    ImportLibrary(Name\S)
  Wend
EndProcedure
Procedure.i Std_ExecuteFile()
  Protected Name.String
  
  While PopSValue(@Name)
    ExecuteFile(Name\S)
  Wend
EndProcedure
Procedure.i Std_CurrentBlock()
  ProcedureReturn PushBlockValue(*Current\TokenQueue, #True)
EndProcedure

; Stream:
Procedure.i Std_Stream_Send()
  Protected Text.String, Symbol.i, Indent.i
  
  Text\S = ""
  Symbol = #Symbol_Output
  Indent = 0
  
  Select *Current\ArgumentQueue\Count
    Case 1
      PopSValue(@Text)
    Case 2
      PopIValue(@Symbol)
      PopSValue(@Text)
    Case 3
      PopIValue(@Symbol)
      PopSValue(@Text)
      PopIValue(@Indent)
  EndSelect
  
  OutputFL(Symbol, Text\S, Indent)
EndProcedure
Procedure.i Std_Stream_Get()
  Protected Message.String, Symbol.i, Indent.i
  
  Message\S = ""
  Symbol = #Symbol_Input
  Indent = 0
  
  Select *Current\ArgumentQueue\Count
    Case 1
      PopSValue(@Message)
    Case 2
      PopIValue(@Symbol)
      PopSValue(@Message)
    Case 3
      PopIValue(@Symbol)
      PopSValue(@Message)
      PopIValue(@Indent)
  EndSelect
  
  ProcedureReturn PushSValue(InputFLT(Symbol, Message\S, Indent))
EndProcedure
Procedure.i Std_Stream_GetCode()
  Protected Code.s, Indent.i
  
  If *Current\ArgumentQueue\Count = 1
    PopIValue(@Indent)
  EndIf
  
  Code = InputC(Indent) : _E()
  ProcedureReturn PushSValue(Code)
EndProcedure
Procedure.i Std_Stream_Request()
  Protected Text.String, Indent.i
  
  Text\S = ""
  Indent = 0
  
  Select *Current\ArgumentQueue\Count
    Case 1
      PopSValue(@Text)
    Case 2
      PopSValue(@Text)
      PopIValue(@Indent)
  EndSelect
  
  ProcedureReturn PushBValue(InputYN(Text\S, Indent))
EndProcedure
Procedure.i Std_Stream_Wait()
  WaitForReturn()
EndProcedure
Procedure.i Std_Stream_Write()
  Protected Text.String, Symbol.i, Indent.i
  
  Text\S = ""
  Symbol = #Symbol_Output
  Indent = 0
  
  Select *Current\ArgumentQueue\Count
    Case 1
      PopSValue(@Text)
    Case 2
      PopIValue(@Symbol)
      PopSValue(@Text)
    Case 3
      PopIValue(@Symbol)
      PopSValue(@Text)
      PopIValue(@Indent)
  EndSelect
  
  OutputFLW(Symbol, Text\S, Indent)
EndProcedure
Procedure.i Std_Stream_Clear()
  If GUI
    SetGadgetText(#GUI_Output, "")
  ElseIf Console
    EnableGraphicalConsole(#True)
    ClearConsole()
    EnableGraphicalConsole(#False)
  ElseIf *MasterCallback
    _Call(*MasterCallback, #Callback_ClearOutput, 0, 0)
  EndIf
EndProcedure
Procedure.i Std_Stream_OutputSymbols()
  PopBValue(@OutputSymbols)
EndProcedure
Procedure.i Std_Stream_OpenGUI()
  OpenGUIWindow()
EndProcedure
Procedure.i Std_Stream_OpenConsole()
  OpenConsole()
  Console = #True
  
  ConsoleTitle("Operno - " + #Version)
EndProcedure

; Coordinate System:
IncludeFile "Sources\CoordinateSystem.pbi"


;******************************************
; Table
;******************************************

DataSection
  StandardLibrary:
    Data.c #Object_Group : Data.s "value" ;{
      Data.c #Object_Function : Data.s "class" ;{
        Data.i @Std_Value_Class()
      ;}
      Data.c #Object_Function : Data.s "duplicate" ;{
        Data.i @Std_Value_Duplicate()
      Data.s "dup"
      ;}
      Data.c #Object_Function : Data.s "setReference" ;{
        Data.i @Std_Value_SetReference()
      Data.s "setRef"
      ;}
      Data.c #Object_Function : Data.s "swap" ;{
        Data.i @Std_Value_Swap()
      ;}
      Data.c #Object_Function : Data.s "swapHeaders" ;{
        Data.i @Std_Value_SwapHeaders()
      Data.s "swapH"
      ;}
      Data.c #Object_Function : Data.s "name" ;{
        Data.i @Std_Value_Name()
      ;}
      Data.c #Object_Function : Data.s "variable" ;{
        Data.i @Std_Value_Variable()
      Data.s "var"
      ;}
      Data.c #Null
    ;}
    Data.c #Object_Function : Data.s "arguments" ;{
      Data.i @Std_Arguments()
    Data.s "args"
    ;}
    Data.c #Object_Group : Data.s "math" ;{
      Data.c #Object_Function : Data.s "sin" ;{
        Data.i @Std_Math_Sin()
      ;}
      Data.c #Object_Function : Data.s "cos" ;{
        Data.i @Std_Math_Cos()
      ;}
      Data.c #Object_Function : Data.s "tan" ;{
        Data.i @Std_Math_Tan()
      ;}
      Data.c #Object_Function : Data.s "mod" ;{
        Data.i @Std_Math_Mod()
      ;}
      Data.c #Object_Function : Data.s "log" ;{
        Data.i @Std_Math_Log()
      ;}
      Data.c #Object_Function : Data.s "asin" ;{
        Data.i @Std_Math_ASin()
      ;}
      Data.c #Object_Function : Data.s "acos" ;{
        Data.i @Std_Math_ACos()
      ;}
      Data.c #Object_Function : Data.s "atan" ;{
        Data.i @Std_Math_ATan()
      ;}
      Data.c #Object_Function : Data.s "fac" ;{
        Data.i @Std_Math_Factorial()
      ;}
      Data.c #Object_Function : Data.s "sgn" ;{
        Data.i @Std_Math_Sgn()
      ;}
      Data.c #Object_Function : Data.s "ggt" ;{
        Data.i @Std_Math_GGT()
      ;}
      Data.c #Object_Function : Data.s "kgv" ;{
        Data.i @Std_Math_KGV()
      ;}
      Data.c #Object_Function : Data.s "random" ;{
        Data.i @Std_Math_Random()
      Data.s "rand"
      ;}
      Data.c #Null
    ;}
    Data.c #Object_Group : Data.s "fileStream" ;{
      Data.c #Object_Function : Data.s "read" ;{
        Data.i @Std_FileStream_Read()
      ;}
      Data.c #Object_Function : Data.s "readIn" ;{
        Data.i @Std_FileStream_ReadIn()
      ;}
      Data.c #Object_Function : Data.s "write" ;{
        Data.i @Std_FileStream_Write()
      ;}
      Data.c #Object_Function : Data.s "readChar" ;{
        Data.i @Std_FileStream_ReadChar()
      ;}
      Data.c #Object_Function : Data.s "writeChar" ;{
        Data.i @Std_FileStream_WriteChar()
      ;}
      Data.c #Object_Function : Data.s "readLine" ;{
        Data.i @Std_FileStream_ReadLine()
      ;}
      Data.c #Object_Function : Data.s "writeLine" ;{
        Data.i @Std_FileStream_WriteLine()
      ;}
      Data.c #Object_Function : Data.s "lines" ;{
        Data.i @Std_FileStream_Lines()
      ;}
      Data.c #Object_Function : Data.s "finished?" ;{
        Data.i @Std_FileStream_IsFinished()
      Data.s "eof?"
      ;}
      Data.c #Object_Function : Data.s "create" ;{
        Data.i @Std_FileStream_Create()
      ;}
      Data.c #Object_Function : Data.s "open" ;{
        Data.i @Std_FileStream_Open()
      ;}
      Data.c #Object_Function : Data.s "close" ;{
        Data.i @Std_FileStream_Close()
      ;}
      Data.c #Object_Function : Data.s _CM(new) ;{
        Data.i @Std_FileStream__New()
      ;}
      Data.c #Object_Function : Data.s _CM(def) ;{
        Data.i @Std_FileStream__Def()
      ;}
      Data.c #Object_Function : Data.s _CM(del) ;{
        Data.i @Std_FileStream__Del()
      ;}
      Data.c #Object_Function : Data.s _CM(true?) ;{
        Data.i @Std_FileStream__IsTrue()
      ;}
      Data.c #Object_Function : Data.s _CM(set) ;{
        Data.i @Std_FileStream__Set()
      ;}
      Data.c #Object_Function : Data.s _CM(fAdd) ;{
        Data.i @Std_FileStream_ReadIn()
      ;}
      Data.c #Object_Function : Data.s _CM(add) ;{
        Data.i @Std_FileStream_Write()
      ;}
      Data.c #Null
    Data.s "fstr"
    ;}
    Data.c #Object_Group : Data.s "system" ;{
      Data.c #Object_Function : Data.s "wait" ;{
        Data.i @Std_System_Wait()
      ;}
      Data.c #Object_Function : Data.s "beep" ;{
        Data.i @Std_System_Beep()
      ;}
      Data.c #Object_Function : Data.s "message" ;{
        Data.i @Std_System_Message()
      Data.s "msg"
      ;}
      Data.c #Object_Function : Data.s "input" ;{
        Data.i @Std_System_Input()
      Data.s "inp"
      ;}
      Data.c #Null
    Data.s "sys"
    ;}
    Data.c #Object_Group : Data.s "program" ;{
      Data.c #Object_Function : Data.s "await" ;{
        Data.i @Std_Program_Await()
      ;}
      Data.c #Object_Function : Data.s "close" ;{
        Data.i @Std_Program_Close()
      ;}
      Data.c #Object_Function : Data.s "open" ;{
        Data.i @Std_Program__DefAndOpen()
      ;}
      Data.c #Object_Function : Data.s "run" ;{
        Data.i @Std_Program_Run()
      ;}
      Data.c #Object_Function : Data.s _CM(new) ;{
        Data.i @Std_Program__New()
      ;}
      Data.c #Object_Function : Data.s _CM(def) ;{
        Data.i @Std_Program__DefAndOpen()
      ;}
      Data.c #Object_Function : Data.s _CM(del) ;{
        Data.i @Std_Program__Del()
      ;}
      Data.c #Object_Function : Data.s _CM(true?) ;{
        Data.i @Std_Program__IsTrue()
      ;}
      Data.c #Object_Function : Data.s _CM(set) ;{
        Data.i @Std_Program__Set()
      ;}
      Data.c #Null
    ;}
    Data.c #Object_Function : Data.s "import" ;{
      Data.i @Std_Import()
    ;}
    Data.c #Object_Function : Data.s "executeFile" ;{
      Data.i @Std_ExecuteFile()
    ;}
    Data.c #Object_Function : Data.s "currentBlock" ;{
      Data.i @Std_CurrentBlock()
    ;}
    Data.c #Object_Group : Data.s "stream" ;{
      Data.c #Object_Function : Data.s "send" ;{
        Data.i @Std_Stream_Send()
      ;}
      Data.c #Object_Function : Data.s "get" ;{
        Data.i @Std_Stream_Get()
      ;}
      Data.c #Object_Function : Data.s "getCode" ;{
        Data.i @Std_Stream_GetCode()
      ;}
      Data.c #Object_Function : Data.s "request" ;{
        Data.i @Std_Stream_Request()
      ;}
      Data.c #Object_Function : Data.s "wait" ;{
        Data.i @Std_Stream_Wait()
      ;}
      Data.c #Object_Function : Data.s "write" ;{
        Data.i @Std_Stream_Write()
      ;}
      Data.c #Object_Function : Data.s "clear" ;{
        Data.i @Std_Stream_Clear()
      ;}
      Data.c #Object_Function : Data.s "outputSymbols" ;{
        Data.i @Std_Stream_OutputSymbols()
      ;}
      Data.c #Object_Function : Data.s "openGUI" ;{
        Data.i @Std_Stream_OpenGUI()
      ;}
      Data.c #Object_Function : Data.s "openConsole" ;{
        Data.i @Std_Stream_OpenConsole()
      ;}
      Data.c #Null
    Data.s "str"
    ;}
    Data.c #Object_Function : Data.s "yield" ;{
      Data.i @Std_Yield()
    ;}
    Data.c #Object_Group : Data.s "coordinateSystem" ;{
      Data.c #Object_Function : Data.s "graph" ;{
        Data.i @Std_CoordSys_Graph()
      ;}
      Data.c #Object_Function : Data.s "point" ;{
        Data.i @Std_CoordSys_Point()
      ;}
      Data.c #Object_Function : Data.s "drawBackground" ;{
        Data.i @Std_CoordSys_DrawBackground()
      ;}
      Data.c #Object_Function : Data.s "drawUnits" ;{
        Data.i @Std_CoordSys_DrawUnits()
      ;}
      Data.c #Object_Function : Data.s "open" ;{
        Data.i @Std_CoordSys_Open()
      ;}
      Data.c #Object_Function : Data.s "save" ;{
        Data.i @Std_CoordSys_Save()
      ;}
      Data.c #Object_Function : Data.s _CM(new) ;{
        Data.i @Std_CoordSys__New()
      ;}
      Data.c #Object_Function : Data.s _CM(def) ;{
        Data.i @Std_CoordSys__Def()
      ;}
      Data.c #Object_Function : Data.s _CM(del) ;{
        Data.i @Std_CoordSys__Del()
      ;}
      Data.c #Null
    Data.s "coordSys"
    ;}
    Data.c #Null
EndDataSection

; IDE Options = PureBasic 4.41 (Windows - x86)
; CursorPosition = 751
; FirstLine = 113
; Folding = AAgAAAAAAAAACYAAAAAAAAAA5
; EnableXP
; CompileSourceDirectory