;**************************************************************
;     Object
;
; Autor: Josef Sniatecki
; Creation date: 07.04.2009
;**************************************************************
;   (c) 2009 Josef Sniatecki
;
; This program is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License
; As published by the Free Software Foundation; either version
; 3 of the License, Or (at your option) any later version.
;
; This program is distributed in the hope that it will be
; useful, but WITHOUT ANY WARRANTY; without even the implied
; warranty of MERCHANTABILITY Or FITNESS For A PARTICULAR
; PURPOSE. See the GNU General Public License For more details.
;
; You should have received a copy of the GNU General Public
; License along With this program;
; If Not, see "http://www.gnu.org/licenses/".
;**************************************************************



; Constructors:
Procedure.i CreateObject(Type.i, Name.s, *Super.Object)
  Protected *Object.Object, Index.i
  
  *Object      = _New(Object)
  *Object\Type = Type
  *Object\Name = Name
  If *Super
    *Object\Super = *Super
    *Object\Item  = AppendQueueItem(*Super\Group\ObjectQueue, *Object)
    Index = ClassMethodID(Name)
    If Index <> -1
      *Super\Group\ClassMethod[Index] = *Object
    EndIf
  EndIf
  ProcedureReturn *Object
EndProcedure
Procedure.i CreateGroup(Name.s, *Super.Object = 0, *Value.Value = 0)
  Protected *Object.Object
  
  *Object                   = CreateObject(#Object_Group, Name, *Super)
  *Object\Group             = _New(Group)
  *Object\Group\ObjectQueue = CreateQueue(@FreeObject())
  *Object\Group\Value       = *Value
  ProcedureReturn *Object
EndProcedure
Procedure.i CreateVirtualTable(TableSize.i)
  Protected *Object.Object
  
  *Object                   = CreateObject(#Object_Group, "value", 0)
  *Object\Group             = _Dim(_Sz(Group) + TableSize)
  *Object\Group\ObjectQueue = CreateQueue(@FreeObject())
  ProcedureReturn *Object
EndProcedure
Procedure.i CreateFunction(Name.s, *Super.Object = 0)
  Protected *Object.Object
  
  *Object          = CreateObject(#Object_Function, Name, *Super)
  *Object\Function = _New(Function)
  ProcedureReturn *Object
EndProcedure
Procedure.i CreateAddressFunction(Name.s, *Address, *Super.Object = 0)
  Protected *Object.Object
  
  *Object                  = CreateObject(#Object_Function, Name, *Super)
  *Object\Function         = _New(Function)
  *Object\Function\Type    = #Function_Address
  *Object\Function\Address = *Address
  ProcedureReturn *Object
EndProcedure
Procedure.i CreateBlockFunction(Name.s, *TokenQueue.Queue, *Class.Object = 0, *Super.Object = 0)
  Protected *Object.Object
  
  *Object                     = CreateObject(#Object_Function, Name, *Super)
  *Object\Function            = _New(Function)
  *Object\Function\Type       = #Function_Block
  *Object\Function\TokenQueue = *TokenQueue
  *Object\Function\Class      = *Class
  ProcedureReturn *Object
EndProcedure
Procedure.i CreateVariable(Name.s, *Value.Value, *Super.Object = 0)
  Protected *Object.Object
  
  *Object          = CreateObject(#Object_Variable, Name, *Super)
  *Object\Variable = _New(Variable)
  
  If *Value
    *Object\Variable\Value = *Value
    If Not *Value\Variable
      *Value\Variable = *Object
    EndIf
  EndIf
  ProcedureReturn *Object
EndProcedure
Procedure.i CreateReferenceObject(Name.s, *Original.Object, *Super.Object = 0)
  Protected *Object.Object
  
  If *Original\Reference
    _Move(*Original, Reference)
  EndIf
  
  *Object           = CreateObject(*Original\Type, Name, *Super)
  *Object\Memory    = *Original\Memory
  *Object\Reference = *Original
  ProcedureReturn *Object
EndProcedure

; Destructors:
Procedure.i FreeObject(*Object.Object)
  Protected Index.i, *Item.QueueItem, *ParentQueue.Queue
  
  If Not *Object\Reference
    Select *Object\Type
      Case #Object_Group
        FreeGroup(*Object\Group)
      Case #Object_Function
        FreeFunction(*Object\Function)
      Case #Object_Variable
        FreeVariable(*Object\Variable)
    EndSelect
  EndIf
  
  If *Object\Super
    Index = ClassMethodID(*Object\Name)
    If Index <> -1
      *Object\Super\Group\ClassMethod[Index] = 0
    EndIf
    *Item        = *Object\Item
    *ParentQueue = *Object\Super\Group\ObjectQueue
    ClearStructure(*Object, Object)
    FreeQueueItem(*ParentQueue, *Item)
  Else
    ClearStructure(*Object, Object)
    FreeMemory(*Object)
  EndIf
EndProcedure
Procedure.i FreeGroup(*Group.Group)
  Protected *Item.QueueItem, *Object.Object
  
  *Item = *Group\ObjectQueue\First
  While *Item
    *Object = *Item\Memory
    _Move(*Item, N)
    FreeObject(*Object)
  Wend
  _Free(*Group\ObjectQueue)
  _Free(*Group)
EndProcedure
Procedure.i FreeFunction(*Function.Function)
  Select *Function\Type
    Case #Function_Block
      FreeQueue(*Function\TokenQueue)
  EndSelect
  _Free(*Function)
EndProcedure
Procedure.i FreeVariable(*Variable.Variable)
  If *Variable\Value
    FreeHardValue(*Variable\Value)
  EndIf
  _Free(*Variable)
EndProcedure

; Special methods:
Procedure.s GroupVariableName(*Object.Object)
  If *Object\Group\Value
    If *Object\Group\Value\Variable
      ProcedureReturn *Object\Group\Value\Variable\Name
    Else
      ProcedureReturn ClassName(*Object\Group\Value\Class)
    EndIf
  Else
    ProcedureReturn *Object\Name
  EndIf
EndProcedure
Procedure.i PopObject(*Object.Object)
  Protected *Item.QueueItem, *Child.Object
  
  *Item = *Object\Group\ObjectQueue\First
  While *Item And *Current\ArgumentQueue\Count
    *Child = *Item\Memory
    Select *Child\Type
      Case #Object_Variable
        PopValue(*Child\Variable\Value)
      Case #Object_Group
        PopObject(*Child)
    EndSelect
    _Move(*Item, N)
  Wend
EndProcedure
Procedure.i ExtractObject(*Super.Object, *Memory.All)
  Protected *Object
  Protected Integer.i, Name.s
  
  While *Memory\C
    ; Type:
    Integer = *Memory\C : *Memory + _Sz(Character)
    ; Name:
    Name = PeekS(*Memory) : *Memory + Len(Name) + _Sz(Byte)
    ; Select type:
    Select Integer
      Case #Object_Group
        *Object = CreateGroup(Name, *Super)
        *Memory = ExtractObject(*Object, *Memory)
      Case #Object_Function
        Integer = *Memory\I : *Memory + _Sz(Integer) ;= Address
        *Object = CreateAddressFunction(Name, Integer, *Super)
      Case #Object_Variable
        Integer = *Memory\C : *Memory + _Sz(Character) ;= Type
        ; Read value:
        Select Integer
          Case #Type_Integer
            Integer = CreateIValue(*Memory\I) : *Memory + _Sz(Integer)
          Case #Type_Float
            Integer = CreateFValue(*Memory\F) : *Memory + _Sz(Float)
          Case #Type_Quad
            Integer = CreateQValue(*Memory\Q) : *Memory + _Sz(Quad)
          Case #Type_Double
            Integer = CreateDValue(*Memory\D) : *Memory + _Sz(Double)
          Case #Type_Bool
            Integer = CreateBValue(*Memory\B) : *Memory + _Sz(Byte)
          Case #Type_String
            Integer = CreateSValue(PeekS(*Memory)) : *Memory + Len(PeekS(*Memory)) + _Sz(Character)
        EndSelect
        *Object = CreateVariable(Name, Integer, *Super)
    EndSelect
    While *Memory\C > 32
      Name = PeekS(*Memory) : *Memory + Len(Name) + _Sz(Byte)
      CreateReferenceObject(Name, *Object, *Super)
    Wend
  Wend
  
  ProcedureReturn *Memory + _Sz(Byte)
EndProcedure
Procedure.i ImportGroup(*Source.Object, *Destination.Object)
  Protected *Item.QueueItem
  
  *Item = *Source\Group\ObjectQueue\First
  While *Item
    CreateReferenceObject(*Item\Object\Name, *Item\Object, *Destination)
    _Move(*Item, N)
  Wend
EndProcedure
Procedure.s FormatObject(*Object.Object, Indent.i = 0)
  Protected String.s, *Item.QueueItem, *SubObject.Object
  
  String = Space(Indent*#IndentLength)
  
  Select *Object\Type
    Case #Object_Group
      If *Object\Group\ObjectQueue\Count
        String + GroupVariableName(*Object) + "{" + #N
        Indent + 1
        
        *Item = *Object\Group\ObjectQueue\First
        While *Item
          *SubObject = *Item\Memory
          If Not *SubObject\Reference
            String + FormatObject(*SubObject, Indent)
          Else
            String + Space(Indent*#IndentLength) + *SubObject\Name + " -> "
            If *SubObject\Reference\Super = *Object
              String + *SubObject\Reference\Name
            Else
              String + ObjectPath(*SubObject\Reference)
            EndIf
          EndIf
          String + #N
          _Move(*Item, N)
        Wend
        
        Indent-1
        String + Space(Indent*#IndentLength) + "}"
      Else
        String + GroupVariableName(*Object) + "{}"
      EndIf
    Case #Object_Variable
      String + *Object\Name +  " = "
      String + ReplaceString(FormatValue(*Object\Variable\Value), #N, #N + Space(Len(String)))
    Case #Object_Function
      String + *Object\Name + "()"
  EndSelect
  
  ProcedureReturn String
EndProcedure
Procedure.i WriteObject(File.i, *Object.Object)
  Protected *Child.Object, *Item.QueueItem
  
  Select *Object\Type
    Case #Object_Group
      *Item = *Object\Group\ObjectQueue\First
      While *Item
        WriteObject(File, *Item\Object)
        _Move(*Item, N)
      Wend
    Case #Object_Variable
      WriteValue(File, *Object\Variable\Value)
  EndSelect
EndProcedure
Procedure.i ReadObject(File.i, *Object.Object)
  Protected *Child.Object, *Item.QueueItem
  
  Select *Object\Type
    Case #Object_Group
      *Item = *Object\Group\ObjectQueue\First
      While *Item
        ReadObject(File, *Item\Object)
        _Move(*Item, N)
      Wend
    Case #Object_Variable
      ReadValue(File, *Object\Variable\Value)
  EndSelect
EndProcedure

; Searching:
Procedure.i GetObject(*Super.Object, Path.s)
  Protected *Pointer.Character, Name.s
  
  *Pointer = @Path
  Repeat
    Select *Pointer\C
      Case '\'
        *Super = FindObject(*Super, Name)
        If Not *Super
          ProcedureReturn #False
        EndIf
        Name = ""
      Case 0
        ProcedureReturn FindObject(*Super, Name)
      Default
        Name + Chr(*Pointer\C)
    EndSelect
    *Pointer + 1
  ForEver
EndProcedure
Procedure.i FindObject(*Super.Object, Name.s)
  Protected *Item.QueueItem, *Object.Object
  
  *Item = *Super\Group\ObjectQueue\First
  While *Item
    *Object = *Item\Memory
    If *Object\Name = Name
      ProcedureReturn *Object
    EndIf
    _Move(*Item, N)
  Wend
EndProcedure
Procedure.i FindSuperObject(*Object.Object, *Super.Object)
  While *Object\Super
    If *Object\Super = *Super
      ProcedureReturn #True
    Else
      _Move(*Object, Super)
    EndIf
  Wend
  ProcedureReturn #False
EndProcedure
Procedure.s ObjectPath(*Object.Object)
  Protected Path.s
  
  If *Object = Main\Object
    ProcedureReturn "main"
  Else
    Path = *Object\Name
    _Move(*Object, Super)
    While *Object And *Object <> Main\Object And *Object <> *StandardLibrary And *Object <> *StandardClassLibrary
      Path = *Object\Name + "\" + Path
      _Move(*Object, Super)
    Wend
    ProcedureReturn Path
  EndIf
EndProcedure

; IDE Options = PureBasic 4.41 (Windows - x86)
; CursorPosition = 55
; FirstLine = 42
; Folding = ----
; EnableXP
; CompileSourceDirectory