;**************************************************************
;     Operations
;
; Author: Josef Sniatecki
; Creation date: 08.04.2009
;**************************************************************
;   (c) 2009 Josef Sniatecki
;
; This program is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License
; As published by the Free Software Foundation; either version
; 3 of the License, Or (at your option) any later version.
;
; This program is distributed in the hope that it will be
; useful, but WITHOUT ANY WARRANTY; without even the implied
; warranty of MERCHANTABILITY Or FITNESS For A PARTICULAR
; PURPOSE. See the GNU General Public License For more details.
;
; You should have received a copy of the GNU General Public
; License along With this program;
; If Not, see "http://www.gnu.org/licenses/".
;**************************************************************



; Main:
Procedure.i OperateValue(Operation.i, *LHS.Value, *RHS.Value)
  Protected Method.i, *Result.Value
  
  ; Get the method:
  Method = OperatorFunction(Operation)
  ; Method for an Operation given?:
  If Method <> #None
    If *LHS
      If *RHS
        If Method <= #ClassMethodCount
          *Result = CallClassMethod(*LHS\Class, Method, Arguments1(*RHS), *LHS, #True, #False) : _E()
          If Not *Result
            ; Try the friend method of the RHS value:
            Method = OperatorFriendMethod(Operation)
            If Method <> -1
              *Result = CallClassMethod(*RHS\Class, Method, Arguments1(*LHS), *RHS, #True, #False) : _E()
            EndIf
          EndIf
        Else
          *Result = _Call(Method, *LHS, *RHS) : _E()
        EndIf
        If *Result
          ; Operation was valid:
          ProcedureReturn *Result
        EndIf
      Else
        If Method > #ClassMethodCount
          *Result = _Call(Method, *LHS, PushVValue()) : _E()
        Else
          _Error("Value on the right needed: " + #Q + Operator(Operation) + #Q)
        EndIf
        If *Result
          ; Operation was valid:
          ProcedureReturn *Result
        EndIf
      EndIf
    ElseIf *RHS
      If Method > #ClassMethodCount
        *Result = _Call(Method, 0, *RHS) : _E()
        If *Result
          ProcedureReturn *Result
        EndIf
      Else
        _Error("Value on the left needed: " + #Q + Operator(Operation) + #Q)
      EndIf
    Else
      If Method > #ClassMethodCount
        *Result = _Call(Method, 0, *RHS) : _E()
        If *Result
          ProcedureReturn *Result
        EndIf
      Else
        _Error("A operator without values is not allowed: " + #Q + Operator(Operation) + #Q)
      EndIf
    EndIf
  Else
    ; Operator reserved for special Operations:
    _Error("The operator is protected: " + #Q + Operator(Operation) + #Q)
  EndIf
  
  ; Operate vectorial:
  If *LHS Or *RHS
    If *LHS And *LHS\Class\Group\ClassMethod[#ClassMethod_Vector]
      *LHS = CallClassMethod(*LHS\Class, #ClassMethod_Vector, -1, *LHS) : _E()
      If *RHS And *RHS\Class\Group\ClassMethod[#ClassMethod_Vector]
        *RHS = CallClassMethod(*RHS\Class, #ClassMethod_Vector, -1, *RHS) : _E()
      EndIf
      ProcedureReturn OperateValue(Operation, *LHS, *RHS)
    ElseIf *RHS And *RHS\Class\Group\ClassMethod[#ClassMethod_Vector]
      *RHS = CallClassMethod(*RHS\Class, #ClassMethod_Vector, -1, *RHS) : _E()
      If *LHS And *LHS\Class\Group\ClassMethod[#ClassMethod_Vector]
        *LHS = CallClassMethod(*LHS\Class, #ClassMethod_Vector, -1, *LHS) : _E()
      EndIf
      ProcedureReturn OperateValue(Operation, *LHS, *RHS)
    Else
      *Result = OperateValueVectorial(Operation, *LHS, *RHS) : _E()
      If *Result
        ProcedureReturn *Result
      EndIf
    EndIf
  EndIf
  
  If *LHS And *RHS
    _Error("Operation not defined: " + ClassName(*LHS\Class) + " " + Operator(Operation) + " " + ClassName(*RHS\Class))
  ElseIf *LHS
    _Error("Operation not defined: " + ClassName(*LHS\Class) + " " + Operator(Operation))
  ElseIf *RHS
    _Error("Operation not defined: " + Operator(Operation) + " " + ClassName(*RHS\Class))
  Else
    _Error("Operation not defined: " + Operator(Operation))
  EndIf
EndProcedure
Procedure.i OperateValueVectorial(Operation.i, *LHS.Value, *RHS.Value)
  Protected *Object.Object, *LItem.QueueItem, *RItem.QueueItem, *LChild.Object, *RChild.Object
  Protected *Result.Value, *ResultValue.Value, *ResultObject.Object, *ResultQueue.Queue
  
  If *LHS And *RHS
    If *LHS\Scope&#Value_Searchable
      *ResultValue        = AddStackValue()
      *ResultValue\Class  = *LHS\Class
      *ResultObject       = CreateGroup("value", 0, *ResultValue)
      *ResultValue\Memory = *ResultObject
      *ResultValue\Scope  | *LHS\Scope&~(#Value_Reference|#Value_Literal)
      
      If *RHS\Scope&#Value_Searchable
        OperateObjectAndObject(Operation, *LHS\Object, *RHS\Object, *ResultObject)
      ElseIf *RHS\Class = *QueueClass
        OperateObjectAndQueue(Operation, *LHS\Object, *RHS\Queue, *ResultObject)
      Else
        OperateObjectAndValue(Operation, *LHS\Object, *RHS, *ResultObject)
      EndIf
    ElseIf *LHS\Class = *QueueClass
      If *RHS\Scope&#Value_Searchable
        *ResultValue        = AddStackValue()
        *ResultValue\Class  = *RHS\Class
        *ResultObject       = CreateGroup("value", 0, *ResultValue)
        *ResultValue\Memory = *ResultObject
        *ResultValue\Scope  | *RHS\Scope&~(#Value_Reference|#Value_Literal)
        
        OperateQueueAndObject(Operation, *LHS\Queue, *RHS\Object, *ResultObject)
      ElseIf *RHS\Class = *QueueClass
        ; Queue and Queue:
        If *LHS\Queue\Count <> *RHS\Queue\Count
          If Operation = #Operator_Set
            ClearQueue(*LHS\Queue)
            CopyQueue(*RHS\Queue, *LHS\Queue, @CopyValue())
            ProcedureReturn *RHS
          Else
            ProcedureReturn #False
          EndIf
        EndIf
        
        *ResultQueue = CreateQueue(@FreeValue())
        *ResultValue = PushQueueValue(*ResultQueue)
        
        OperateQueueAndQueue(Operation, *LHS\Queue, *RHS\Queue, *ResultQueue)
      Else
        ; Queue and Value:
        *ResultQueue = CreateQueue(@FreeValue())
        *ResultValue = PushQueueValue(*ResultQueue)
        
        OperateQueueAndValue(Operation, *LHS\Queue, *RHS, *ResultQueue)
      EndIf
    ElseIf *RHS\Scope&#Value_Searchable
      ; Value and Object:
      *ResultValue        = AddStackValue()
      *ResultValue\Class  = *RHS\Class
      *ResultObject       = CreateGroup("value", 0, *ResultValue)
      *ResultValue\Memory = *ResultObject
      *ResultValue\Scope  | *RHS\Scope&~(#Value_Reference|#Value_Literal)
      
      OperateValueAndObject(Operation, *LHS, *RHS\Object, *ResultObject)
    ElseIf *RHS\Class = *QueueClass
      ;Value and Queue:
      *ResultQueue = CreateQueue(@FreeValue())
      *ResultValue = PushQueueValue(*ResultQueue)
      
      OperateValueAndQueue(Operation, *LHS, *RHS\Queue, *ResultQueue)
    Else
      ProcedureReturn #Null
    EndIf
  ElseIf *RHS
    If *RHS\Scope&#Value_Searchable
      ; Nothing and Object:
      *ResultValue        = AddStackValue()
      *ResultValue\Class  = *RHS\Class
      *ResultObject       = CreateGroup("value", 0, *ResultValue)
      *ResultValue\Memory = *ResultObject
      *ResultValue\Scope  | *RHS\Scope&~(#Value_Reference|#Value_Literal)
      
      OperateObject(Operation, *RHS\Object, *ResultObject)
    ElseIf *RHS\Class = *QueueClass
      ; Nothing and Queue:
      *ResultQueue = CreateQueue(@FreeValue())
      *ResultValue = PushQueueValue(*ResultQueue)
      
      OperateQueue(Operation, *RHS\Queue, *ResultQueue)
    EndIf
  EndIf
  
  If OperatorCompare(Operation)
    *ResultValue = CompareValue(*ResultValue) : _E()
    ProcedureReturn PushBValue(*ResultValue)
  ElseIf Operation = #Operator_Set
    ProcedureReturn *RHS
  Else
    ProcedureReturn *ResultValue
  EndIf
EndProcedure

Procedure.i OperateObjectAndObject(Operation.i, *LObject.Object, *RObject.Object, *ResultObject.Object)
  Protected *LItem.QueueItem, *RItem.QueueItem, *LChild.Object, *RChild.Object
  Protected *Result.Value
  
  If *LObject\Group\ObjectQueue\Count <> *RObject\Group\ObjectQueue\Count
    _Error("Invalid count for a vectorial operation!")
  EndIf
  
  *LItem = *LObject\Group\ObjectQueue\First
  *RItem = *RObject\Group\ObjectQueue\First
  
  While *LItem
    *LChild = *LItem\Object
    *RChild = *RItem\Object
    
    If *LChild\Type <> *RChild\Type
      _Error("Types do not match!")
    EndIf
    
    Select *LChild\Type
      Case #Object_Variable
        *Result = OperateValue(Operation, *LChild\Variable\Value, *RChild\Variable\Value) : _E()
        CreateVariable(*LChild\Name, CreateReferenceValue(*Result), *ResultObject)
      Case #Object_Group
        OperateObjectAndObject(Operation, *LChild, *RChild, CreateGroup(*LChild\Name, *ResultObject, 0)) : _E()
    EndSelect
    
    _Move(*LItem, N)
    _Move(*RItem, N)
  Wend
EndProcedure
Procedure.i OperateObjectAndQueue(Operation.i, *LObject.Object, *RQueue.Queue, *ResultObject.Object)
  Protected *LItem.QueueItem, *RItem.QueueItem, *LChild.Object, *RValue.Value
  Protected *Result.Value
  
  If *LObject\Group\ObjectQueue\Count <> *RQueue\Count
    _Error("Invalid count for a vectorial operation!")
  EndIf
  
  *LItem = *LObject\Group\ObjectQueue\First
  *RItem = *RQueue\First
  
  While *LItem
    *LChild = *LItem\Object
    *RValue = *RItem\Value
    
    Select *LChild\Type
      Case #Object_Variable
        *Result = OperateValue(Operation, *LChild\Variable\Value, *RValue) : _E()
        CreateVariable(*LChild\Name, CreateReferenceValue(*Result), *ResultObject)
      Case #Object_Group
        If *RValue\Scope&#Value_Searchable
          OperateObjectAndQueue(Operation, *LChild, *RValue\Object, CreateGroup(*LChild\Name, *ResultObject, 0))
        ElseIf *RValue\Class = *QueueClass
          OperateObjectAndObject(Operation, *LChild, *RValue\Queue, CreateGroup(*LChild\Name, *ResultObject, 0))
        Else
          OperateObjectAndValue(Operation, *LChild, *RValue, CreateGroup(*LChild\Name, *ResultObject, 0))
        EndIf
        _E()
    EndSelect
    
    _Move(*LItem, N)
    _Move(*RItem, N)
  Wend
EndProcedure
Procedure.i OperateObjectAndValue(Operation.i, *LObject.Object, *RValue.Value, *ResultObject.Object)
  Protected *LItem.QueueItem, *LChild.Object
  Protected *Result.Value
  
  *LItem = *LObject\Group\ObjectQueue\First
  While *LItem
    *LChild = *LItem\Object
    
    Select *LChild\Type
      Case #Object_Variable
        *Result = OperateValue(Operation, *LChild\Variable\Value, *RValue) : _E()
        CreateVariable(*LChild\Name, CreateReferenceValue(*Result), *ResultObject)
      Case #Object_Group
        OperateObjectAndValue(Operation, *LChild, *RValue, CreateGroup(*LChild\Name, *ResultObject)) : _E()
    EndSelect
    
    _Move(*LItem, N)
  Wend
EndProcedure
Procedure.i OperateQueueAndObject(Operation.i, *LQueue.Queue, *RObject.Object, *ResultObject.Object)
  Protected *LItem.QueueItem, *RItem.QueueItem, *LValue.Value, *RChild.Object
  Protected *Result.Value
  
  If *LQueue\Count <> *RObject\Group\ObjectQueue\Count
    _Error("Invalid count for a vectorial operation!")
  EndIf
  
  *LItem = *LQueue\First
  *RItem = *RObject\Group\ObjectQueue\First
  
  While *LItem
    *LValue = *LItem\Value
    *RChild = *RItem\Object
    
    Select *RChild\Type
      Case #Object_Variable
        *Result = OperateValue(Operation, *LValue, *RChild\Variable\Value) : _E()
        CreateVariable(*RChild\Name, CreateReferenceValue(*Result), *ResultObject)
      Case #Object_Group
        If *LValue\Scope&#Value_Searchable
          OperateObjectAndObject(Operation, *LValue, *RChild, CreateGroup(*RChild\Name, *ResultObject, 0))
        ElseIf *LValue\Class = *QueueClass
          OperateQueueAndObject(Operation, *LValue, *RChild, CreateGroup(*RChild\Name, *ResultObject, 0))
        Else
          OperateValueAndObject(Operation, *LValue, *RChild, CreateGroup(*RChild\Name, *ResultObject, 0))
        EndIf
        _E()
    EndSelect
    
    _Move(*LItem, N)
    _Move(*RItem, N)
  Wend
EndProcedure
Procedure.i OperateValueAndObject(Operation.i, *LValue.Value, *RObject.Object, *ResultObject.Object)
  Protected *RItem.QueueItem, *RChild.Object
  Protected *Result.Value
  
  *RItem = *RObject\Group\ObjectQueue\First
  While *RItem
    *RChild = *RItem\Object
    
    Select *RChild\Type
      Case #Object_Variable
        *Result = OperateValue(Operation, *LValue, *RChild\Variable\Value) : _E()
        CreateVariable(*RChild\Name, CreateReferenceValue(*Result), *ResultObject)
      Case #Object_Group
        OperateValueAndObject(Operation, *LValue, *RChild, CreateGroup(*RChild\Name, *ResultObject)) : _E()
    EndSelect
    
    _Move(*RItem, N)
  Wend
EndProcedure
Procedure.i OperateQueueAndQueue(Operation.i, *LQueue.Queue, *RQueue.Queue, *ResultQueue.Queue)
  Protected *LItem.QueueItem, *RItem.QueueItem
  Protected *Result.Value
  
  If *LQueue\Count <> *RQueue\Count
    _Error("Invalid count for a vectorial operation!")
  EndIf
  
  *LItem = *LQueue\First
  *RItem = *RQueue\First
  
  While *LItem
    *Result = OperateValue(Operation, *LItem\Value, *RItem\Value) : _E()
    AppendQueueItem(*ResultQueue, CreateReferenceValue(*Result))
    
    _Move(*LItem, N)
    _Move(*RItem, N)
  Wend
EndProcedure
Procedure.i OperateQueueAndValue(Operation.i, *LQueue.Queue, *RValue.Value, *ResultQueue.Queue)
  Protected *LItem.QueueItem
  Protected *Result.Value
  
  *LItem = *LQueue\First
  While *LItem
    *Result = OperateValue(Operation, *LItem\Value, *RValue) : _E()
    AppendQueueItem(*ResultQueue, CreateReferenceValue(*Result))
    
    _Move(*LItem, N)
  Wend
EndProcedure
Procedure.i OperateValueAndQueue(Operation.i, *LValue.Value, *RQueue.Queue, *ResultQueue.Queue)
  Protected *RItem.QueueItem
  Protected *Result.Value
  
  *RItem = *RQueue\First
  While *RItem
    *Result = OperateValue(Operation, *LValue, *RItem\Value) : _E()
    AppendQueueItem(*ResultQueue, CreateReferenceValue(*Result))
    
    _Move(*RItem, N)
  Wend
EndProcedure
Procedure.i OperateObject(Operation.i, *Object.Object, *ResultObject.Object)
  Protected *Item.QueueItem, *Child.Object
  Protected *Result.Value
  
  *Item = *Object\Group\ObjectQueue\First
  While *Item
    *Child = *Item\Object
    
    Select *Child\Type
      Case #Object_Variable
        *Result = OperateValue(Operation, 0, *Child\Variable\Value) : _E()
        CreateVariable(*Child\Name, CreateReferenceValue(*Result), *ResultObject)
      Case #Object_Group
        OperateObject(Operation, *Child, CreateGroup(*Child\Name, *ResultObject)) : _E()
    EndSelect
    
    _Move(*Item, N)
  Wend
EndProcedure
Procedure.i OperateQueue(Operation.i, *Queue.Queue, *ResultQueue.Queue)
  Protected *Item.QueueItem
  Protected *Result.Value
  
  *Item = *Queue\First
  
  While *Item
    *Result = OperateValue(Operation, 0, *Item\Value) : _E()
    AppendQueueItem(*ResultQueue, CreateReferenceValue(*Result))
    
    _Move(*Item, N)
  Wend
EndProcedure

Procedure.i CompareValue(*Value.Value)
  If *Value\Class = *BooleanClass
    ProcedureReturn *Value\Integer\I
  ElseIf *Value\Scope&#Value_Searchable
    ProcedureReturn CompareObject(*Value\Object)
  ElseIf *Value\Class = *QueueClass
    ProcedureReturn CompareQueue(*Value\Queue)
  Else
    _Error("Invalid class for vectorial comparing: " + #Q + ClassName(*Value\Class) + #Q)
  EndIf
EndProcedure
Procedure.i CompareObject(*Object.Object)
  Protected *Item.QueueItem, *Child.Object
  
  *Item = *Object\Group\ObjectQueue\First
  While *Item
    *Child = *Item\Memory
    
    Select *Child\Type
      Case #Object_Variable
        If Not CompareValue(*Child\Variable\Value)
          ProcedureReturn #False
        EndIf
      Case #Object_Group
        If Not CompareObject(*Object)
          ProcedureReturn #False
        EndIf
    EndSelect
    
    _Move(*Item, N)
  Wend
  
  ProcedureReturn #True
EndProcedure
Procedure.i CompareQueue(*Queue.Queue)
  Protected *Item.QueueItem
  
  *Item = *Queue\First
  While *Item
    If CompareValue(*Item\Value)
      ProcedureReturn #False
    EndIf
    _Move(*Item, N)
  Wend
  
  ProcedureReturn #True
EndProcedure

; Operator callbacks:
Procedure.i SubtractOperator(*LHS.Value, *RHS.Value)
  If *LHS And *RHS
    ProcedureReturn CallClassMethod(*LHS\Class, #ClassMethod_Subtract, Arguments1(*RHS), *LHS, #True, #False)
  ElseIf *RHS
    ProcedureReturn CallClassMethod(*RHS\Class, #ClassMethod_Negative, -1, *RHS, #True, #False)
  EndIf
EndProcedure
Procedure.i RootOperator(*LHS.Value, *RHS.Value)
  Protected Exponent.f
  
  If *LHS And *RHS
    Select *RHS\Class
      Case *IntegerClass
        Exponent = 1.0 / *RHS\Integer\I
      Case *QuadClass
        Exponent = 1.0 / *RHS\Quad\Q
      Case *FloatClass
        Exponent = 1.0 / *RHS\Float\F
      Case *DoubleClass
        Exponent = 1.0 / *RHS\Double\D
      Default
        *RHS = CallClassMethod(*FloatClass, #ClassMethod_Divide, Arguments1(*RHS), PushFValue(1)) : _E()
        Exponent = ValueF(*RHS)                                                                   : _E()
    EndSelect
    ProcedureReturn CallClassMethod(*LHS\Class, #ClassMethod_Power, Arguments1(PushFValue(Exponent)), *LHS)
  ElseIf *RHS
    ProcedureReturn CallClassMethod(*RHS\Class, #ClassMethod_Power, Arguments1(PushFValue(0.5)), *RHS)
  EndIf
EndProcedure
Procedure.i NotEqualOperator(*LHS.Value, *RHS.Value)
  Protected *Result.Value
  
  *Result = CallClassMethod(*LHS\Class, #ClassMethod_Equal, Arguments1(*RHS), *LHS, #True, #False) : _E()
  If *Result
    *Result\Integer\I = 1 - *Result\Integer\I
  EndIf
  ProcedureReturn *Result
EndProcedure
Procedure.i LessEqualOperator(*LHS.Value, *RHS.Value)
  Protected *Result.Value
  
  *Result = CallClassMethod(*LHS\Class, #ClassMethod_Less, Arguments1(*RHS), *LHS, #True, #False) : _E()
  If *Result And Not *Result\Integer\I
    *Result = CallClassMethod(*LHS\Class, #ClassMethod_Equal, Arguments1(*RHS), *LHS, #True, #False) : _E()
    ProcedureReturn *Result
  Else
    ProcedureReturn *Result
  EndIf
EndProcedure
Procedure.i GreaterEqualOperator(*LHS.Value, *RHS.Value)
  Protected *Result.Value
  
  *Result = CallClassMethod(*LHS\Class, #ClassMethod_Greater, Arguments1(*RHS), *LHS, #True, #False) : _E()
  If *Result And Not *Result\Integer\I
    *Result = CallClassMethod(*LHS\Class, #ClassMethod_Equal, Arguments1(*RHS), *LHS, #True, #False) : _E()
    ProcedureReturn *Result
  Else
    ProcedureReturn *Result
  EndIf
EndProcedure
Procedure.i IncrementOperator(*LHS.Value, *RHS.Value)
  Protected *Value.Value
  
  If *LHS
    ; Speed for integers:
    If *LHS\Class = *IntegerClass
      *LHS\Integer\I + 1
      ProcedureReturn *LHS
    Else
      *Value = CallClassMethod(*LHS\Class, #ClassMethod_Increment, -1, *LHS, #True, #False) : _E()
      ProcedureReturn *Value
    EndIf
  ElseIf *RHS
    ; Speed for integers:
    If *RHS\Class = *IntegerClass
      *Value = PushIValue(*RHS\Integer\I)
      *RHS\Integer\I + 1
      ProcedureReturn *Value
    Else
      If *RHS\Class\Group\ClassMethod[#ClassMethod_Increment]
        *Value = DuplicateValue(*LHS)
        CallClassMethod(*RHS\Class, #ClassMethod_Increment, -1, *RHS) : _E()
        ProcedureReturn *Value
      Else
        ProcedureReturn #False
      EndIf
    EndIf
  EndIf
EndProcedure
Procedure.i DecrementOperator(*LHS.Value, *RHS.Value)
  Protected *Value.Value
  
  If *LHS
    ; Speed for integers:
    If *LHS\Class = *IntegerClass
      *LHS\Integer\I - 1
      ProcedureReturn *LHS
    Else
      *Value = CallClassMethod(*LHS\Class, #ClassMethod_Decrement, -1, *LHS, #True, #False) : _E()
      ProcedureReturn *Value
    EndIf
  ElseIf *RHS
    ; Speed for integers:
    If *RHS\Class = *IntegerClass
      *Value = PushIValue(*RHS\Integer\I)
      *RHS\Integer\I - 1
      ProcedureReturn *Value
    Else
      If *RHS\Class\Group\ClassMethod[#ClassMethod_Decrement]
        *Value = DuplicateValue(*LHS)
        CallClassMethod(*RHS\Class, #ClassMethod_Decrement, -1, *RHS) : _E()
        ProcedureReturn *Value
      Else
        ProcedureReturn #False
      EndIf
    EndIf
  EndIf
EndProcedure
Procedure.i FormatOperator(*LHS.Value, *RHS.Value)
  Protected String1.s, String2.s
  
  If *RHS
    If *LHS
      If *LHS\Class = *StringClass
        String1 = FormatValue(*RHS) : _E()
        ProcedureReturn PushSValue(*LHS\String\S + String1)
      Else
        String1 = FormatValue(*LHS) : _E()
        String2 = FormatValue(*RHS) : _E()
        ProcedureReturn PushSValue(String1 + String2)
      EndIf
    Else
      String1 = FormatValue(*RHS) : _E()
      ProcedureReturn PushSValue(String1)
    EndIf
  EndIf
EndProcedure
Procedure.i InputOperator(*LHS.Value, *RHS.Value)
  Protected Name.s, *Item.QueueItem, *Input.Value
  
  If *LHS
    If *RHS
      *Input = Execute(InputFLT(#Symbol_Input, ValueS(*LHS)), 0, #True)
      If *Input
        SetValue(*RHS, *Input)
        ProcedureReturn *RHS
      ElseIf Not Truncation
        _Error("No value typed!")
      EndIf
    Else
      *Input = Execute(InputFLT(#Symbol_Input, ValueS(*LHS)), 0, #True) : _E()
      If *Input
        ProcedureReturn *Input
      ElseIf Not Truncation
        _Error("No value typed!")
      EndIf
    EndIf
  Else
    If *RHS
      If *RHS\Class = *QueueClass
        *Item = *RHS\Queue\First
        While *Item
          *LHS = *Item\Value
          *Input = Execute(InputFLT(#Symbol_Input, ValueName(*LHS)), 0, #True) : _E()
          If *Input
            SetValue(*LHS, *Input)
          ElseIf Not Truncation
            _Error("No value typed!")
          EndIf
          _Move(*Item, N)
        Wend
        ProcedureReturn *RHS
      Else
        *Input = Execute(InputFLT(#Symbol_Input, ValueName(*RHS)), 0, #True)
        If *Input
          SetValue(*RHS, *Input)
          ProcedureReturn *RHS
        Else
          _Error("No value typed!")
        EndIf
      EndIf
    Else
      *Input = Execute(InputFL(#Symbol_Input), 0, #True)
      If *Input
        ProcedureReturn *Input
      ElseIf Not Truncation
        _Error("No value typed!")
      EndIf
    EndIf
  EndIf
EndProcedure

; Short methods:
Procedure.i IsValueTrue(*Value.Value)
  If *Value
    ; Speed for bools and integers:
    If *Value\Class = *BooleanClass Or *Value\Class = *IntegerClass
      ProcedureReturn *Value\Integer\I
    Else
      *Value = CallClassMethod(*Value\Class, #ClassMethod_IsTrue, -1, *Value)
      If *Value
        ProcedureReturn ValueB(*Value)
      Else
        _Error("No value typed for an expression!")
      EndIf
    EndIf
  Else
    ProcedureReturn PushBValue(#False)
  EndIf
EndProcedure
Procedure.i SetValue(*Value.Value, *Source.Value)
  OperateValue(#Operator_Set, *Value, *Source)
  ProcedureReturn *Value
EndProcedure
Procedure.i AddValue(*Value.Value, *Other.Value)
  ProcedureReturn OperateValue(#Operator_DoAdd, *Value, *Other)
EndProcedure
Procedure.i SubtractValue(*Value.Value, *Other.Value)
  ProcedureReturn OperateValue(#Operator_DoSubtract, *Value, *Other)
EndProcedure
Procedure.i MultiplyValue(*Value.Value, *Other.Value)
  ProcedureReturn OperateValue(#Operator_DoMultiply, *Value, *Other)
EndProcedure
Procedure.i DivideValue(*Value.Value, *Other.Value)
  ProcedureReturn OperateValue(#Operator_DoDivide, *Value, *Other)
EndProcedure
Procedure.i IncrementValue(*Value.Value)
  OperateValue(#Operator_Increment, *Value, 0)
EndProcedure
Procedure.i DecrementValue(*Value.Value)
  OperateValue(#Operator_Decrement, *Value, 0)
EndProcedure
Procedure.i ValueSum(*First.Value, *Second.Value)
  If *First And *Second
    ProcedureReturn OperateValue(#Operator_Add, *First, *Second)
  Else
    ProcedureReturn #Null
  EndIf
EndProcedure
Procedure.i ValueDifference(*First.Value, *Second.Value)
  If *First And *Second
    ProcedureReturn OperateValue(#Operator_Subtract, *First, *Second)
  Else
    ProcedureReturn #Null
  EndIf
EndProcedure
Procedure.i ValueProduct(*First.Value, *Second.Value)
  If *First And *Second
    ProcedureReturn OperateValue(#Operator_Multiply, *First, *Second)
  Else
    ProcedureReturn #Null
  EndIf
EndProcedure
Procedure.i ValueQuotient(*First.Value, *Second.Value)
  If *First And *Second
    ProcedureReturn OperateValue(#Operator_Divide, *First, *Second)
  Else
    ProcedureReturn #Null
  EndIf
EndProcedure
Procedure.i ValuePower(*First.Value, *Second.Value)
  If *First And *Second
    ProcedureReturn OperateValue(#Operator_Power, *First, *Second)
  Else
    ProcedureReturn #Null
  EndIf
EndProcedure
Procedure.i IsValueEqual(*Value.Value, *Other.Value)
  If *Value And *Other
    ProcedureReturn IsValueTrue(OperateValue(#Operator_Equal, *Value, *Other))
  Else
    ProcedureReturn #Null
  EndIf
EndProcedure
Procedure.i IsValueLess(*Value.Value, *Other.Value)
  If *Value And *Other
    ProcedureReturn IsValueTrue(OperateValue(#Operator_Less, *Value, *Other))
  Else
    ProcedureReturn #Null
  EndIf
EndProcedure
Procedure.i IsValueGreater(*Value.Value, *Other.Value)
  If *Value And *Other
    ProcedureReturn IsValueTrue(OperateValue(#Operator_Greater, *Value, *Other))
  Else
    ProcedureReturn #Null
  EndIf
EndProcedure
Procedure.s FormatValue(*Value.Value)
  If *Value
    If *Value\Class\Group\ClassMethod[#ClassMethod_Format]
      *Value = Call(*Value\Class\Group\ClassMethod[#ClassMethod_Format], -1, *Value) : _E()
      If *Value
        ProcedureReturn ValueS(*Value)
      Else
        _Error("No result of the " + #Q + "_form" + #Q + " method: " + #Q + ObjectPath(*Value\Class) + "\_form" + #Q)
      EndIf
    ElseIf *Value\Scope&#Value_Searchable
      ProcedureReturn FormatObject(*Value\Object)
    Else
      ProcedureReturn ClassName(*Value\Class) + "()"
    EndIf
  Else
    ProcedureReturn "<UNTYPED>"
  EndIf
EndProcedure
Procedure.i WriteValue(File.i, *Value.Value)
  Protected *Item.QueueItem
  
  Select *Value\Class
    Case *IntegerClass
      WriteInteger(File, *Value\Integer\I)
    Case *FloatClass
      WriteFloat(File, *Value\Float\F)
    Case *QuadClass
      WriteQuad(File, *Value\Quad\Q)
    Case *DoubleClass
      WriteDouble(File, *Value\Double\D)
    Case *StringClass
      WriteNTString(File, *Value\String\S)
    Case *BooleanClass
      WriteByte(File, *Value\Integer\I)
    Case *QueueClass
      *Item = *Value\Queue\First
      While *Item
        WriteValue(File, *Item\Value)
        _Move(*Item, N)
      Wend
    Case *ObjectClass
      WriteObject(File, *Value\Object)
    Default
      If *Value\Class\Group\ClassMethod[#ClassMethod_FileWrite]
        Call(*Value\Class\Group\ClassMethod[#ClassMethod_FileWrite], Arguments1(PushIValue(File)), *Value)
      ElseIf *Value\Scope&#Value_Searchable
        WriteObject(File, *Value\Object)
      Else
        _Error("Can not write value: " + #Q + ClassName(*Value\Class) + #Q)
      EndIf
  EndSelect
EndProcedure
Procedure.i ReadValue(File.i, *Value.Value)
  Protected *Item.QueueItem
  
  Select *Value\Class
    Case *IntegerClass
      *Value\Integer\I = ReadInteger(File)
    Case *FloatClass
      *Value\Float\F = ReadFloat(File)
    Case *QuadClass
      *Value\Quad\Q = ReadQuad(File)
    Case *DoubleClass
      *Value\Double\D = ReadDouble(File)
    Case *StringClass
      *Value\String\S = ReadNTString(File)
    Case *BooleanClass
      *Value\Integer\I = ReadByte(File)
    Case *ObjectClass
      ReadObject(File, *Value\Object)
    Case *QueueClass
      *Item = *Value\Queue\First
      While *Item
        ReadValue(File, *Item\Value)
        _Move(*Item, N)
      Wend
    Default
      If *Value\Class\Group\ClassMethod[#ClassMethod_FileRead]
        Call(*Value\Class\Group\ClassMethod[#ClassMethod_FileRead], Arguments1(PushIValue(File)), *Value)
      ElseIf *Value\Scope&#Value_Searchable
        ReadObject(File, *Value\Object)
      Else
        _Error("Can not read value: " + #Q + ClassName(*Value\Class) + #Q)
      EndIf
  EndSelect
EndProcedure

; IDE Options = PureBasic 4.41 (Windows - x86)
; CursorPosition = 474
; FirstLine = 360
; Folding = -RVgAAA+
; EnableXP
; CompileSourceDirectory