;**************************************************************
;     Value
;
; Autor: Josef Sniatecki
; Creation date: 07.04.2009
;**************************************************************
;   (c) 2009 Josef Sniatecki
;
; This program is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License
; As published by the Free Software Foundation; either version
; 3 of the License, Or (at your option) any later version.
;
; This program is distributed in the hope that it will be
; useful, but WITHOUT ANY WARRANTY; without even the implied
; warranty of MERCHANTABILITY Or FITNESS For A PARTICULAR
; PURPOSE. See the GNU General Public License For more details.
;
; You should have received a copy of the GNU General Public
; License along With this program;
; If Not, see "http://www.gnu.org/licenses/".
;**************************************************************



; Allocate values:
Procedure.i CreateValue(*Class.Object, *SourceValue.Value = 0)
  Protected *Value.Value
  
  *Value = _New(Value)
  
  If *Class\Reference
    _Move(*Class, Reference)
  EndIf
  
  *Value\Class = *Class
  
  CallClassMethod(*Class, #ClassMethod_New, -1, *Value, #False, #False)
  If *SourceValue
    CallClassMethod(*Class, #ClassMethod_Set, Arguments1(*SourceValue), *Value, #False)
  EndIf
  
  ProcedureReturn *Value
EndProcedure
Procedure.i CreateVValue()
  Protected *Value.Value
  
  *Value = _New(Value)
  
  *Value\Class = *VariantClass
  *Value\Scope | #Value_Binary
  
  ProcedureReturn *Value
EndProcedure
Procedure.i CreateIValue(Integer.i)
  Protected *Value.Value
  
  *Value = _New(Value)
  
  *Value\Class     = *IntegerClass
  *Value\Integer   = _New(Integer)
  *Value\Integer\I = Integer
  *Value\Scope     | #Value_Binary
  
  ProcedureReturn *Value
EndProcedure
Procedure.i CreateFValue(Float.f)
  Protected *Value.Value
  
  *Value = _New(Value)
  
  *Value\Class   = *FloatClass
  *Value\Float   = _New(Float)
  *Value\Float\F = Float
  *Value\Scope   | #Value_Binary
  
  ProcedureReturn *Value
EndProcedure
Procedure.i CreateQValue(Quad.q)
  Protected *Value.Value
  
  *Value = _New(Value)
  
  *Value\Class  = *QuadClass
  *Value\Quad   = _New(Quad)
  *Value\Quad\Q = Quad
  *Value\Scope  | #Value_Binary
  
  ProcedureReturn *Value
EndProcedure
Procedure.i CreateDValue(Double.d)
  Protected *Value.Value
  
  *Value = _New(Value)
  
  *Value\Class    = *DoubleClass
  *Value\Double   = _New(Double)
  *Value\Double\D = Double
  *Value\Scope    | #Value_Binary
  
  ProcedureReturn *Value
EndProcedure
Procedure.i CreateBValue(State.i)
  Protected *Value.Value
  
  *Value = _New(Value)
  
  *Value\Class     = *BooleanClass
  *Value\Integer   = _New(Integer)
  *Value\Integer\I = State
  *Value\Scope     | #Value_Binary
  
  ProcedureReturn *Value
EndProcedure
Procedure.i CreateSValue(String.s)
  Protected *Value.Value
  
  *Value = _New(Value)
  
  *Value\Class    = *StringClass
  *Value\String   = _New(String)
  *Value\String\S = String
  *Value\Scope    |#Value_Binary
  
  ProcedureReturn *Value
EndProcedure
Procedure.i CreateSPValue(*String, *Character.Character, *EndOfString = 0)
  Protected *Value.Value
  
  *Value = _New(Value)
  
  *Value\Class                   = *StringPointerClass
  *Value\StringPointer           = _New(StringPointer)
  *Value\StringPointer\String    = *String
  *Value\StringPointer\Character = *Character
  If *EndOfString : *Value\StringPointer\EndOfString = *EndOfString : Else : *Value\StringPointer\EndOfString = *String + MemoryStringLength(*String) : EndIf
  *Value\Scope|#Value_Binary
  
  ProcedureReturn *Value
EndProcedure
Procedure.i CreateQueueValue(*Queue.Queue = 0, Reference.i = #False)
  Protected *Value.Value
  
  *Value       = _New(Value)
  *Value\Class = *QueueClass
  If Reference
    *Value\Queue = *Queue
    *Value\Scope|#Value_Binary|#Value_Reference
  Else
    If *Queue
      *Value\Queue = *Queue
    Else
      *Value\Queue = CreateQueue(@FreeValue())
    EndIf
    *Value\Scope|#Value_Binary
  EndIf
  
  ProcedureReturn *Value
EndProcedure
Procedure.i CreateBlockValue(*TokenQueue.Queue = 0, Reference.i = #False)
  Protected *Value.Value
  
  *Value       = _New(Value)
  *Value\Class = *BlockClass
  If Reference
    *Value\Queue = *TokenQueue
    *Value\Scope|#Value_Binary|#Value_Reference
  Else
    If *TokenQueue
      *Value\Queue = *TokenQueue
    Else
      *Value\Queue = CreateQueue(@FreeToken())
    EndIf
    *Value\Scope|#Value_Binary
  EndIf
  
  ProcedureReturn *Value
EndProcedure
Procedure.i CreateTokenValue(*Item.QueueItem)
  Protected *Value.Value
  
  *Value       = _New(Value)
  *Value\Class = *TokenClass
  *Value\Item  = *Item
  *Value\Scope | #Value_Binary
  
  ProcedureReturn *Value
EndProcedure
Procedure.i CreatePValue(*Object.Object)
  Protected *Value.Value
  
  *Value        = _New(Value)
  *Value\Class  = *PointerClass
  *Value\Object = *Object
  *Value\Scope  = #Value_Binary|#Value_Searchable
  
  ProcedureReturn *Value
EndProcedure
Procedure.i CreateReferenceValue(*Original.Value)
  Protected *Value.Value
  
  *Value          = _New(Value)
  PrepareReference(*Value, *Original)
  *Value\Class    = *Original\Class
  *Value\Memory   = *Original\Memory
  *Value\Scope    = *Original\Scope|#Value_Reference
  *Value\Variable = *Original\Variable
  
  ProcedureReturn *Value
EndProcedure

; Special methods:
Procedure.i FreeValue(*Value.Value)
  Protected PreviousError.i, PreviousTruncation.i
  
  If *Value\Scope&#Value_Reference Or Not (*Value\Scope&#Value_Literal Or *Value\Scope&#Value_Stack Or *Value\Variable)
    Swap Error, PreviousError
    Swap Truncation, PreviousTruncation
    
    If Not *Value\Scope&#Value_Reference
      CallClassMethod(*Value\Class, #ClassMethod_Delete, -1, *Value, #False, #False)
      If Not *Value\Scope&#Value_Binary And *Value\Scope&#Value_Searchable
        FreeObject(*Value\Memory)
      EndIf
      If *Value\ReferenceQueue
        ResetReferences(*Value\ReferenceQueue)
      EndIf
    ElseIf *Value\Original
      FreeReference(*Value)
    EndIf
    FreeMemory(*Value)
    
    Swap Truncation, PreviousTruncation
    Swap Error, PreviousError
  EndIf
  
  ProcedureReturn #True
EndProcedure
Procedure.i FreeHardValue(*Value.Value)
  Protected PreviousError.i, PreviousTruncation.i
  
  Swap Error, PreviousError
  Swap Truncation, PreviousTruncation
  
  If Not *Value\Scope&#Value_Reference
    CallClassMethod(*Value\Class, #ClassMethod_Delete, -1, *Value, #False, #False)
    If Not *Value\Scope&#Value_Binary And *Value\Scope&#Value_Searchable
      FreeObject(*Value\Memory)
    EndIf
    If *Value\ReferenceQueue
      ResetReferences(*Value\ReferenceQueue)
    EndIf
  ElseIf *Value\Original
    FreeReference(*Value)
  EndIf
  FreeMemory(*Value)
  
  Swap Truncation, PreviousTruncation
  Swap Error, PreviousError
  
  ProcedureReturn #True
EndProcedure
Procedure.i CopyValue(*Source.Value, *Destination.Value)
  *Destination\Class = *Source\Class
  CallClassMethod(*Destination\Class, #ClassMethod_New, -1, *Destination, #False)
  OperateValue(#Operator_Set, *Destination, *Source)
  ProcedureReturn #True
EndProcedure
Procedure.i DuplicateValue(*Value.Value)
  Protected *Copy.Value
  
  *Copy = CreateValue(*Value\Class) : _E()
  OperateValue(#Operator_Set, *Copy, *Value) : _EG(DuplicateValue)
  ProcedureReturn *Copy
  
  DuplicateValue_Error:
    If *Copy
      FreeValue(*Copy)
    EndIf
    ProcedureReturn #False
EndProcedure
Procedure.s ValueName(*Value.Value)
  If *Value\Variable
    ProcedureReturn *Value\Variable\Name
  ElseIf *Value\Class = *ObjectClass
    ProcedureReturn *Value\Object\Name
  Else
    ProcedureReturn ClassName(*Value\Class)
  EndIf
EndProcedure
Procedure.s ValuePath(*Value.Value)
  If *Value\Variable
    ProcedureReturn ObjectPath(*Value\Variable)
  ElseIf *Value\Class = *ObjectClass
    ProcedureReturn ObjectPath(*Value\Object)
  Else
    ProcedureReturn ClassName(*Value\Class)
  EndIf
EndProcedure

; Push values in the stack:
Procedure.i PushValue(*Class.Object, *SourceValue.Value = 0)
  Protected *Value.Value
  
  *Value = AddStackValue()
  
  If *Class\Reference
    _Move(*Class, Reference)
  EndIf
  
  *Value\Class = *Class
  
  CallClassMethod(*Class, #ClassMethod_New, -1, *Value, #False, #False)
  If *SourceValue
    CallClassMethod(*Class, #ClassMethod_Define, Arguments1(*SourceValue), *Value, #False)
  EndIf
  ProcedureReturn *Value
EndProcedure
Procedure.i PushVValue()
  Protected *Value.Value
  
  *Value = AddStackValue()
  
  *Value\Class = *VariantClass
  *Value\Scope | #Value_Binary
  
  ProcedureReturn *Value
EndProcedure
Procedure.i PushIValue(Integer.i)
  Protected *Value.Value
  
  *Value = AddStackValue()
  
  *Value\Class     = *IntegerClass
  *Value\Integer   = _New(Integer)
  *Value\Integer\I = Integer
  *Value\Scope     | #Value_Binary
  
  ProcedureReturn *Value
EndProcedure
Procedure.i PushFValue(Float.f)
  Protected *Value.Value
  
  *Value = AddStackValue()
  
  *Value\Class   = *FloatClass
  *Value\Float   = _New(Float)
  *Value\Float\F = Float
  *Value\Scope   | #Value_Binary
  
  ProcedureReturn *Value
EndProcedure
Procedure.i PushQValue(Quad.q)
  Protected *Value.Value
  
  *Value = AddStackValue()
  
  *Value\Class  = *QuadClass
  *Value\Quad   = _New(Quad)
  *Value\Quad\Q = Quad
  *Value\Scope  | #Value_Binary
  
  ProcedureReturn *Value
EndProcedure
Procedure.i PushDValue(Double.d)
  Protected *Value.Value
  
  *Value = AddStackValue()
  
  *Value\Class    = *DoubleClass
  *Value\Double   = _New(Double)
  *Value\Double\D = Double
  *Value\Scope    | #Value_Binary
  
  ProcedureReturn *Value
EndProcedure
Procedure.i PushBValue(State.i)
  Protected *Value.Value
  
  *Value = AddStackValue()
  
  *Value\Class     = *BooleanClass
  *Value\Integer   = _New(Integer)
  *Value\Integer\I = State
  *Value\Scope     | #Value_Binary
  
  ProcedureReturn *Value
EndProcedure
Procedure.i PushSValue(String.s)
  Protected *Value.Value
  
  *Value = AddStackValue()
  
  *Value\Class    = *StringClass
  *Value\String   = _New(String)
  *Value\String\S = String
  *Value\Scope    | #Value_Binary
  
  ProcedureReturn *Value
EndProcedure
Procedure.i PushSPValue(*String, *Character.Character, *EndOfString = 0)
  Protected *Value.Value
  
  *Value = AddStackValue()
  
  *Value\Class                   = *StringPointerClass
  *Value\StringPointer           = _New(StringPointer)
  *Value\StringPointer\String    = *String
  *Value\StringPointer\Character = *Character
  If *EndOfString : *Value\StringPointer\EndOfString = *EndOfString : Else : *Value\StringPointer\EndOfString = *String + MemoryStringLength(*String) : EndIf
  *Value\Scope|#Value_Binary
  
  ProcedureReturn *Value
EndProcedure
Procedure.i PushQueueValue(*Queue.Queue = 0, Reference.i = #False)
  Protected *Value.Value
  
  *Value = AddStackValue()
  
  *Value\Class = *QueueClass
  If Reference
    *Value\Queue = *Queue
    *Value\Scope|#Value_Binary|#Value_Reference
  Else
    If *Queue
      *Value\Queue = *Queue
    Else
      *Value\Queue = CreateQueue(@FreeValue())
    EndIf
    *Value\Scope|#Value_Binary
  EndIf
  
  ProcedureReturn *Value
EndProcedure
Procedure.i PushBlockValue(*TokenQueue.Queue = 0, Reference.i = #False)
  Protected *Value.Value
  
  *Value       = AddStackValue()
  
  *Value\Class = *BlockClass
  If *TokenQueue
    *Value\Queue = *TokenQueue
  Else
    *Value\Queue = CreateQueue(@FreeToken())
  EndIf
  *Value\Scope|#Value_Binary|(#Value_Reference*Reference)
  
  ProcedureReturn *Value
EndProcedure
Procedure.i PushTokenValue(*Item.QueueItem)
  Protected *Value.Value
  
  *Value = AddStackValue()
  
  *Value\Class = *TokenClass
  *Value\Item  = *Item
  *Value\Scope | #Value_Binary
  
  ProcedureReturn *Value
EndProcedure
Procedure.i PushPValue(*Object.Object)
  Protected *Value.Value
  
  *Value        = AddStackValue()
  
  *Value\Class  = *PointerClass
  *Value\Object = *Object
  *Value\Scope  | #Value_Binary|#Value_Searchable
  
  ProcedureReturn *Value
EndProcedure
Procedure.i PushOValue(*Object.Object, Reference.i = #False)
  Protected *Value.Value
  
  *Value        = AddStackValue()
  
  *Value\Class  = *ObjectClass
  *Value\Object = *Object
  *Value\Scope  | #Value_Binary|#Value_Searchable|(#Value_Reference*Reference)
  
  ProcedureReturn *Value
EndProcedure
Procedure.i PushReferenceValue(*Original.Value)
  Protected *Value.Value
  
  *Value          = AddStackValue()
  PrepareReference(*Value, *Original)
  
  *Value\Class    = *Original\Class
  *Value\Memory   = *Original\Memory
  *Value\Scope    | *Original\Scope|#Value_Reference
  *Value\Variable = *Original\Variable
  
  ProcedureReturn *Value
EndProcedure
Procedure.i PushDuplicatedValue(*Value.Value)
  Protected *Copy.Value
  
  *Copy = PushValue(*Value\Class) : _E()
  SetValue(*Copy, *Value)        : _EG(PushDuplicateValue)
  ProcedureReturn *Copy
  
  PushDuplicateValue_Error:
    If *Copy
      FreeValue(*Copy)
    EndIf
    ProcedureReturn #False
EndProcedure

; Get value contents:
Procedure.i ValueI(*Value.Value)
  Protected *Result.Value
  
  Select *Value\Class
    Case *IntegerClass, *BooleanClass
      ProcedureReturn *Value\Integer\I
    Case *FloatClass
      ProcedureReturn *Value\Float\F
    Case *QuadClass
      ProcedureReturn *Value\Quad\Q
    Case *DoubleClass
      ProcedureReturn *Value\Double\D
    Case *StringClass
      ProcedureReturn Val(*Value\String\S)
    Default
      *Result = CallClassMethod(*Value\Class, #ClassMethod_FriendSet, Arguments1(PushIValue(0)), *Value)
      If *Result
        ProcedureReturn ValueI(*Result)
      Else
        _Error("Convertion failed: " + ClassName(*Value) + " to integer")
      EndIf
  EndSelect
EndProcedure
Procedure.f ValueF(*Value.Value)
  Protected *Result.Value
  
  Select *Value\Class
    Case *FloatClass
      ProcedureReturn *Value\Float\F
    Case *IntegerClass, *BooleanClass
      ProcedureReturn *Value\Integer\I
    Case *QuadClass
      ProcedureReturn *Value\Quad\Q
    Case *DoubleClass
      ProcedureReturn *Value\Double\D
    Case *StringClass
      ProcedureReturn ValF(*Value\String\S)
    Default
      *Result = CallClassMethod(*Value\Class, #ClassMethod_FriendSet, Arguments1(PushFValue(0.0)), *Value)
      If *Result
        ProcedureReturn ValueF(*Result)
      Else
        _Error("Convertion failed: " + ClassName(*Value) + " to float")
      EndIf
  EndSelect
EndProcedure
Procedure.q ValueQ(*Value.Value)
  Protected *Result.Value
  
  Select *Value\Class
    Case *QuadClass
      ProcedureReturn *Value\Quad\Q
    Case *IntegerClass, *BooleanClass
      ProcedureReturn *Value\Integer\I
    Case *FloatClass
      ProcedureReturn *Value\Float\F
    Case *DoubleClass
      ProcedureReturn *Value\Double\D
    Case *StringClass
      ProcedureReturn Val(*Value\String\S)
    Case *VariantClass
      ProcedureReturn 0
    Default
      *Result = CallClassMethod(*Value\Class, #ClassMethod_FriendSet, Arguments1(PushQValue(0)), *Value)
      If *Result
        ProcedureReturn ValueQ(*Result)
      Else
        _Error("Convertion failed: " + ClassName(*Value) + " to quad")
      EndIf
  EndSelect
EndProcedure
Procedure.d ValueD(*Value.Value)
  Protected *Result.Value
  
  Select *Value\Class
    Case *DoubleClass
      ProcedureReturn *Value\Double\D
    Case *IntegerClass, *BooleanClass
      ProcedureReturn *Value\Integer\I
    Case *FloatClass
      ProcedureReturn *Value\Float\F
    Case *QuadClass
      ProcedureReturn *Value\Quad\Q
    Case *StringClass
      ProcedureReturn ValD(*Value\String\S)
    Default
      *Result = CallClassMethod(*Value\Class, #ClassMethod_FriendSet, Arguments1(PushDValue(0.0)), *Value)
      If *Result
        ProcedureReturn ValueD(*Result)
      Else
        _Error("Convertion failed: " + ClassName(*Value) + " to double")
      EndIf
  EndSelect
EndProcedure
Procedure.i ValueB(*Value.Value)
  Protected *Result.Value
  
  Select *Value\Class
    Case *BooleanClass
      ProcedureReturn *Value\Integer\I
    Case *IntegerClass
      If *Value\Integer\I
        ProcedureReturn #True
      Else
        ProcedureReturn #False
      EndIf
    Case *StringClass
      If LCase(*Value\String\S) = "true"
        ProcedureReturn #True
      ElseIf LCase(*Value\String\S) = "false"
        ProcedureReturn #False
      EndIf
    Default
      *Result = CallClassMethod(*Value\Class, #ClassMethod_FriendSet, Arguments1(PushBValue(#False)), *Value)
      If *Result
        ProcedureReturn ValueB(*Result)
      Else
        _Error("Convertion failed: " + ClassName(*Value) + " to boolean")
      EndIf
  EndSelect
EndProcedure
Procedure.s ValueS(*Value.Value)
  Protected *Result.Value
  
  Select *Value\Class
    Case *StringClass
      ProcedureReturn *Value\String\S
    Case *IntegerClass
      ProcedureReturn Str(*Value\Integer\I)
    Case *FloatClass
      ProcedureReturn StrF(*Value\Float\F)
    Case *QuadClass
      ProcedureReturn Str(*Value\Quad\Q)
    Case *DoubleClass
      ProcedureReturn StrD(*Value\Double\D)
    Case *VariantClass
      ProcedureReturn ""
    Case *BooleanClass
      If *Value\Integer\I
        ProcedureReturn "True"
      Else
        ProcedureReturn "False"
      EndIf
    Default
      *Result = CallClassMethod(*Value\Class, #ClassMethod_FriendSet, Arguments1(PushSValue("")), *Value)
      If *Result
        ProcedureReturn ValueS(*Result)
      Else
        _Error("Convertion failed: " + ClassName(*Value) + " to string")
      EndIf
  EndSelect
EndProcedure
Procedure.i ValueBlock(*Value.Value)
  Protected *TokenQueue.Queue, *Result.Value
  
  Select *Value\Class
    Case *BlockClass
      ProcedureReturn *Value\Queue
    Case *StringClass
      *TokenQueue = CreateQueue(@FreeToken())
      Parse(*TokenQueue, @*Value\String\S)
      *TokenQueue\Last\Token\Seperator = #Seperator_Block
      ProcedureReturn *TokenQueue
    Default
      *Result = CallClassMethod(*Value\Class, #ClassMethod_FriendSet, Arguments1(PushBlockValue(#Null)), *Value)
      If *Result
        ProcedureReturn ValueBlock(*Result)
      Else
        _Error("Convertion failed: " + ClassName(*Value) + " to block")
      EndIf
  EndSelect
EndProcedure
Procedure.i ValueP(*Value.Value)
  Protected *Result.Value
  
  Select *Value\Class
    Case *PointerClass, *ObjectClass
      ProcedureReturn *Value\Object
    Default
      *Result = CallClassMethod(*Value\Class, #ClassMethod_FriendSet, Arguments1(PushPValue(#Null)), *Value)
      If *Result
        ProcedureReturn ValueP(*Result)
      Else
        _Error("Convertion failed: " + ClassName(*Value) + " to pointer")
      EndIf
  EndSelect
EndProcedure

; Pop arguments:
Procedure.i PopValue(*Result.Value)
  If *Current\ArgumentQueue\Count
    SetValue(*Result, *Current\ArgumentQueue\First\Value)
    FreeFirstQueueItem(*Current\ArgumentQueue)
    ProcedureReturn #True
  Else
    ProcedureReturn #False
  EndIf
EndProcedure
Procedure.i PopIValue(*Result.Integer)
  If *Current\ArgumentQueue\Count
    *Result\I = ValueI(*Current\ArgumentQueue\First\Value)
    FreeFirstQueueItem(*Current\ArgumentQueue)
    ProcedureReturn #True
  Else
    ProcedureReturn #False
  EndIf
EndProcedure
Procedure.i PopFValue(*Result.Float)
  If *Current\ArgumentQueue\Count
    *Result\F = ValueF(*Current\ArgumentQueue\First\Value)
    FreeFirstQueueItem(*Current\ArgumentQueue)
    ProcedureReturn #True
  Else
    ProcedureReturn #False
  EndIf
EndProcedure
Procedure.i PopQValue(*Result.Quad)
  If *Current\ArgumentQueue\Count
    *Result\Q = ValueQ(*Current\ArgumentQueue\First\Value)
    FreeFirstQueueItem(*Current\ArgumentQueue)
    ProcedureReturn #True
  Else
    ProcedureReturn #False
  EndIf
EndProcedure
Procedure.i PopDValue(*Result.Double)
  If *Current\ArgumentQueue\Count
    *Result\D = ValueD(*Current\ArgumentQueue\First\Value)
    FreeFirstQueueItem(*Current\ArgumentQueue)
    ProcedureReturn #True
  Else
    ProcedureReturn #False
  EndIf
EndProcedure
Procedure.i PopBValue(*Result.Integer)
  If *Current\ArgumentQueue\Count
    *Result\I = ValueB(*Current\ArgumentQueue\First\Value)
    FreeFirstQueueItem(*Current\ArgumentQueue)
    ProcedureReturn #True
  Else
    ProcedureReturn #False
  EndIf
EndProcedure
Procedure.i PopSValue(*Result.String)
  If *Current\ArgumentQueue\Count
    *Result\S = ValueS(*Current\ArgumentQueue\First\Value)
    FreeFirstQueueItem(*Current\ArgumentQueue)
    ProcedureReturn #True
  Else
    ProcedureReturn #False
  EndIf
EndProcedure
Procedure.i PopBlockValue(*Result.Queue)
  Protected *Block.Queue
  
  If *Current\ArgumentQueue\Count
    *Block = ValueBlock(*Current\ArgumentQueue\First\Value) : _E()
    ClearQueue(*Result)
    CopyQueue(*Block, *Result, @CopyToken())
    FreeFirstQueueItem(*Current\ArgumentQueue)
    ProcedureReturn #True
  Else
    ProcedureReturn #False
  EndIf
EndProcedure
Procedure.i PopPValue(*Result.Integer)
  If *Current\ArgumentQueue\Count
    *Result\I = ValueP(*Current\ArgumentQueue\First\Value)
    FreeFirstQueueItem(*Current\ArgumentQueue)
    ProcedureReturn #True
  Else
    ProcedureReturn #False
  EndIf
EndProcedure
Procedure.i PopValueAddress(*Result.Integer)
  If *Current\ArgumentQueue\Count
    *Result\I = *Current\ArgumentQueue\First\Value
    FreeFirstQueueItem(*Current\ArgumentQueue)
    ProcedureReturn #True
  Else
    ProcedureReturn #False
  EndIf
EndProcedure

; Create argument queues:
Procedure.i Arguments1(*Value1.Value)
  Protected *ArgumentQueue.Queue
  
  *ArgumentQueue = CreateQueue(#Queue_SaveMemory)
  AppendQueueItem(*ArgumentQueue, *Value1)
  ProcedureReturn *ArgumentQueue
EndProcedure
Procedure.i Arguments2(*Value1.Value, *Value2.Value)
  Protected *ArgumentQueue.Queue
  
  *ArgumentQueue = CreateQueue(#Queue_SaveMemory)
  AppendQueueItem(*ArgumentQueue, *Value1)
  AppendQueueItem(*ArgumentQueue, *Value2)
  ProcedureReturn *ArgumentQueue
EndProcedure

; IDE Options = PureBasic 4.41 (Windows - x86)
; CursorPosition = 190
; FirstLine = 19
; Folding = AgBACDAAA9
; EnableXP
; CompileSourceDirectory